@@ -32,10 +32,14 @@ import java.nio.file.Path;
 import java.nio.file.Paths;
 import java.util.ArrayList;
 import java.util.Comparator;
+import java.util.EnumSet;
 import java.util.List;
 import java.util.Locale;
 import java.util.ServiceLoader;
+import java.util.Set;
+import java.util.stream.Collectors;
 
+import javax.tools.Diagnostic;
 import javax.tools.DiagnosticListener;
 import javax.tools.JavaCompiler;
 import javax.tools.JavaCompiler.CompilationTask;
@@ -70,15 +74,36 @@ public class BatchTestUtils {
 	public static String _tmpGenFolderName;
 	private static File _tmpGenDir;
 
+	public static final class DiagnosticReport<S> implements DiagnosticListener<S> {
+		public StringBuffer buffer;
+		private List<Diagnostic<? extends S>> diagnostics = new ArrayList<>();
+		DiagnosticReport() {
+			this.buffer = new StringBuffer();
+		}
+		public void report(Diagnostic<? extends S> diagnostic) {
+			diagnostics.add(diagnostic);
+			buffer.append(diagnostic.getMessage(Locale.getDefault()));
+			buffer.append("\n");
+		}
+		public List<Diagnostic<? extends S>> get(Diagnostic.Kind first, Diagnostic.Kind... rest) {
+			Set<Diagnostic.Kind> wanted = EnumSet.of(first, rest);
+			return diagnostics.stream().filter(d -> wanted.contains(d.getKind())).collect(Collectors.toList());
+		}
+		public String toString() {
+			return this.buffer.toString();
+		}
+	}
+
 	/**
 	 * Create a class that contains an annotation that generates another class,
 	 * and compile it.  Verify that generation and compilation succeeded.
 	 */
-	public static void compileOneClass(JavaCompiler compiler, List<String> options, File inputFile) {
-		compileOneClass(compiler, options, inputFile, false);
+	public static DiagnosticReport<JavaFileObject> compileOneClass(JavaCompiler compiler, List<String> options, File inputFile) {
+		return compileOneClass(compiler, options, inputFile, false);
 	}
-	public static void compileOneClass(JavaCompiler compiler, List<String> options, File inputFile, boolean useJLS8Processors) {
-		StandardJavaFileManager manager = compiler.getStandardFileManager(null, Locale.getDefault(), Charset.defaultCharset());
+	public static DiagnosticReport<JavaFileObject> compileOneClass(JavaCompiler compiler, List<String> options, File inputFile, boolean useJLS8Processors) {
+		DiagnosticReport<JavaFileObject> diagnostics = new DiagnosticReport<>();
+		StandardJavaFileManager manager = compiler.getStandardFileManager(diagnostics, Locale.getDefault(), Charset.defaultCharset());
 
 		// create new list containing inputfile
 		List<File> files = new ArrayList<File>();
@@ -94,7 +119,7 @@ public class BatchTestUtils {
 		addProcessorPaths(options, useJLS8Processors, true);
 		options.add("-XprintRounds");
 		options.add("-XprintProcessorInfo");
-		CompilationTask task = compiler.getTask(printWriter, manager, null, options, null, units);
+		CompilationTask task = compiler.getTask(printWriter, manager, diagnostics, options, null, units);
 		Boolean result = task.call();
 
 		if (!result.booleanValue()) {
@@ -107,6 +132,7 @@ public class BatchTestUtils {
 		} catch (IOException e) {
 			e.printStackTrace();
 		}
+		return diagnostics;
 	}
 
 	public static void compileTree(JavaCompiler compiler, List<String> options, File targetFolder) {
@@ -373,7 +399,7 @@ public class BatchTestUtils {
 		}
 	}
 	protected static void findClassesUnder(Path root, Path folder, List<String> classes, String moduleName) throws IOException {
-		if (folder == null) 
+		if (folder == null)
 			folder = root;
 		try (DirectoryStream<Path> stream = Files.newDirectoryStream(folder)) {
 	        for (Path entry : stream) {
