@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2017, 2018 IBM Corporation and others.
+ * Copyright (c) 2017, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -73,7 +73,7 @@ public class CompilerToolJava9Tests extends TestCase {
 	private static File _tmpBinDir;
 	public static String _tmpGenFolderName;
 	private static File _tmpGenDir;
-	
+
 	private static String modules_directory;
 	public CompilerToolJava9Tests(String name) {
 		super(name);
@@ -128,7 +128,7 @@ public class CompilerToolJava9Tests extends TestCase {
 		assertTrue("couldn't mkdirs " + _tmpSrcFolderName, _tmpSrcDir.exists());
 
 		modules_directory = getPluginDirectoryPath() + File.separator + "resources" + File.separator + "module_locations";
-		
+
 		Path moduleInfo = Paths.get(modules_directory, "source", "SimpleModules", "module.one", "module-info.java");
 		assertTrue("File should exist: " + moduleInfo, Files.isReadable(moduleInfo));
 		moduleInfo = Paths.get(modules_directory, "source", "SimpleModules", "module.two", "module-info.java");
@@ -164,7 +164,7 @@ public class CompilerToolJava9Tests extends TestCase {
 						sourceVersion.compareTo(SourceVersion.RELEASE_3) <= 0) {
 					continue;
 				}
-					
+
 				assertTrue("source version " + sourceVersion + " should be supported"
 						+ "by compiler " + compiler.getClass().getName(), sourceVersions.contains(sourceVersion));
 			}
@@ -173,7 +173,6 @@ public class CompilerToolJava9Tests extends TestCase {
 			assertTrue("source version 9 should be supported", sourceVersions.contains(obj));
 		}
 	}
-	// Incomplete tests - fails both with Javac and ECJ
 	public void testGetLocationForModule2() throws IOException {
 		if (this.isJREBelow9) return;
 		for(int i = 0; i < 2; i++) {
@@ -536,12 +535,179 @@ public class CompilerToolJava9Tests extends TestCase {
 				}
 				assertEquals("incorrect no of output locations", 1, count);
 				assertTrue("output location for module.two not found", found);
-				
+
 			} catch (UnsupportedOperationException ex) {
 				fail(cName + ":Should support getLocationForModule()");
 			}
 		}
 	}
+	// Test that non-module folders inside module-path are not considered 
+	// to be automatic modules.
+	public void testBug565748() throws IOException {
+		if (this.isJREBelow9) return;
+		JavaCompiler compiler = this.compilers[1];
+		String tmpFolder = _tmpFolder;
+		StandardJavaFileManager manager = compiler.getStandardFileManager(null, Locale.getDefault(), Charset.defaultCharset());
+
+		// create new list containing input file
+		List<File> files = new ArrayList<File>();
+		files.add(new File(modules_directory + File.separator + "source" + File.separator + "SimpleModules" +
+										File.separator + "module.one" + File.separator + "module-info.java"));
+		files.add(new File(modules_directory + File.separator + "source" + File.separator + "SimpleModules" +
+				File.separator + "module.two" + File.separator + "module-info.java"));
+		Iterable<? extends JavaFileObject> units = manager.getJavaFileObjectsFromFiles(files);
+		StringWriter stringWriter = new StringWriter();
+		PrintWriter printWriter = new PrintWriter(stringWriter);
+
+		List<String> options = new ArrayList<String>();
+		options.add("-d");
+		options.add(tmpFolder);
+		options.add("--module-source-path");
+		options.add(modules_directory + File.separator + "source" + File.separator + "SimpleModules");
+		ByteArrayOutputStream errBuffer = new ByteArrayOutputStream();
+		PrintWriter err = new PrintWriter(errBuffer);
+		CompilerInvocationDiagnosticListener listener = new CompilerInvocationDiagnosticListener(err) {
+			@Override
+			public void report(Diagnostic<? extends JavaFileObject> diagnostic) {
+				JavaFileObject source = diagnostic.getSource();
+				assertNotNull("No source", source);
+				super.report(diagnostic);
+			}
+		};
+ 		CompilationTask task = compiler.getTask(printWriter, manager, listener, options, null, units);
+ 		// check the classpath location
+		Boolean result = task.call();
+		printWriter.flush();
+		printWriter.close();
+ 		if (!result.booleanValue()) {
+ 			System.err.println("Compilation failed unexpectedly: " + stringWriter.getBuffer().toString());
+ 	 		assertTrue("Compilation failed ", false);
+ 		}
+ 		
+ 		// Try using the same outut as module-path and compile the new module (mod.test)
+		manager = compiler.getStandardFileManager(null, Locale.getDefault(), Charset.defaultCharset());
+
+		// create new list containing input file
+		files = new ArrayList<File>();
+		files.add(new File(modules_directory + File.separator + "bug565748" + File.separator + 
+									"mod.test" + File.separator + "module-info.java"));
+		units = manager.getJavaFileObjectsFromFiles(files);
+		stringWriter = new StringWriter();
+		printWriter = new PrintWriter(stringWriter);
+
+		options = new ArrayList<String>();
+		options.add("-d");
+		options.add(tmpFolder);
+		options.add("--module-path");
+		options.add(tmpFolder);
+		options.add("--module-source-path");
+		options.add(modules_directory + File.separator + "bug565748");
+		errBuffer = new ByteArrayOutputStream();
+		err = new PrintWriter(errBuffer);
+		listener = new CompilerInvocationDiagnosticListener(err) {
+			@Override
+			public void report(Diagnostic<? extends JavaFileObject> diagnostic) {
+				JavaFileObject source = diagnostic.getSource();
+				assertNotNull("No source", source);
+				super.report(diagnostic);
+			}
+		};
+ 		task = compiler.getTask(printWriter, manager, listener, options, null, units);
+		result = task.call();
+		printWriter.flush();
+		printWriter.close();
+ 		if (!result.booleanValue()) {
+ 			System.err.println("Compilation failed unexpectedly: " + stringWriter.getBuffer().toString());
+ 	 		assertTrue("Compilation failed ", false);
+ 		}
+ 		
+ 		// Delete the module-info.class from the previously compiled modules
+ 		// and try compiling the same module mod.test
+ 		File file = new File(tmpFolder + File.separator + "mod.test" + File.separator + "module-info.class");
+ 		file.delete();
+ 		file = new File(tmpFolder + File.separator + "mod.test");
+ 		file.delete();
+ 		file = new File(tmpFolder + File.separator + "module.one" + File.separator + "module-info.class");
+ 		file.delete();
+ 		file = new File(tmpFolder + File.separator + "module.two" + File.separator + "module-info.class");
+ 		file.delete();
+ 		manager = compiler.getStandardFileManager(null, Locale.getDefault(), Charset.defaultCharset());
+
+ 		// create new list containing input file
+ 		files = new ArrayList<File>();
+ 		files.add(new File(modules_directory + File.separator + "bug565748" + File.separator + 
+ 				"mod.test" + File.separator + "module-info.java"));
+ 		units = manager.getJavaFileObjectsFromFiles(files);
+ 		stringWriter = new StringWriter();
+ 		printWriter = new PrintWriter(stringWriter);
+
+ 		options = new ArrayList<String>();
+ 		options.add("-d");
+ 		options.add(tmpFolder);
+ 		options.add("--module-path");
+ 		options.add(tmpFolder);
+ 		options.add("--module-source-path");
+ 		options.add(modules_directory + File.separator + "bug565748");
+ 		errBuffer = new ByteArrayOutputStream();
+ 		err = new PrintWriter(errBuffer);
+ 		listener = new CompilerInvocationDiagnosticListener(err) {
+ 			@Override
+ 			public void report(Diagnostic<? extends JavaFileObject> diagnostic) {
+ 				JavaFileObject source = diagnostic.getSource();
+ 				assertNotNull("No source", source);
+ 				super.report(diagnostic);
+ 			}
+ 		};
+ 		task = compiler.getTask(printWriter, manager, listener, options, null, units);
+ 		result = task.call();
+ 		printWriter.flush();
+ 		printWriter.close();
+ 		if (result.booleanValue()) {
+ 			System.err.println("Compilation should fail: " + stringWriter.getBuffer().toString());
+ 			assertTrue("Compilation did not fail ", false);
+ 		}
+	}
+	public void testBug566749() throws IOException {
+		if (this.isJREBelow9) return;
+		JavaCompiler compiler = this.compilers[1];
+		String tmpFolder = _tmpFolder;
+		StandardJavaFileManager manager = compiler.getStandardFileManager(null, Locale.getDefault(), Charset.defaultCharset());
+
+		// create new list containing input file
+		List<File> files = new ArrayList<File>();
+		files.add(new File(modules_directory + File.separator + "bug566749" + File.separator + 
+ 				"mod.test" + File.separator + "module-info.java"));
+		
+		Iterable<? extends JavaFileObject> units = manager.getJavaFileObjectsFromFiles(files);
+		StringWriter stringWriter = new StringWriter();
+		PrintWriter printWriter = new PrintWriter(stringWriter);
+
+		List<String> options = new ArrayList<String>();
+		options = new ArrayList<String>();
+ 		options.add("-d");
+ 		options.add(tmpFolder);
+ 		options.add("--module-path");
+ 		options.add(tmpFolder);
+ 		options.add("--module-source-path");
+ 		options.add(modules_directory + File.separator + "bug566749");
+		ByteArrayOutputStream errBuffer = new ByteArrayOutputStream();
+		PrintWriter err = new PrintWriter(errBuffer);
+		CompilerInvocationDiagnosticListener listener = new CompilerInvocationDiagnosticListener(err) {
+			@Override
+			public void report(Diagnostic<? extends JavaFileObject> diagnostic) {
+				super.report(diagnostic);
+			}
+		};
+ 		CompilationTask task = compiler.getTask(printWriter, manager, listener, options, null, units);
+ 		// check the classpath location
+		Boolean result = task.call();
+		printWriter.flush();
+		printWriter.close();
+ 		if (result.booleanValue()) {
+ 			System.err.println("Compilation did not fail as expected: " + stringWriter.getBuffer().toString());
+ 	 		assertTrue("Compilation did not fail as expected ", false);
+ 		}
+	}
 	public void testGetJavaFileObjects() {
 		if (this.isJREBelow9) return;
 	}
@@ -566,17 +732,17 @@ public class CompilerToolJava9Tests extends TestCase {
 	public void testAsPath() {
 		if (this.isJREBelow9) return;
 	}
-	
+
 	/*-- Code for testing bug 533830 --*/
-	
+
 	public void testBug533830_1() throws IOException {
 		if (this.isJREBelow9) return;
-		
+
 		File src = createClassSource(
 			"package p;\n"
-		+	"public class X {}"	
+		+	"public class X {}"
 		);
-		
+
 		CompilerBuilder b = new CompilerBuilder()
 			.option("--release", "8")
 			.option("-source", "7")
@@ -588,25 +754,25 @@ public class CompilerToolJava9Tests extends TestCase {
 		} catch(IllegalArgumentException iae) {
 			assertEquals("option -source is not supported when --release is used", iae.getMessage());
 		}
-		
+
 		//-- We must now also have a diagnostic
 		assertTrue("The diagnostic listener did not receive an error for the illegal option", b.listener().hasDiagnostic("option -source is not supported when --release is used"));
 	}
-	
+
 	/**
 	 * Helps with building a compiler invocation, handling the common parts of testing.
 	 */
 	private final class CompilerBuilder {
 		private final JavaCompiler compiler;
-		
+
 		private final List<String> options = new ArrayList<>();
-		
+
 		private final List<File> files = new ArrayList<>();
-		
+
 		private final StringWriter errorWriter = new StringWriter();
-		
-		private DiagListener listener; 
-		
+
+		private DiagListener listener;
+
 		CompilerBuilder() {
 			compiler = compilers[1];
 		}
@@ -621,30 +787,29 @@ public class CompilerToolJava9Tests extends TestCase {
 			if(! options.contains("-d")) {
 				option("-d", tmpFolder);
 			}
-			
+
 			listener = new DiagListener(errWriter);
-			
+
 			ForwardingJavaFileManager<StandardJavaFileManager> forwardingManager = createFileManager(manager);
 			compiler.getTask(errWriter, forwardingManager, listener, options, null, units);
 		}
-		
+
 		public CompilerBuilder option(String... s) {
-			for(int i = 0; i < s.length; i++)
-				options.add(s[i]);
+			options.addAll(Arrays.asList(s));
 			return this;
 		}
-		
+
 		public CompilerBuilder file(File f) {
 			files.add(f);
 			return this;
 		}
-		
+
 		public DiagListener listener() {
 			if(null == listener)
 				throw new IllegalStateException("Call compile() before using the listener");
 			return listener;
 		}
-		
+
 		public ForwardingJavaFileManager<StandardJavaFileManager> createFileManager(StandardJavaFileManager manager) {
 			ForwardingJavaFileManager<StandardJavaFileManager> forwardingJavaFileManager = new ForwardingJavaFileManager<StandardJavaFileManager>(manager) {
 				@Override
@@ -682,20 +847,20 @@ public class CompilerToolJava9Tests extends TestCase {
 			return forwardingJavaFileManager;
 		}
 	}
-	
+
 	static private final class DiagListener extends CompilerInvocationDiagnosticListener {
 		private List<Diagnostic<? extends JavaFileObject>> errorList = new ArrayList<>();
 
 		DiagListener(PrintWriter err) {
 			super(err);
 		}
-		
+
 		@Override
 		public void report(Diagnostic<? extends JavaFileObject> diagnostic) {
 			errorList.add(diagnostic);
 			super.report(diagnostic);
 		}
-		
+
 		public boolean hasDiagnostic(String match) {
 			for(Diagnostic<? extends JavaFileObject> d: errorList) {
 				String msg = d.getMessage(Locale.US).toLowerCase();
@@ -705,7 +870,7 @@ public class CompilerToolJava9Tests extends TestCase {
 			return false;
 		}
 	}
-	
+
 	private File createClassSource(String source) throws IOException {
 		String tmpFolder = _tmpFolder;
 		File inputFile = new File(tmpFolder, "X.java");
@@ -714,15 +879,15 @@ public class CompilerToolJava9Tests extends TestCase {
 			return inputFile;
 		}
 	}
-	
+
 	/*-- end code for bug 533830 --*/
-	
+
 	/**
 	 * Recursively delete the contents of a directory, including any subdirectories.
 	 * This is not optimized to handle very large or deep directory trees efficiently.
 	 * @param f is either a normal file (which will be deleted) or a directory
 	 * (which will be emptied and then deleted).
-	 * @throws IOException 
+	 * @throws IOException
 	 */
 	public static void deleteTree(File f) throws IOException
 	{
