@@ -13,6 +13,7 @@
  *    IBM Corporation - fix for 342936
  *    Kenneth Olson - Contribution for bug 188796 - [jsr199] Using JSR199 to extend ECJ
  *    Dennis Hendriks - Contribution for bug 188796 - [jsr199] Using JSR199 to extend ECJ
+ *    Frits Jalvingh  - fix for bug 533830.
  *******************************************************************************/
 package org.eclipse.jdt.internal.compiler.tool;
 
@@ -93,6 +94,7 @@ public class EclipseCompilerImpl extends Main {
 				performCompilation();
 			}
 		} catch(IllegalArgumentException e) {
+			this.diagnosticListener.report(new ExceptionDiagnostic(e));
 			this.logger.logException(e);
 			if (this.systemExitWhenFinished) {
 				cleanup();
@@ -100,15 +102,16 @@ public class EclipseCompilerImpl extends Main {
 			}
 			return false;
 		} catch (RuntimeException e) { // internal compiler failure
-			e.printStackTrace();
+			this.diagnosticListener.report(new ExceptionDiagnostic(e));
 			this.logger.logException(e);
 			return false;
 		} finally {
 			cleanup();
 		}
-		if (this.globalErrorsCount == 0)
-			return true;
-		return false;
+		if (this.failOnWarning && this.globalWarningsCount > 0) {
+			return false;
+		}
+		return this.globalErrorsCount == 0;
 	}
 
 	private void cleanup() {
@@ -145,7 +148,7 @@ public class EclipseCompilerImpl extends Main {
 						throw new IllegalArgumentException(this.bind("unit.missing", name)); //$NON-NLS-1$
 					CompilationUnit cu = new CompilationUnit(null,
 							name,
-							null, 
+							null,
 							this.destinationPaths[i],
 							shouldIgnoreOptionalProblems(this.ignoreOptionalProblemsFromFolders, name.toCharArray()), this.modNames[i]) {
 
@@ -518,7 +521,7 @@ public class EclipseCompilerImpl extends Main {
 						Classpath classpath = FileSystem.getClasspath(
 								file.getAbsolutePath(),
 								null,
-								null, 
+								null,
 								this.options,
 								this.releaseVersion);
 							if (classpath != null) {
@@ -542,7 +545,7 @@ public class EclipseCompilerImpl extends Main {
 					system = FileSystem.getOlderSystemRelease(javaHome.getAbsolutePath(), versionFromJdkLevel, null);
 				} else {
 					system = FileSystem.getJrtClasspath(javaHome.toString(), null, null, null);
-				}						
+				}
 				Classpath classpath = new ClasspathJsr199(system, this.fileManager, StandardLocation.PLATFORM_CLASS_PATH);
 				fileSystemClasspaths.add(classpath);
 			} else {
@@ -564,7 +567,7 @@ public class EclipseCompilerImpl extends Main {
 					Classpath classpath = FileSystem.getClasspath(
 							file.getAbsolutePath(),
 							null,
-							null, 
+							null,
 							this.options,
 							this.releaseVersion);
 					if (classpath != null) {
@@ -578,7 +581,7 @@ public class EclipseCompilerImpl extends Main {
 					Classpath classpath = FileSystem.getClasspath(
 						file.getAbsolutePath(),
 						null,
-						null, 
+						null,
 						this.options,
 						this.releaseVersion);
 					if (classpath != null) {
@@ -602,15 +605,19 @@ public class EclipseCompilerImpl extends Main {
 							Collection<String> moduleNames = classpath.getModuleNames(null);
 							for (String modName : moduleNames) {
 								Path p = Paths.get(classpath.getPath());
-								standardJavaFileManager.setLocationForModule(StandardLocation.MODULE_SOURCE_PATH, modName, 
+								standardJavaFileManager.setLocationForModule(StandardLocation.MODULE_SOURCE_PATH, modName,
 										Collections.singletonList(p));
 								p = Paths.get(classpath.getDestinationPath());
 							}
 						}
 						fileSystemClasspaths.addAll(modulepaths);
 					}
+				} catch (IllegalStateException e) {
+					// Ignore this as JRE 9 throws IllegalStateException for getLocation returning null
+				} catch (IllegalArgumentException e) {
+					throw e;
 				} catch (Exception e) {
-					// TODO: Revisit when JRE 9 no longer throws IllegalStateException for getLocation.
+					this.logger.logException(e);
 				}
 				try {
 					locationFiles = standardJavaFileManager.getLocation(StandardLocation.MODULE_PATH);
@@ -622,7 +629,7 @@ public class EclipseCompilerImpl extends Main {
 									Collection<String> moduleNames = classpath.getModuleNames(null);
 									for (String string : moduleNames) {
 										Path path = Paths.get(classpath.getPath());
-										standardJavaFileManager.setLocationForModule(StandardLocation.MODULE_PATH, string, 
+										standardJavaFileManager.setLocationForModule(StandardLocation.MODULE_PATH, string,
 												Collections.singletonList(path));
 									}
 								}
@@ -632,8 +639,12 @@ public class EclipseCompilerImpl extends Main {
 							}
 						}
 					}
+				} catch (IllegalStateException e) {
+					// Ignore this as JRE 9 throws IllegalStateException for getLocation returning null
+				} catch (IllegalArgumentException e) {
+					throw e;
 				} catch (Exception e) {
-					// TODO: Revisit when JRE 9 no longer throws IllegalStateException for getLocation.
+					this.logger.logException(e);
 				}
 			}
 		} else if (javaFileManager != null) {
@@ -650,7 +661,7 @@ public class EclipseCompilerImpl extends Main {
 				if (this.fileManager.hasLocation(StandardLocation.SYSTEM_MODULES)) {
 					classpath = new ClasspathJsr199(this.fileManager, StandardLocation.SYSTEM_MODULES);
 					fileSystemClasspaths.add(classpath);
-				}				
+				}
 				if (this.fileManager.hasLocation(StandardLocation.PATCH_MODULE_PATH)) {
 					classpath = new ClasspathJsr199(this.fileManager, StandardLocation.PATCH_MODULE_PATH);
 					fileSystemClasspaths.add(classpath);
@@ -697,7 +708,7 @@ public class EclipseCompilerImpl extends Main {
 		for (@SuppressWarnings("rawtypes")
 			Iterator iterator = this.extraProblems.iterator(); iterator.hasNext(); ) {
 			final CategorizedProblem problem = (CategorizedProblem) iterator.next();
-			if (this.diagnosticListener != null) {
+			if (this.diagnosticListener != null && !isIgnored(problem)) {
 				Diagnostic<JavaFileObject> diagnostic = new Diagnostic<JavaFileObject>() {
 					@Override
 					public String getCode() {
@@ -750,7 +761,11 @@ public class EclipseCompilerImpl extends Main {
 					@Override
 					public JavaFileObject getSource() {
 						if (problem instanceof DefaultProblem) {
-							File f = new File(new String(((DefaultProblem) problem).getOriginatingFileName()));
+							char[] originatingName = ((DefaultProblem) problem).getOriginatingFileName();
+							if (originatingName == null) {
+								return null;
+							}
+							File f = new File(new String(originatingName));
 							if (f.exists()) {
 								Charset charset = (EclipseCompilerImpl.this.fileManager instanceof EclipseFileManager) ?
 														((EclipseFileManager) EclipseCompilerImpl.this.fileManager).charset : Charset.defaultCharset();
@@ -839,6 +854,11 @@ public class EclipseCompilerImpl extends Main {
 			return this.original.isWarning();
 		}
 
+		@Override
+		public boolean isInfo() {
+			return this.original.isInfo();
+		}
+
 		@Override
 		public void setSourceEnd(int sourceEnd) {
 			this.original.setSourceEnd(sourceEnd);
@@ -863,6 +883,6 @@ public class EclipseCompilerImpl extends Main {
 		public String getMarkerType() {
 			return this.original.getMarkerType();
 		}
-		
+
 	}
 }
