@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2006, 2017 IBM Corporation and others.
+ * Copyright (c) 2006, 2019 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -91,7 +91,7 @@ public class EclipseFileManager implements StandardJavaFileManager {
 	JrtFileSystem jrtSystem;
 	public ResourceBundle bundle;
 	private String releaseVersion;
-	
+
 	public EclipseFileManager(Locale locale, Charset charset) {
 		this.locale = locale == null ? Locale.getDefault() : locale;
 		this.charset = charset == null ? Charset.defaultCharset() : charset;
@@ -140,7 +140,7 @@ public class EclipseFileManager implements StandardJavaFileManager {
 		}
 		this.classloaders.clear();
 	}
-	
+
 	private void collectAllMatchingFiles(Location location, File file, String normalizedPackageName, Set<Kind> kinds, boolean recurse, ArrayList<JavaFileObject> collector) {
 		if (file.equals(this.jrtHome)) {
 			if (location instanceof ModuleLocationWrapper) {
@@ -416,7 +416,7 @@ public class EclipseFileManager implements StandardJavaFileManager {
 		}
 		return null;
 	}
-	
+
 	/* (non-Javadoc)
 	 * @see javax.tools.JavaFileManager#getFileForOutput(javax.tools.JavaFileManager.Location, java.lang.String, java.lang.String, javax.tools.FileObject)
 	 */
@@ -438,7 +438,7 @@ public class EclipseFileManager implements StandardJavaFileManager {
 			throw new IllegalArgumentException("location is empty : " + location);//$NON-NLS-1$
 		}
 	}
-	
+
 	/* (non-Javadoc)
 	 * @see javax.tools.JavaFileManager#getJavaFileForInput(javax.tools.JavaFileManager.Location, java.lang.String, javax.tools.JavaFileObject.Kind)
 	 */
@@ -644,11 +644,11 @@ public class EclipseFileManager implements StandardJavaFileManager {
 								setLocation(StandardLocation.PLATFORM_CLASS_PATH, bootclasspaths);
 							} else if ((this.flags & EclipseFileManager.HAS_ENDORSED_DIRS) != 0) {
 								// endorseddirs have been processed first
-								setLocation(StandardLocation.PLATFORM_CLASS_PATH, 
+								setLocation(StandardLocation.PLATFORM_CLASS_PATH,
 										concatFiles(iterable, bootclasspaths));
 							} else {
 								// extdirs have been processed first
-								setLocation(StandardLocation.PLATFORM_CLASS_PATH, 
+								setLocation(StandardLocation.PLATFORM_CLASS_PATH,
 										prependFiles(iterable, bootclasspaths));
 							}
 						}
@@ -747,7 +747,7 @@ public class EclipseFileManager implements StandardJavaFileManager {
 				case "--module-source-path": //$NON-NLS-1$
 					if (remaining.hasNext()) {
 						final Iterable<? extends File> sourcepaths = getPathsFrom(remaining.next());
-						if (sourcepaths != null && this.isOnJvm9) 
+						if (sourcepaths != null && this.isOnJvm9)
 							setLocation(StandardLocation.MODULE_SOURCE_PATH, sourcepaths);
 						return true;
 					} else {
@@ -759,7 +759,7 @@ public class EclipseFileManager implements StandardJavaFileManager {
 					}
 					if (remaining.hasNext()) {
 						Iterable<? extends File> iterable = getLocation(StandardLocation.PLATFORM_CLASS_PATH);
-						setLocation(StandardLocation.PLATFORM_CLASS_PATH, 
+						setLocation(StandardLocation.PLATFORM_CLASS_PATH,
 								concatFiles(iterable, getExtdirsFrom(remaining.next())));
 						this.flags |= EclipseFileManager.HAS_EXT_DIRS;
 						return true;
@@ -769,7 +769,7 @@ public class EclipseFileManager implements StandardJavaFileManager {
 				case "-endorseddirs": //$NON-NLS-1$
 					if (remaining.hasNext()) {
 						Iterable<? extends File> iterable = getLocation(StandardLocation.PLATFORM_CLASS_PATH);
-						setLocation(StandardLocation.PLATFORM_CLASS_PATH, 
+						setLocation(StandardLocation.PLATFORM_CLASS_PATH,
 								prependFiles(iterable, getEndorsedDirsFrom(remaining.next())));
 						this.flags |= EclipseFileManager.HAS_ENDORSED_DIRS;
 						return true;
@@ -890,12 +890,12 @@ public class EclipseFileManager implements StandardJavaFileManager {
 	}
 
 	private boolean isArchive(File f) {
-		if (isJrt(f)) 
+		if (isJrt(f))
 			return false;
 		String extension = getExtension(f);
 		return extension.equalsIgnoreCase(".jar") || extension.equalsIgnoreCase(".zip"); //$NON-NLS-1$ //$NON-NLS-2$
 	}
-	
+
 	private boolean isJrt(File f) {
 		return f.getName().toLowerCase().equals(JrtFileSystem.BOOT_MODULE);
 	}
@@ -929,7 +929,7 @@ public class EclipseFileManager implements StandardJavaFileManager {
 		if (allPaths == null) {
 			throw new IllegalArgumentException("Unknown location : " + location);//$NON-NLS-1$
 		}
-		
+
 		ArrayList<JavaFileObject> collector = new ArrayList<>();
 		String normalizedPackageName = normalized(packageName);
 		for (Path file : allPaths) {
@@ -959,8 +959,10 @@ public class EclipseFileManager implements StandardJavaFileManager {
 		for (Iterator<? extends File> iterator = iterable2.iterator(); iterator.hasNext(); ) {
 			list.add(iterator.next());
 		}
-		for (Iterator<? extends File> iterator = iterable.iterator(); iterator.hasNext(); ) {
-			list.add(iterator.next());
+		if (iterable != null) {
+			for (Iterator<? extends File> iterator = iterable.iterator(); iterator.hasNext(); ) {
+				list.add(iterator.next());
+			}
 		}
 		return list;
 	}
@@ -1034,7 +1036,7 @@ public class EclipseFileManager implements StandardJavaFileManager {
 		// '.*[.*'
 		final int bracketClosed = 11;
 		// '.*([.*])+'
-	
+
 		final int error = 99;
 		int state = start;
 		String token = null;
@@ -1242,7 +1244,7 @@ public class EclipseFileManager implements StandardJavaFileManager {
 				customEncoding,
 				isSourceOnly,
 				accessRuleSet,
-				destPath, 
+				destPath,
 				null,
 				this.releaseVersion);
 		if (currentClasspath != null) {
@@ -1359,22 +1361,22 @@ public class EclipseFileManager implements StandardJavaFileManager {
 	@Override
 	public void setLocationFromPaths(Location location, Collection<? extends Path> paths) throws IOException {
 		setLocation(location, getFiles(paths));
-		if (location == StandardLocation.MODULE_PATH || location == StandardLocation.MODULE_SOURCE_PATH) { 
+		if (location == StandardLocation.MODULE_PATH || location == StandardLocation.MODULE_SOURCE_PATH) {
 			// FIXME: same for module source path?
 			Map<String, String> options = new HashMap<>();
 			// FIXME: Find a way to get the options from the EclipseCompiler and pass it to the parser.
-			// FIXME: need to be the latest and not hardcoded value
-			options.put(CompilerOptions.OPTION_Compliance, CompilerOptions.VERSION_10);
-			options.put(CompilerOptions.OPTION_Source, CompilerOptions.VERSION_10);
-			options.put(CompilerOptions.OPTION_TargetPlatform, CompilerOptions.VERSION_10);
+			String latest = CompilerOptions.getLatestVersion();
+			options.put(CompilerOptions.OPTION_Compliance, latest);
+			options.put(CompilerOptions.OPTION_Source, latest);
+			options.put(CompilerOptions.OPTION_TargetPlatform, latest);
 			CompilerOptions compilerOptions = new CompilerOptions(options);
-			ProblemReporter problemReporter = 
+			ProblemReporter problemReporter =
 					new ProblemReporter(
 						DefaultErrorHandlingPolicies.proceedWithAllProblems(),
 						compilerOptions,
 						new DefaultProblemFactory());
 			for (Path path : paths) {
-				List<Classpath> mp = ModuleFinder.findModules(path.toFile(), null, 
+				List<Classpath> mp = ModuleFinder.findModules(path.toFile(), null,
 						new Parser(problemReporter, true), null, true, this.releaseVersion);
 				for (Classpath cp : mp) {
 					Collection<String> moduleNames = cp.getModuleNames(null);
@@ -1422,7 +1424,17 @@ public class EclipseFileManager implements StandardJavaFileManager {
 		Location result = this.locationHandler.getLocation(location, moduleName);
 		if (result == null && location == StandardLocation.CLASS_OUTPUT) {
 			LocationWrapper wrapper = this.locationHandler.getLocation(StandardLocation.MODULE_SOURCE_PATH, moduleName);
-			deriveOutputLocationForModules(moduleName, wrapper.paths);
+			// There are cases where we don't have module source path in that case we need to create
+			// classes in default location
+			if (wrapper == null) {
+				result = location;
+			} else {
+				deriveOutputLocationForModules(moduleName, wrapper.paths);
+				result = getLocationForModule(location, moduleName);
+			}
+		} else if (result == null && location == StandardLocation.SOURCE_OUTPUT) {
+			LocationWrapper wrapper = this.locationHandler.getLocation(StandardLocation.MODULE_SOURCE_PATH, moduleName);
+			deriveSourceOutputLocationForModules(moduleName, wrapper.paths);
 			result = getLocationForModule(location, moduleName);
 		}
 		return result;
@@ -1505,6 +1517,28 @@ public class EclipseFileManager implements StandardJavaFileManager {
 			}
 		}
 	}
+	private void deriveSourceOutputLocationForModules(String moduleName, Collection<? extends Path> paths) {
+		LocationWrapper wrapper = this.locationHandler.getLocation(StandardLocation.SOURCE_OUTPUT, moduleName);
+		if (wrapper == null) {
+			// First get from our internally known location for legacy/unnamed location
+			wrapper = this.locationHandler.getLocation(StandardLocation.SOURCE_OUTPUT, ""); //$NON-NLS-1$
+			if (wrapper == null) {
+				wrapper = this.locationHandler.getLocation(StandardLocation.SOURCE_OUTPUT);
+			}
+			if (wrapper != null) {
+				Iterator<? extends Path> iterator = wrapper.paths.iterator();
+				if (iterator.hasNext()) {
+					try {
+					// Per module output location is always a singleton list
+					Path path = iterator.next().resolve(moduleName);
+					this.locationHandler.setLocation(StandardLocation.SOURCE_OUTPUT, moduleName, Collections.singletonList(path));
+					} catch(Exception e) {
+						e.printStackTrace();
+					}
+				}
+			}
+		}
+	}
 	@Override
 	public void setLocationForModule(Location location, String moduleName, Collection<? extends Path> paths) throws IOException {
 		validateModuleLocation(location, moduleName);
