@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2010 IBM Corporation and others.
+ * Copyright (c) 2000, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -33,7 +33,9 @@ public final class SourceType {
 	private SourceMethod[] methods;
 	private int numberOfMethods;
 	private SourceField[] fields;
+	private SourceField[] recordComponents;
 	private int numberOfFields;
+	private int numberOfComponents;
 	private char[] source;
 	SourceType parent;
 	char[][] typeParameterNames;
@@ -79,6 +81,20 @@ protected void addField(SourceField sourceField) {
 	}
 	this.fields[this.numberOfFields++] = sourceField;
 }
+protected void addRecordComponent(SourceField comp) {
+	if (this.recordComponents == null) {
+		this.recordComponents = new SourceField[4];
+	}
+	if (this.numberOfComponents == this.recordComponents.length) {
+		System.arraycopy(
+			this.recordComponents,
+			0,
+			this.recordComponents = new SourceField[this.numberOfComponents * 2],
+			0,
+			this.numberOfComponents);
+	}
+	this.recordComponents[this.numberOfComponents++] = comp;
+}
 protected void addMemberType(SourceType sourceMemberType) {
 	if(this.memberTypes == null) {
 		this.memberTypes = new SourceType[4];
@@ -242,6 +258,7 @@ public String tabString(int tab) {
 		s = s + "\t";
 	return s;
 }
+@Override
 public String toString() {
 	return toString(0);
 }
