@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2019 IBM Corporation and others.
+ * Copyright (c) 2000, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -221,6 +221,7 @@ public static TestSuite getTestSuite(boolean addComplianceDiagnoseTest) {
 		tests_13.add(SelectionParserTest12.class);
 		tests_13.add(ModuleDeclarationSyntaxTest.class);
 		tests_13.add(JEP286ReservedWordTest.class);
+		//TODO:To add SwitchExpressionYieldTests here as well as master
 		// Reset forgotten subsets tests
 		TestCase.TESTS_PREFIX = null;
 		TestCase.TESTS_NAMES = null;
@@ -229,6 +230,51 @@ public static TestSuite getTestSuite(boolean addComplianceDiagnoseTest) {
 		TestCase.RUN_ONLY_ID = null;
 		all.addTest(AbstractCompilerTest.buildComplianceTestSuite(ClassFileConstants.getComplianceLevelForJavaVersion(ClassFileConstants.MAJOR_VERSION_13), tests_13));
 	}
+	if ((possibleComplianceLevels & AbstractCompilerTest.F_14) != 0) {
+		ArrayList tests_14 = (ArrayList)testClasses.clone();
+		tests_14.addAll(TEST_CLASSES_1_5);
+		tests_14.add(ParserTest1_7.class);
+		tests_14.add(LambdaExpressionSyntaxTest.class);
+		tests_14.add(ReferenceExpressionSyntaxTest.class);
+		tests_14.add(TypeAnnotationSyntaxTest.class);
+		tests_14.add(CompletionParserTest18.class);
+		tests_14.add(SelectionParserTest18.class);
+		tests_14.add(SelectionParserTest9.class);
+		tests_14.add(SelectionParserTest10.class);
+		tests_14.add(SelectionParserTest12.class);
+		tests_14.add(ModuleDeclarationSyntaxTest.class);
+		tests_14.add(JEP286ReservedWordTest.class);
+		// Reset forgotten subsets tests
+		TestCase.TESTS_PREFIX = null;
+		TestCase.TESTS_NAMES = null;
+		TestCase.TESTS_NUMBERS= null;
+		TestCase.TESTS_RANGE = null;
+		TestCase.RUN_ONLY_ID = null;
+		all.addTest(AbstractCompilerTest.buildComplianceTestSuite(ClassFileConstants.getComplianceLevelForJavaVersion(ClassFileConstants.MAJOR_VERSION_14), tests_14));
+	}
+	if ((possibleComplianceLevels & AbstractCompilerTest.F_15) != 0) {
+		ArrayList tests_15 = (ArrayList)testClasses.clone();
+		tests_15.addAll(TEST_CLASSES_1_5);
+		tests_15.add(ParserTest1_7.class);
+		tests_15.add(LambdaExpressionSyntaxTest.class);
+		tests_15.add(ReferenceExpressionSyntaxTest.class);
+		tests_15.add(TypeAnnotationSyntaxTest.class);
+		tests_15.add(CompletionParserTest18.class);
+		tests_15.add(SelectionParserTest18.class);
+		tests_15.add(SelectionParserTest9.class);
+		tests_15.add(SelectionParserTest10.class);
+		tests_15.add(SelectionParserTest12.class);
+		tests_15.add(ModuleDeclarationSyntaxTest.class);
+		tests_15.add(JEP286ReservedWordTest.class);
+		tests_15.add(PatternMatchingSelectionTest.class);
+		// Reset forgotten subsets tests
+		TestCase.TESTS_PREFIX = null;
+		TestCase.TESTS_NAMES = null;
+		TestCase.TESTS_NUMBERS= null;
+		TestCase.TESTS_RANGE = null;
+		TestCase.RUN_ONLY_ID = null;
+		all.addTest(AbstractCompilerTest.buildComplianceTestSuite(ClassFileConstants.getComplianceLevelForJavaVersion(ClassFileConstants.MAJOR_VERSION_15), tests_15));
+	}
 	return all;
 }
 public static Test suite() {
