@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2003, 2014 IBM Corporation and others.
+ * Copyright (c) 2003, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -367,125 +367,125 @@ public void test009() throws Exception {
 			ClassFileBytesDisassembler.DETAILED);
 
 	String expectedOutput =
-			"  // Method descriptor #15 ([Ljava/lang/String;)V\n" + 
-			"  // Stack: 3, Locals: 4\n" + 
-			"  public static void main(java.lang.String[] args);\n" + 
-			"     0  getstatic java.lang.System.out : java.io.PrintStream [16]\n" + 
-			"     3  ldc <String \"1\"> [22]\n" + 
-			"     5  invokevirtual java.io.PrintStream.print(java.lang.String) : void [24]\n" + 
-			"     8  aconst_null\n" + 
-			"     9  aconst_null\n" + 
-			"    10  if_acmpne 21\n" + 
-			"    13  getstatic java.lang.System.out : java.io.PrintStream [16]\n" + 
-			"    16  ldc <String \"2\"> [30]\n" + 
-			"    18  invokevirtual java.io.PrintStream.print(java.lang.String) : void [24]\n" + 
-			"    21  new java.lang.StringBuffer [32]\n" + 
-			"    24  dup\n" + 
-			"    25  ldc <String \"[\"> [34]\n" + 
-			"    27  invokespecial java.lang.StringBuffer(java.lang.String) [36]\n" + 
-			"    30  aconst_null\n" + 
-			"    31  invokevirtual java.lang.StringBuffer.append(java.lang.Object) : java.lang.StringBuffer [38]\n" + 
-			"    34  ldc <String \"]\"> [42]\n" + 
-			"    36  invokevirtual java.lang.StringBuffer.append(java.lang.String) : java.lang.StringBuffer [44]\n" + 
-			"    39  invokevirtual java.lang.StringBuffer.toString() : java.lang.String [47]\n" + 
-			"    42  ldc <String \"[null]\"> [51]\n" + 
-			"    44  if_acmpne 51\n" + 
-			"    47  iconst_1\n" + 
-			"    48  goto 52\n" + 
-			"    51  iconst_0\n" + 
-			"    52  istore_1 [b]\n" + 
-			"    53  getstatic java.lang.System.out : java.io.PrintStream [16]\n" + 
-			"    56  ldc <String \"3\"> [53]\n" + 
-			"    58  invokevirtual java.io.PrintStream.print(java.lang.String) : void [24]\n" + 
-			"    61  aconst_null\n" + 
-			"    62  astore_2 [s]\n" + 
-			"    63  aload_2 [s]\n" + 
-			"    64  ifnonnull 75\n" + 
-			"    67  getstatic java.lang.System.out : java.io.PrintStream [16]\n" + 
-			"    70  ldc <String \"4\"> [55]\n" + 
-			"    72  invokevirtual java.io.PrintStream.print(java.lang.String) : void [24]\n" + 
-			"    75  ldc <String \"aaa\"> [57]\n" + 
-			"    77  astore_3 [s2]\n" + 
-			"    78  getstatic java.lang.System.out : java.io.PrintStream [16]\n" + 
-			"    81  ldc <String \"5\"> [59]\n" + 
-			"    83  invokevirtual java.io.PrintStream.println(java.lang.String) : void [61]\n" + 
-			"    86  return\n" + 
-			"      Line numbers:\n" + 
-			"        [pc: 0, line: 3]\n" + 
-			"        [pc: 8, line: 4]\n" + 
-			"        [pc: 13, line: 5]\n" + 
-			"        [pc: 21, line: 6]\n" + 
-			"        [pc: 53, line: 7]\n" + 
-			"        [pc: 61, line: 8]\n" + 
-			"        [pc: 63, line: 9]\n" + 
-			"        [pc: 75, line: 10]\n" + 
-			"        [pc: 78, line: 11]\n" + 
-			"        [pc: 86, line: 12]\n" + 
-			"      Local variable table:\n" + 
-			"        [pc: 0, pc: 87] local: args index: 0 type: java.lang.String[]\n" + 
-			"        [pc: 53, pc: 87] local: b index: 1 type: boolean\n" + 
-			"        [pc: 63, pc: 87] local: s index: 2 type: java.lang.String\n" + 
+			"  // Method descriptor #15 ([Ljava/lang/String;)V\n" +
+			"  // Stack: 3, Locals: 4\n" +
+			"  public static void main(java.lang.String[] args);\n" +
+			"     0  getstatic java.lang.System.out : java.io.PrintStream [16]\n" +
+			"     3  ldc <String \"1\"> [22]\n" +
+			"     5  invokevirtual java.io.PrintStream.print(java.lang.String) : void [24]\n" +
+			"     8  aconst_null\n" +
+			"     9  aconst_null\n" +
+			"    10  if_acmpne 21\n" +
+			"    13  getstatic java.lang.System.out : java.io.PrintStream [16]\n" +
+			"    16  ldc <String \"2\"> [30]\n" +
+			"    18  invokevirtual java.io.PrintStream.print(java.lang.String) : void [24]\n" +
+			"    21  new java.lang.StringBuffer [32]\n" +
+			"    24  dup\n" +
+			"    25  ldc <String \"[\"> [34]\n" +
+			"    27  invokespecial java.lang.StringBuffer(java.lang.String) [36]\n" +
+			"    30  aconst_null\n" +
+			"    31  invokevirtual java.lang.StringBuffer.append(java.lang.Object) : java.lang.StringBuffer [38]\n" +
+			"    34  ldc <String \"]\"> [42]\n" +
+			"    36  invokevirtual java.lang.StringBuffer.append(java.lang.String) : java.lang.StringBuffer [44]\n" +
+			"    39  invokevirtual java.lang.StringBuffer.toString() : java.lang.String [47]\n" +
+			"    42  ldc <String \"[null]\"> [51]\n" +
+			"    44  if_acmpne 51\n" +
+			"    47  iconst_1\n" +
+			"    48  goto 52\n" +
+			"    51  iconst_0\n" +
+			"    52  istore_1 [b]\n" +
+			"    53  getstatic java.lang.System.out : java.io.PrintStream [16]\n" +
+			"    56  ldc <String \"3\"> [53]\n" +
+			"    58  invokevirtual java.io.PrintStream.print(java.lang.String) : void [24]\n" +
+			"    61  aconst_null\n" +
+			"    62  astore_2 [s]\n" +
+			"    63  aload_2 [s]\n" +
+			"    64  ifnonnull 75\n" +
+			"    67  getstatic java.lang.System.out : java.io.PrintStream [16]\n" +
+			"    70  ldc <String \"4\"> [55]\n" +
+			"    72  invokevirtual java.io.PrintStream.print(java.lang.String) : void [24]\n" +
+			"    75  ldc <String \"aaa\"> [57]\n" +
+			"    77  astore_3 [s2]\n" +
+			"    78  getstatic java.lang.System.out : java.io.PrintStream [16]\n" +
+			"    81  ldc <String \"5\"> [59]\n" +
+			"    83  invokevirtual java.io.PrintStream.println(java.lang.String) : void [61]\n" +
+			"    86  return\n" +
+			"      Line numbers:\n" +
+			"        [pc: 0, line: 3]\n" +
+			"        [pc: 8, line: 4]\n" +
+			"        [pc: 13, line: 5]\n" +
+			"        [pc: 21, line: 6]\n" +
+			"        [pc: 53, line: 7]\n" +
+			"        [pc: 61, line: 8]\n" +
+			"        [pc: 63, line: 9]\n" +
+			"        [pc: 75, line: 10]\n" +
+			"        [pc: 78, line: 11]\n" +
+			"        [pc: 86, line: 12]\n" +
+			"      Local variable table:\n" +
+			"        [pc: 0, pc: 87] local: args index: 0 type: java.lang.String[]\n" +
+			"        [pc: 53, pc: 87] local: b index: 1 type: boolean\n" +
+			"        [pc: 63, pc: 87] local: s index: 2 type: java.lang.String\n" +
 			"        [pc: 78, pc: 87] local: s2 index: 3 type: java.lang.String\n";
 
 	String expectedOutput15 =
-			"  // Method descriptor #15 ([Ljava/lang/String;)V\n" + 
-			"  // Stack: 3, Locals: 4\n" + 
-			"  public static void main(java.lang.String[] args);\n" + 
-			"     0  getstatic java.lang.System.out : java.io.PrintStream [16]\n" + 
-			"     3  ldc <String \"1\"> [22]\n" + 
-			"     5  invokevirtual java.io.PrintStream.print(java.lang.String) : void [24]\n" + 
-			"     8  aconst_null\n" + 
-			"     9  aconst_null\n" + 
-			"    10  if_acmpne 21\n" + 
-			"    13  getstatic java.lang.System.out : java.io.PrintStream [16]\n" + 
-			"    16  ldc <String \"2\"> [30]\n" + 
-			"    18  invokevirtual java.io.PrintStream.print(java.lang.String) : void [24]\n" + 
-			"    21  new java.lang.StringBuilder [32]\n" + 
-			"    24  dup\n" + 
-			"    25  ldc <String \"[\"> [34]\n" + 
-			"    27  invokespecial java.lang.StringBuilder(java.lang.String) [36]\n" + 
-			"    30  aconst_null\n" + 
-			"    31  invokevirtual java.lang.StringBuilder.append(java.lang.Object) : java.lang.StringBuilder [38]\n" + 
-			"    34  ldc <String \"]\"> [42]\n" + 
-			"    36  invokevirtual java.lang.StringBuilder.append(java.lang.String) : java.lang.StringBuilder [44]\n" + 
-			"    39  invokevirtual java.lang.StringBuilder.toString() : java.lang.String [47]\n" + 
-			"    42  ldc <String \"[null]\"> [51]\n" + 
-			"    44  if_acmpne 51\n" + 
-			"    47  iconst_1\n" + 
-			"    48  goto 52\n" + 
-			"    51  iconst_0\n" + 
-			"    52  istore_1 [b]\n" + 
-			"    53  getstatic java.lang.System.out : java.io.PrintStream [16]\n" + 
-			"    56  ldc <String \"3\"> [53]\n" + 
-			"    58  invokevirtual java.io.PrintStream.print(java.lang.String) : void [24]\n" + 
-			"    61  aconst_null\n" + 
-			"    62  astore_2 [s]\n" + 
-			"    63  aload_2 [s]\n" + 
-			"    64  ifnonnull 75\n" + 
-			"    67  getstatic java.lang.System.out : java.io.PrintStream [16]\n" + 
-			"    70  ldc <String \"4\"> [55]\n" + 
-			"    72  invokevirtual java.io.PrintStream.print(java.lang.String) : void [24]\n" + 
-			"    75  ldc <String \"aaa\"> [57]\n" + 
-			"    77  astore_3 [s2]\n" + 
-			"    78  getstatic java.lang.System.out : java.io.PrintStream [16]\n" + 
-			"    81  ldc <String \"5\"> [59]\n" + 
-			"    83  invokevirtual java.io.PrintStream.println(java.lang.String) : void [61]\n" + 
-			"    86  return\n" + 
-			"      Line numbers:\n" + 
-			"        [pc: 0, line: 3]\n" + 
-			"        [pc: 8, line: 4]\n" + 
-			"        [pc: 13, line: 5]\n" + 
-			"        [pc: 21, line: 6]\n" + 
-			"        [pc: 53, line: 7]\n" + 
-			"        [pc: 61, line: 8]\n" + 
-			"        [pc: 63, line: 9]\n" + 
-			"        [pc: 75, line: 10]\n" + 
-			"        [pc: 78, line: 11]\n" + 
-			"        [pc: 86, line: 12]\n" + 
-			"      Local variable table:\n" + 
-			"        [pc: 0, pc: 87] local: args index: 0 type: java.lang.String[]\n" + 
-			"        [pc: 53, pc: 87] local: b index: 1 type: boolean\n" + 
-			"        [pc: 63, pc: 87] local: s index: 2 type: java.lang.String\n" + 
+			"  // Method descriptor #15 ([Ljava/lang/String;)V\n" +
+			"  // Stack: 3, Locals: 4\n" +
+			"  public static void main(java.lang.String[] args);\n" +
+			"     0  getstatic java.lang.System.out : java.io.PrintStream [16]\n" +
+			"     3  ldc <String \"1\"> [22]\n" +
+			"     5  invokevirtual java.io.PrintStream.print(java.lang.String) : void [24]\n" +
+			"     8  aconst_null\n" +
+			"     9  aconst_null\n" +
+			"    10  if_acmpne 21\n" +
+			"    13  getstatic java.lang.System.out : java.io.PrintStream [16]\n" +
+			"    16  ldc <String \"2\"> [30]\n" +
+			"    18  invokevirtual java.io.PrintStream.print(java.lang.String) : void [24]\n" +
+			"    21  new java.lang.StringBuilder [32]\n" +
+			"    24  dup\n" +
+			"    25  ldc <String \"[\"> [34]\n" +
+			"    27  invokespecial java.lang.StringBuilder(java.lang.String) [36]\n" +
+			"    30  aconst_null\n" +
+			"    31  invokevirtual java.lang.StringBuilder.append(java.lang.Object) : java.lang.StringBuilder [38]\n" +
+			"    34  ldc <String \"]\"> [42]\n" +
+			"    36  invokevirtual java.lang.StringBuilder.append(java.lang.String) : java.lang.StringBuilder [44]\n" +
+			"    39  invokevirtual java.lang.StringBuilder.toString() : java.lang.String [47]\n" +
+			"    42  ldc <String \"[null]\"> [51]\n" +
+			"    44  if_acmpne 51\n" +
+			"    47  iconst_1\n" +
+			"    48  goto 52\n" +
+			"    51  iconst_0\n" +
+			"    52  istore_1 [b]\n" +
+			"    53  getstatic java.lang.System.out : java.io.PrintStream [16]\n" +
+			"    56  ldc <String \"3\"> [53]\n" +
+			"    58  invokevirtual java.io.PrintStream.print(java.lang.String) : void [24]\n" +
+			"    61  aconst_null\n" +
+			"    62  astore_2 [s]\n" +
+			"    63  aload_2 [s]\n" +
+			"    64  ifnonnull 75\n" +
+			"    67  getstatic java.lang.System.out : java.io.PrintStream [16]\n" +
+			"    70  ldc <String \"4\"> [55]\n" +
+			"    72  invokevirtual java.io.PrintStream.print(java.lang.String) : void [24]\n" +
+			"    75  ldc <String \"aaa\"> [57]\n" +
+			"    77  astore_3 [s2]\n" +
+			"    78  getstatic java.lang.System.out : java.io.PrintStream [16]\n" +
+			"    81  ldc <String \"5\"> [59]\n" +
+			"    83  invokevirtual java.io.PrintStream.println(java.lang.String) : void [61]\n" +
+			"    86  return\n" +
+			"      Line numbers:\n" +
+			"        [pc: 0, line: 3]\n" +
+			"        [pc: 8, line: 4]\n" +
+			"        [pc: 13, line: 5]\n" +
+			"        [pc: 21, line: 6]\n" +
+			"        [pc: 53, line: 7]\n" +
+			"        [pc: 61, line: 8]\n" +
+			"        [pc: 63, line: 9]\n" +
+			"        [pc: 75, line: 10]\n" +
+			"        [pc: 78, line: 11]\n" +
+			"        [pc: 86, line: 12]\n" +
+			"      Local variable table:\n" +
+			"        [pc: 0, pc: 87] local: args index: 0 type: java.lang.String[]\n" +
+			"        [pc: 53, pc: 87] local: b index: 1 type: boolean\n" +
+			"        [pc: 63, pc: 87] local: s index: 2 type: java.lang.String\n" +
 			"        [pc: 78, pc: 87] local: s2 index: 3 type: java.lang.String\n";
 
 	if (this.complianceLevel >= ClassFileConstants.JDK1_5) {
@@ -1171,7 +1171,7 @@ public void testAllConstants() {
 	verifyConstantEqualsAndHashcode(enumConstantSignature, enumConstantSignature2, enumConstantSignature3, byteConstant);
 	verifyConstantEqualsAndHashcode(enumConstantSignature, enumConstantSignature4, enumConstantSignature3, byteConstant);
 	assertNotNull(Constant.NotAConstant.toString());
-	
+
 	verifyValues(byteConstant, charConstant, booleanConstant, doubleConstant, floatConstant, intConstant, longConstant, shortConstant, stringConstant);
 	// check equals between to null string constants
 	assertTrue(stringConstant4.equals(stringConstant5));
@@ -1408,7 +1408,7 @@ private void verifyConstantEqualsAndHashcode(
 	assertFalse(o.equals(null));
 	assertFalse(o.hashCode() == o2.hashCode());
 	assertNotNull(o.toString());
-	
+
 	if (o instanceof Constant) {
 		assertTrue("Not the same values", ((Constant) o).hasSameValue((Constant) o3));
 		assertFalse("Have same values", ((Constant) o).hasSameValue((Constant) o2));
@@ -1449,27 +1449,27 @@ public void test021() {
 			"	}\n" +
 			"}",
 		},
-		"0\n" + 
-		"-9223372036854775808\n" + 
-		"-9223372036854775808\n" + 
-		"-9223372036854775808\n" + 
-		"-9223372036854775808\n" + 
-		"-9223372036854775808\n" + 
-		"-9223372036854775808\n" + 
-		"9223372036854775807\n" + 
-		"9223372036854775807\n" + 
-		"9223372036854775807\n" + 
-		"9223372036854775807\n" + 
-		"9223372036854775807\n" + 
-		"9223372036854775807\n" + 
-		"-1\n" + 
-		"-1\n" + 
-		"-1\n" + 
-		"-1\n" + 
-		"-1\n" + 
-		"-1\n" + 
-		"-1\n" + 
-		"8070450532247928831\n" + 
+		"0\n" +
+		"-9223372036854775808\n" +
+		"-9223372036854775808\n" +
+		"-9223372036854775808\n" +
+		"-9223372036854775808\n" +
+		"-9223372036854775808\n" +
+		"-9223372036854775808\n" +
+		"9223372036854775807\n" +
+		"9223372036854775807\n" +
+		"9223372036854775807\n" +
+		"9223372036854775807\n" +
+		"9223372036854775807\n" +
+		"9223372036854775807\n" +
+		"-1\n" +
+		"-1\n" +
+		"-1\n" +
+		"-1\n" +
+		"-1\n" +
+		"-1\n" +
+		"-1\n" +
+		"8070450532247928831\n" +
 		"8070450532247928831\n" +
 		"0\n" +
 		"0");
@@ -1492,25 +1492,118 @@ public void test022() {
 			"		System.out.println(0x0000000000000ffffffff); // OK\n" +
 			"		System.out.println(037777777777); // OK\n" +
 			"		System.out.println(-0x1); // OK\n" +
-			"		System.out.println(0xDADACAFE);\n" + 
+			"		System.out.println(0xDADACAFE);\n" +
 			"		System.out.println(0x0000000000000); // OK\n" +
 			"	}\n" +
 			"}",
 		},
-		"0\n" + 
-		"-2147483648\n" + 
-		"-2147483648\n" + 
-		"-2147483648\n" + 
-		"2147483647\n" + 
-		"2147483647\n" + 
-		"2147483647\n" + 
-		"-1\n" + 
-		"-1\n" + 
-		"-1\n" + 
+		"0\n" +
+		"-2147483648\n" +
+		"-2147483648\n" +
+		"-2147483648\n" +
+		"2147483647\n" +
+		"2147483647\n" +
+		"2147483647\n" +
+		"-1\n" +
+		"-1\n" +
+		"-1\n" +
 		"-1\n" +
 		"-623195394\n" +
 		"0");
 }
+public void testBug566332_01() {
+	if (this.complianceLevel < ClassFileConstants.JDK1_8) {
+		return;
+	}
+	this.runConformTest(
+			new String[] {
+				"X.java",
+				"public class X {\n"
+				+ "    public static void main(String[] args) {\n"
+				+ "        String switchVar = \"abc\";\n"
+				+ "        final String caseStr =  true ? \"abc\" : \"def\";\n"
+				+ "        switch (switchVar) {\n"
+				+ "            case caseStr: System.out.println(\"Pass\");\n"
+				+ "        }\n"
+				+ "    }\n"
+				+ "} ",
+			},
+			"Pass");
+}
+public void testBug566332_02() {
+	if (this.complianceLevel < ClassFileConstants.JDK1_8) {
+		return;
+	}
+	this.runConformTest(
+			new String[] {
+				"X.java",
+				"public class X {\n"
+				+ "    public static void main(String[] args) {\n"
+				+ "        String switchVar = \"abc\";\n"
+				+ "        final String caseStr =  false ? \"abc\" : \"def\";\n"
+				+ "        switch (switchVar) {\n"
+				+ "            case caseStr: System.out.println(\"Pass\");\n"
+				+ "        }\n"
+				+ "    }\n"
+				+ "} ",
+			},
+			"");
+}
+public void testBug566332_03() {
+	if (this.complianceLevel < ClassFileConstants.JDK1_8) {
+		return;
+	}
+	this.runNegativeTest(
+			new String[] {
+				"X.java",
+				"public class X {\n"
+				+ "    public static void main(String[] args) {\n"
+				+ "        boolean b = true;\n"
+				+ "        String switchVar = \"abc\";\n"
+				+ "        final String caseStr =  b ? \"abc\" : \"def\";\n"
+				+ "        switch (switchVar) {\n"
+				+ "            case caseStr: System.out.println(\"Pass\");\n"
+				+ "        }\n"
+				+ "    }\n"
+				+ "} ",
+			},
+			"----------\n" +
+			"1. ERROR in X.java (at line 7)\n" +
+			"	case caseStr: System.out.println(\"Pass\");\n" +
+			"	     ^^^^^^^\n" +
+			"case expressions must be constant expressions\n" +
+			"----------\n");
+}
+// Same as testBug566332_01(), but without the variable being final
+public void testBug566332_04() {
+	if (this.complianceLevel < ClassFileConstants.JDK1_8) {
+		return;
+	}
+	this.runNegativeTest(
+			new String[] {
+				"X.java",
+				"public class X {\n"
+				+ "    public static void main(String[] args) {\n"
+				+ "        String switchVar = \"abc\";\n"
+				+ "        String caseStr =  true ? \"abc\" : \"def\";\n"
+				+ "        switch (switchVar) {\n"
+				+ "            case caseStr: System.out.println(\"Pass\");\n"
+				+ "        }\n"
+				+ "    }\n"
+				+ "} ",
+			},
+			"----------\n" +
+			"1. WARNING in X.java (at line 4)\n" +
+			"	String caseStr =  true ? \"abc\" : \"def\";\n" +
+			"	                                 ^^^^^\n" +
+			"Dead code\n" +
+			"----------\n" +
+			"2. ERROR in X.java (at line 6)\n" +
+			"	case caseStr: System.out.println(\"Pass\");\n" +
+			"	     ^^^^^^^\n" +
+			"case expressions must be constant expressions\n" +
+			"----------\n");
+}
 public static Class testClass() {
 	return ConstantTest.class;
 }
