@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2010, 2019 IBM Corporation and others.
+ * Copyright (c) 2010, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -64,8 +64,8 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 	public StandAloneASTParserTest(String name) {
 		super(name);
 	}
-	
-	private static final int AST_JLS_LATEST = AST.JLS13;
+
+	private static final int AST_JLS_LATEST = AST.JLS15;
 
 	public ASTNode runConversion(
 			int astLevel,
@@ -94,7 +94,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 	}
 	public void testBug529654_001() {
 		String contents =
-				"module m {\n" + 
+				"module m {\n" +
 				"}";
 		ASTParser parser = ASTParser.newParser(AST_JLS_LATEST);
 		parser.setSource(contents.toCharArray());
@@ -117,13 +117,13 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 	}
 	public void test1() {
 		String contents =
-				"package p;\n" + 
-				"public class X {\n" + 
-				"	public int i;\n" + 
-				"	public static void main(String[] args) {\n" + 
-				"		int length = args.length;\n" + 
-				"		System.out.println(length);\n" + 
-				"	}\n" + 
+				"package p;\n" +
+				"public class X {\n" +
+				"	public int i;\n" +
+				"	public static void main(String[] args) {\n" +
+				"		int length = args.length;\n" +
+				"		System.out.println(length);\n" +
+				"	}\n" +
 				"}";
 		ASTNode node = runConversion(AST_JLS_LATEST, contents, true, true, true, "p/X.java");
 		assertTrue("Should be a compilation unit", node instanceof CompilationUnit);
@@ -149,7 +149,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 		MethodInvocation invocation = (MethodInvocation) expression;
 		Expression expression2 = invocation.getExpression();
 		assertNotNull("No binding", expression2.resolveTypeBinding());
-		
+
 		FieldDeclaration fieldDeclaration = (FieldDeclaration) typeDeclaration.bodyDeclarations().get(0);
 		VariableDeclarationFragment fragment2 = (VariableDeclarationFragment) fieldDeclaration.fragments().get(0);
 		IVariableBinding variableBinding2 = fragment2.resolveBinding();
@@ -198,15 +198,15 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 		final IBinding[] bindings = new IBinding[1];
 
 		String contents =
-			"package p;\n" + 
-			"public class X extends Y {\n" + 
-			"	public int i;\n" + 
-			"	public static void main(String[] args) {\n" + 
-			"		int length = args.length;\n" + 
-			"		System.out.println(length);\n" + 
-			"	}\n" + 
+			"package p;\n" +
+			"public class X extends Y {\n" +
+			"	public int i;\n" +
+			"	public static void main(String[] args) {\n" +
+			"		int length = args.length;\n" +
+			"		System.out.println(length);\n" +
+			"	}\n" +
 			"}";
-		
+
 		File packageDir = new File(rootDir, "p");
 		packageDir.mkdir();
 		File file = new File(packageDir, "X.java");
@@ -225,7 +225,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 		}
 
 		String contents2 =
-			"package p;\n" + 
+			"package p;\n" +
 			"public class Y {}";
 		File fileY = new File(packageDir, "Y.java");
 		Writer writer2 = null;
@@ -245,7 +245,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 		try {
 			final String canonicalPath = file.getCanonicalPath();
 			final CompilationUnit[] units = new CompilationUnit[1];
-	
+
 			FileASTRequestor requestor = new FileASTRequestor() {
 				public void acceptBinding(String bindingKey, IBinding binding) {
 					if (key.equals(bindingKey)) {
@@ -258,11 +258,11 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 					}
 				}
 			};
-	
+
 			parser.setEnvironment(null, new String[] { rootDir.getCanonicalPath() }, null, true);
-	
+
 			parser.createASTs(new String[] {canonicalPath}, null, new String[] {key}, requestor, null);
-	
+
 			assertNotNull("No binding", bindings[0]);
 			assertEquals("Wrong type of binding", IBinding.TYPE, bindings[0].getKind());
 			ITypeBinding typeBinding = (ITypeBinding) bindings[0];
@@ -309,15 +309,15 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 		final IBinding[] bindings = new IBinding[2];
 
 		String contents =
-			"package p;\n" + 
-			"public class X extends Y {\n" + 
-			"	public int i;\n" + 
-			"	public static void main(String[] args) {\n" + 
-			"		int length = args.length;\n" + 
-			"		System.out.println(length);\n" + 
-			"	}\n" + 
+			"package p;\n" +
+			"public class X extends Y {\n" +
+			"	public int i;\n" +
+			"	public static void main(String[] args) {\n" +
+			"		int length = args.length;\n" +
+			"		System.out.println(length);\n" +
+			"	}\n" +
 			"}";
-		
+
 		File packageDir = new File(rootDir, "p");
 		packageDir.mkdir();
 		File file = new File(packageDir, "X.java");
@@ -336,7 +336,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 		}
 
 		String contents2 =
-			"package p;\n" + 
+			"package p;\n" +
 			"public class Y {}";
 		File fileY = new File(packageDir, "Y.java");
 		Writer writer2 = null;
@@ -356,7 +356,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 		try {
 			final String canonicalPath = file.getCanonicalPath();
 			final CompilationUnit[] units = new CompilationUnit[1];
-	
+
 			FileASTRequestor requestor = new FileASTRequestor() {
 				public void acceptBinding(String bindingKey, IBinding binding) {
 					if (key.equals(bindingKey)) {
@@ -373,11 +373,11 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 					}
 				}
 			};
-	
+
 			parser.setEnvironment(null, new String[] { rootDir.getCanonicalPath() }, null, true);
-	
+
 			parser.createASTs(new String[] {canonicalPath}, null, new String[] {key}, requestor, null);
-	
+
 			assertNotNull("No binding", bindings[0]);
 			assertEquals("Wrong type of binding", IBinding.TYPE, bindings[0].getKind());
 			ITypeBinding typeBinding = (ITypeBinding) bindings[0];
@@ -396,7 +396,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 			fileY.delete();
 		}
 	}
-	
+
 	/**
 	 * @deprecated
 	 * @throws IOException
@@ -413,11 +413,11 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 		final String key = "Lp/C;";
 		final IBinding[] bindings = new IBinding[2];
 
-		String contents = 
-			"package p;\n" + 
-			"public class A{}\n" + 
+		String contents =
+			"package p;\n" +
+			"public class A{}\n" +
 			"class B{}";
-		
+
 		File packageDir = new File(rootDir, "p");
 		packageDir.mkdir();
 		File file = new File(packageDir, "A.java");
@@ -436,7 +436,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 		}
 
 		String contents2 =
-			"package p;\n" + 
+			"package p;\n" +
 			"public class C extends B {}";
 		File fileY = new File(packageDir, "C.java");
 		Writer writer2 = null;
@@ -456,7 +456,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 		try {
 			final String canonicalPath = fileY.getCanonicalPath();
 			final CompilationUnit[] units = new CompilationUnit[1];
-	
+
 			FileASTRequestor requestor = new FileASTRequestor() {
 				public void acceptBinding(String bindingKey, IBinding binding) {
 					if (key.equals(bindingKey)) {
@@ -473,7 +473,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 					}
 				}
 			};
-	
+
 			parser.setEnvironment(null, new String[] { rootDir.getCanonicalPath() }, null, true);
 			org.eclipse.jdt.internal.core.builder.AbstractImageBuilder.MAX_AT_ONCE = 0;
 			parser.createASTs(new String[] {canonicalPath}, null, new String[] {key}, requestor, null);
@@ -502,11 +502,11 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 		final String key = "Lp/C;";
 		final IBinding[] bindings = new IBinding[2];
 
-		String contents = 
-			"package p;\n" + 
-			"public class A{}\n" + 
+		String contents =
+			"package p;\n" +
+			"public class A{}\n" +
 			"class B{}";
-		
+
 		File packageDir = new File(rootDir, "p");
 		packageDir.mkdir();
 		File file = new File(packageDir, "A.java");
@@ -546,7 +546,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 		try {
 			final String canonicalPath = fileY.getCanonicalPath();
 			final CompilationUnit[] units = new CompilationUnit[1];
-	
+
 			FileASTRequestor requestor = new FileASTRequestor() {
 				public void acceptBinding(String bindingKey, IBinding binding) {
 					if (key.equals(bindingKey)) {
@@ -563,7 +563,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 					}
 				}
 			};
-	
+
 			parser.setEnvironment(null, new String[] { rootDir.getCanonicalPath() }, null, true);
 			parser.createASTs(new String[] {canonicalPath}, null, new String[] {key}, requestor, null);
 			assertNotNull("No ast", units[0]);
@@ -580,11 +580,11 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 		File rootDir = new File(System.getProperty("java.io.tmpdir"));
 
 		String contents =
-			"enum X {\n" + 
-			"              /** */\n" + 
-			"    FOO\n" + 
+			"enum X {\n" +
+			"              /** */\n" +
+			"    FOO\n" +
 			"}";
-		
+
 		File file = new File(rootDir, "X.java");
 		Writer writer = null;
 		try {
@@ -601,7 +601,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 		}
 
 		String contents2 =
-			"package p;\n" + 
+			"package p;\n" +
 			"class Y {}";
 		File packageDir = new File(rootDir, "p");
 		packageDir.mkdir();
@@ -706,7 +706,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 	    			   "  public static final class if {\n"+
                        "    public static final if ËŠ = new if(null, null, null, null);\n"+
                        "  }\n" +
-                        "}";	    		
+                        "}";
 	    ASTParser parser = ASTParser.newParser(AST.JLS9);
 	    parser.setSource(input.toCharArray());
 		parser.setResolveBindings(true);
@@ -714,7 +714,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 		parser.setBindingsRecovery(true);
 		parser.setKind(ASTParser.K_COMPILATION_UNIT);
 		parser.setEnvironment(null, new String[] {null}, null, true);
-		
+
 		Hashtable<String, String> options1 = JavaCore.getDefaultOptions();
 		options1.put(JavaCore.COMPILER_COMPLIANCE, JavaCore.VERSION_1_8);
 	    options1.put(JavaCore.COMPILER_SOURCE, JavaCore.VERSION_1_8);
@@ -725,7 +725,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 	@Deprecated
 	public void testBug526996_001() {
 		File rootDir = new File(System.getProperty("java.io.tmpdir"));
-		String contents = 
+		String contents =
 				"public class X {\n" +
 				"    public X() {\n" +
 				"        this.f16132b =\n" +
@@ -739,7 +739,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 				"            }\n" +
 				"        }\n" +
 				"\n";
-		
+
 		File file = new File(rootDir, "X.java");
 		Writer writer = null;
 		try {
@@ -818,7 +818,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 	}
 	public void testBug526996_002() {
 		File rootDir = new File(System.getProperty("java.io.tmpdir"));
-		String contents = 
+		String contents =
 						"public class zzei {\n"+
 						"    private final Context mContext;\n"+
 						"    private final String zzAg;\n"+
@@ -1221,7 +1221,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 						"        return com_google_android_gms_internal_zzei_zze;\n"+
 						"    }\n"+
 						"}\n";
-		
+
 		File file = new File(rootDir, "zzei.java");
 		Writer writer = null;
 		try {
@@ -1636,7 +1636,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 		parser.setBindingsRecovery(true);
 		parser.setKind(ASTParser.K_COMPILATION_UNIT);
 		parser.setEnvironment(null, new String[] {null}, null, true);
-		parser.setResolveBindings(true);	
+		parser.setResolveBindings(true);
 		Map<String, String> options = getCompilerOptions();
 		options.put(JavaCore.COMPILER_COMPLIANCE, JavaCore.VERSION_10);
 		options.put(JavaCore.COMPILER_SOURCE, JavaCore.VERSION_10);
@@ -1658,11 +1658,11 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 		File rootDir = new File(System.getProperty("java.io.tmpdir"));
 
 		String contents =
-			"enum X {\n" + 
-			"              /** */\n" + 
-			"    FOO\n" + 
+			"enum X {\n" +
+			"              /** */\n" +
+			"    FOO\n" +
 			"}";
-		
+
 		File file = new File(rootDir, "X.java");
 		Writer writer = null;
 		try {
@@ -1689,11 +1689,11 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 		String canonicalPath2 = fileY.getCanonicalPath();
 
 		contents =
-				"enum X {\n" + 
-				"              /** */\n" + 
-				"    FOO\n" + 
+				"enum X {\n" +
+				"              /** */\n" +
+				"    FOO\n" +
 				"}";
-			
+
 			File file2 = new File(rootDir, "X.java");
 			writer = null;
 			try {
@@ -1726,7 +1726,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 			fileY.delete();
 		}
 	}
-	
+
 	/*
 	 * To test isVar returning false for ast level 10 and compliance 9
 	 */
@@ -1734,7 +1734,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 		String contents =
 				"public class X {\n" +
 				"	public static void main(String[] args) {\n" +
-				"		var s = new Y();\n" + 
+				"		var s = new Y();\n" +
 				"	}\n" +
 				"}\n" +
 				"class Y {}";
@@ -1751,7 +1751,7 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 			options.put(JavaCore.COMPILER_SOURCE, JavaCore.VERSION_9);
 			options.put(JavaCore.COMPILER_CODEGEN_TARGET_PLATFORM, JavaCore.VERSION_9);
 			parser.setCompilerOptions(options);
-	
+
 			ASTNode node = parser.createAST(null);
 			assertTrue("Should be a compilation unit", node instanceof CompilationUnit);
 			CompilationUnit cu = (CompilationUnit) node;
@@ -1766,8 +1766,9 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 			SimpleName simpleName = (SimpleName) name;
 			assertFalse("A var", simpleName.isVar());
 	}
+	// no longer a preview feature, test is not relevant
 	@Deprecated
-	public void testBug545383_01() throws JavaModelException {
+	public void _testBug545383_01() throws JavaModelException {
 		String contents =
 				"class X {\n"+
 				"	public static int foo(int i) {\n"+
@@ -1790,14 +1791,14 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 		options.put(CompilerOptions.OPTION_EnablePreviews, CompilerOptions.ENABLED);
 		options.put(CompilerOptions.OPTION_ReportPreviewFeatures, CompilerOptions.IGNORE);
 		parser.setCompilerOptions(options);
-	
+
 		ASTNode node = parser.createAST(null);
 		assertTrue("Should be a compilation unit", node instanceof CompilationUnit);
 		CompilationUnit cu = (CompilationUnit) node;
 		IProblem[] problems = cu.getProblems();
 		assertTrue(problems.length > 0);
 		assertTrue(problems[0].toString().contains("preview"));
-	}	
+	}
 	public void testBug547900_01() throws JavaModelException {
 		String contents =
 				"class X {\n"+
@@ -1815,13 +1816,11 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 		parser.setEnvironment(null, null, null, true);
 		parser.setResolveBindings(false);
 		Map<String, String> options = getCompilerOptions();
-		options.put(JavaCore.COMPILER_COMPLIANCE, JavaCore.VERSION_13);
-		options.put(JavaCore.COMPILER_SOURCE, JavaCore.VERSION_13);
-		options.put(JavaCore.COMPILER_CODEGEN_TARGET_PLATFORM, JavaCore.VERSION_13);
-		options.put(CompilerOptions.OPTION_EnablePreviews, CompilerOptions.ENABLED);
-		options.put(CompilerOptions.OPTION_ReportPreviewFeatures, CompilerOptions.IGNORE);
+		options.put(JavaCore.COMPILER_COMPLIANCE, JavaCore.VERSION_14);
+		options.put(JavaCore.COMPILER_SOURCE, JavaCore.VERSION_14);
+		options.put(JavaCore.COMPILER_CODEGEN_TARGET_PLATFORM, JavaCore.VERSION_14);
 		parser.setCompilerOptions(options);
-	
+
 		ASTNode node = parser.createAST(null);
 		assertTrue("Should be a compilation unit", node instanceof CompilationUnit);
 		CompilationUnit cu = (CompilationUnit) node;
@@ -1845,20 +1844,20 @@ public class StandAloneASTParserTest extends AbstractRegressionTest {
 			String fileName2 = "C9947f.java";
 			f1 = createFile(
 					packDir, fileName1,
-					"package x;\n" + 
-					"\n" + 
-					"class EnsureImpl$1 {\n" + 
+					"package x;\n" +
+					"\n" +
+					"class EnsureImpl$1 {\n" +
 					"}\n");
 			f2 = createFile(
 					packDir, fileName2,
-					"package x;\n" + 
-					"public final class C9947f {\n" + 
-					"    public C9947f() {\n" + 
-					"        try {\n" + 
-					"            new x.EnsureImpl$1();\n" + 
-					"        } catch (Throwable unused) {\n" + 
-					"        }\n" + 
-					"    }\n" + 
+					"package x;\n" +
+					"public final class C9947f {\n" +
+					"    public C9947f() {\n" +
+					"        try {\n" +
+					"            new x.EnsureImpl$1();\n" +
+					"        } catch (Throwable unused) {\n" +
+					"        }\n" +
+					"    }\n" +
 					"}\n");
 			ASTParser parser = ASTParser.newParser(AST_JLS_LATEST);
 			parser.setResolveBindings(true);
