@@ -16,6 +16,7 @@ package org.eclipse.jdt.core.tests.eval;
 import java.io.File;
 import java.lang.reflect.Modifier;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Map;
 
 import junit.framework.Test;
@@ -434,6 +435,7 @@ public class EvaluationTest extends AbstractCompilerTest implements StopableTest
 		class ResultRequestor extends Requestor {
 			ArrayList collectedProblems = new ArrayList();
 			boolean gotDisplayString = false;
+			@Override
 			public void acceptResult(EvaluationResult result) {
 				assertEquals("Evaluation type", EvaluationResult.T_CODE_SNIPPET, result.getEvaluationType());
 				//assertEquals("Evaluation id", codeSnippet, result.getEvaluationID());
@@ -444,9 +446,7 @@ public class EvaluationTest extends AbstractCompilerTest implements StopableTest
 				} else {
 					assertTrue("Has problem", result.hasProblems());
 					CategorizedProblem[] problems = result.getProblems();
-					for (int i = 0; i < problems.length; i++) {
-						this.collectedProblems.add(problems[i]);
-					}
+					this.collectedProblems.addAll(Arrays.asList(problems));
 				}
 			}
 		}
@@ -475,6 +475,7 @@ public class EvaluationTest extends AbstractCompilerTest implements StopableTest
 		fail(formatted + "expected:<" + expectedString + "> but was:<" + actualString + ">");
 	}
 
+	@Override
 	public Map getCompilerOptions() {
 		Map defaultOptions = super.getCompilerOptions();
 		defaultOptions.put(CompilerOptions.OPTION_LocalVariableAttribute, CompilerOptions.DO_NOT_GENERATE);
@@ -496,6 +497,7 @@ public class EvaluationTest extends AbstractCompilerTest implements StopableTest
 		return new DefaultProblemFactory(java.util.Locale.getDefault());
 	}
 
+	@Override
 	public void initialize(CompilerTestSetup setUp) {
 		super.initialize(setUp);
 		EvaluationSetup evalSetUp = (EvaluationSetup)setUp;
@@ -517,6 +519,7 @@ public class EvaluationTest extends AbstractCompilerTest implements StopableTest
 	protected void installVariables(final int expectedNumber) {
 		class InstallRequestor extends Requestor {
 			int count = 0;
+			@Override
 			public void acceptResult(EvaluationResult result) {
 				assertTrue("Has problems", !result.hasProblems());
 				assertTrue("Has value", result.hasValue());
@@ -551,6 +554,7 @@ public class EvaluationTest extends AbstractCompilerTest implements StopableTest
 		this.env = new FileSystem(Util.concatWithClassLibs(EvaluationSetup.EVAL_DIRECTORY + File.separator + LocalVMLauncher.REGULAR_CLASSPATH_DIRECTORY, false), new String[0], encoding);
 	}
 
+	@Override
 	public void stop() {
 		if (this.target != null) {
 			this.target.disconnect(); // Close the socket first so that the OS resource has a chance to be freed.
