@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2014 IBM Corporation and others.
+ * Copyright (c) 2000, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -148,7 +148,7 @@ private String className(byte[] classDefinition) {
 Object createCodeSnippet(Class snippetClass) {
 	Object object = null;
 	try {
-		object = snippetClass.newInstance();
+		object = snippetClass.getDeclaredConstructor().newInstance();
 	} catch (InstantiationException e) {
 		e.printStackTrace();
 		this.ide.sendResult(void.class, null);
@@ -157,6 +157,22 @@ Object createCodeSnippet(Class snippetClass) {
 		e.printStackTrace();
 		this.ide.sendResult(void.class, null);
 		return null;
+	} catch (IllegalArgumentException e) {
+		e.printStackTrace();
+		this.ide.sendResult(void.class, null);
+		return null;
+	} catch (InvocationTargetException e) {
+		e.printStackTrace();
+		this.ide.sendResult(void.class, null);
+		return null;
+	} catch (NoSuchMethodException e) {
+		e.printStackTrace();
+		this.ide.sendResult(void.class, null);
+		return null;
+	} catch (SecurityException e) {
+		e.printStackTrace();
+		this.ide.sendResult(void.class, null);
+		return null;
 	}
 	return object;
 }
@@ -219,6 +235,7 @@ public static void main(String[] args) {
 		theRunner.start();
 	} else {
 		Thread server = new Thread() {
+			@Override
 			public void run() {
 				theRunner.start();
 			}
@@ -277,7 +294,7 @@ void processClasses(boolean mustRun, byte[][] classDefinitions) {
 	}
 
 	// load the classes and collect code snippet classes
-	Vector codeSnippetClasses = new Vector();
+	List<Class> codeSnippetClasses = new ArrayList<>();
 	for (int i = 0; i < newClasses.length; i++) {
 		String className = newClasses[i];
 		Class clazz = null;
@@ -310,7 +327,7 @@ void processClasses(boolean mustRun, byte[][] classDefinitions) {
 		} else if (superclass.equals(this.codeSnippetClass)) {
 			// It may be a code snippet class with no global variable
 			if (methods.length == 1 && methods[0].getName().equals(RUN_METHOD_NAME)) {
-				codeSnippetClasses.addElement(clazz);
+				codeSnippetClasses.add(clazz);
 			}
 			// Evaluate global variables and send result back
 			Field[] fields = clazz.getDeclaredFields();
@@ -328,14 +345,14 @@ void processClasses(boolean mustRun, byte[][] classDefinitions) {
 			}
 		} else if (this.codeSnippetClass.equals(superclass.getSuperclass()) && methods.length == 1 && methods[0].getName().equals("run")) {
 			// It is a code snippet class with a global variable superclass
-			codeSnippetClasses.addElement(clazz);
+			codeSnippetClasses.add(clazz);
 		}
 	}
 
 	// run the code snippet classes
 	if (codeSnippetClasses.size() != 0 && mustRun) {
-		for (Enumeration e = codeSnippetClasses.elements(); e.hasMoreElements();) {
-			Object codeSnippet = createCodeSnippet((Class) e.nextElement());
+		for (Class class1 : codeSnippetClasses) {
+			Object codeSnippet = createCodeSnippet(class1);
 			if (codeSnippet != null) {
 				runCodeSnippet(codeSnippet);
 			}
@@ -347,6 +364,7 @@ void processClasses(boolean mustRun, byte[][] classDefinitions) {
  */
 void runCodeSnippet(final Object snippet) {
 	Thread thread = new Thread() {
+		@Override
 		public void run() {
 			try {
 				try {
@@ -382,6 +400,7 @@ void runCodeSnippet(final Object snippet) {
  */
 public void start() {
 	Thread thread = new Thread("Code snippet runner") {
+		@Override
 		public void run() {
 			try {
 				CodeSnippetRunner.this.ide.connect();
