@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2019 IBM Corporation and others.
+ * Copyright (c) 2000, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -45,6 +45,8 @@ public class AbstractCompilerTest extends TestCase {
 	public static final int F_11  = 0x100;
 	public static final int F_12  = 0x200;
 	public static final int F_13  = 0x400;
+	public static final int F_14  = 0x800;
+	public static final int F_15  = 0x1000;
 
 	public static final boolean RUN_JAVAC = CompilerOptions.ENABLED.equals(System.getProperty("run.javac"));
 	private static final int UNINITIALIZED = -1;
@@ -53,10 +55,14 @@ public class AbstractCompilerTest extends TestCase {
 
 	protected long complianceLevel;
 	protected boolean enableAPT = false;
+	protected boolean enablePreview = false;
 	protected static boolean isJRE9Plus = false; // Stop gap, so tests need not be run at 9, but some tests can be adjusted for JRE 9
+	protected static boolean isJRE10Plus = false;
 	protected static boolean isJRE11Plus = false;
 	protected static boolean isJRE12Plus = false;
 	protected static boolean isJRE13Plus = false;
+	protected static boolean isJRE14Plus = false;
+	protected static boolean isJRE15Plus = false;
 	protected static boolean reflectNestedClassUseDollar;
 
 	/**
@@ -107,6 +113,12 @@ public class AbstractCompilerTest extends TestCase {
 		if ((complianceLevels & AbstractCompilerTest.F_13) != 0) {
 			suite.addTest(buildUniqueComplianceTestSuite(evaluationTestClass, ClassFileConstants.getComplianceLevelForJavaVersion(ClassFileConstants.MAJOR_VERSION_13)));
 		}
+		if ((complianceLevels & AbstractCompilerTest.F_14) != 0) {
+			suite.addTest(buildUniqueComplianceTestSuite(evaluationTestClass, ClassFileConstants.getComplianceLevelForJavaVersion(ClassFileConstants.MAJOR_VERSION_14)));
+		}
+		if ((complianceLevels & AbstractCompilerTest.F_15) != 0) {
+			suite.addTest(buildUniqueComplianceTestSuite(evaluationTestClass, ClassFileConstants.getComplianceLevelForJavaVersion(ClassFileConstants.MAJOR_VERSION_15)));
+		}
 	}
 
 	/**
@@ -155,16 +167,17 @@ public class AbstractCompilerTest extends TestCase {
 		if ((complianceLevels & AbstractCompilerTest.F_13) != 0) {
 			suite.addTest(buildComplianceTestSuite(testClasses, setupClass, ClassFileConstants.getComplianceLevelForJavaVersion(ClassFileConstants.MAJOR_VERSION_13)));
 		}
+		if ((complianceLevels & AbstractCompilerTest.F_14) != 0) {
+			suite.addTest(buildComplianceTestSuite(testClasses, setupClass, ClassFileConstants.getComplianceLevelForJavaVersion(ClassFileConstants.MAJOR_VERSION_14)));
+		}
+		if ((complianceLevels & AbstractCompilerTest.F_15) != 0) {
+			suite.addTest(buildComplianceTestSuite(testClasses, setupClass, ClassFileConstants.getComplianceLevelForJavaVersion(ClassFileConstants.MAJOR_VERSION_15)));
+		}
 		return suite;
 	}
 
 	 public static void setpossibleComplianceLevels(int complianceLevel) {
          possibleComplianceLevels = complianceLevel;
-         int lessthan9 = F_1_3 | F_1_4 | F_1_5 | F_1_6 | F_1_7 | F_1_8;
-         isJRE9Plus = !isJRELevel(lessthan9);
-         isJRE13Plus = isJRELevel(F_13);
-         isJRE12Plus = isJRE13Plus || isJRELevel(F_12);
-         isJRE11Plus = isJRE12Plus || isJRELevel(F_11);
 	 }
 
 	/**
@@ -196,7 +209,7 @@ public class AbstractCompilerTest extends TestCase {
 		TestSuite complianceSuite = null;
 		try {
 			Constructor constructor = setupClass.getConstructor(new Class[]{long.class});
-			complianceSuite = (TestSuite)constructor.newInstance(new Object[]{new Long(complianceLevel)});
+			complianceSuite = (TestSuite)constructor.newInstance(new Object[]{Long.valueOf(complianceLevel)});
 		} catch (IllegalAccessException e) {
 			e.printStackTrace();
 		} catch (InstantiationException e) {
@@ -323,8 +336,21 @@ public class AbstractCompilerTest extends TestCase {
 				suite.addTest(buildUniqueComplianceTestSuite(evaluationTestClass, ClassFileConstants.getComplianceLevelForJavaVersion(ClassFileConstants.MAJOR_VERSION_13)));
 			}
 		}
+		checkCompliance(evaluationTestClass, minimalCompliance, suite, complianceLevels, AbstractCompilerTest.F_14, ClassFileConstants.MAJOR_VERSION_14, 14);
+		checkCompliance(evaluationTestClass, minimalCompliance, suite, complianceLevels, AbstractCompilerTest.F_15, ClassFileConstants.MAJOR_VERSION_15, 15);
 		return suite;
 	}
+	private static void checkCompliance(Class evaluationTestClass, int minimalCompliance, TestSuite suite,
+			int complianceLevels, int abstractCompilerTestCompliance, int classFileConstantsVersion, int errMessageCompliance) {
+		int lev = complianceLevels & abstractCompilerTestCompliance;
+		if (lev != 0) {
+			if (lev < minimalCompliance) {
+				System.err.println("Cannot run "+evaluationTestClass.getName()+" at compliance" + errMessageCompliance + "!");
+			} else {
+				suite.addTest(buildUniqueComplianceTestSuite(evaluationTestClass, ClassFileConstants.getComplianceLevelForJavaVersion(classFileConstantsVersion)));
+			}
+		}
+	}
 
 	/**
 	 * Build a regression test setup suite for a compliance and a test suite to run.
@@ -362,7 +388,7 @@ public class AbstractCompilerTest extends TestCase {
 				} else {
 					complianceString = "unknown";
 				}
-				
+
 			}
 
 			System.err.println("Cannot run "+evaluationTestClass.getName()+" at compliance "+complianceString+"!");
@@ -381,6 +407,12 @@ public class AbstractCompilerTest extends TestCase {
 	 */
 	public static long highestComplianceLevels() {
 		int complianceLevels = AbstractCompilerTest.getPossibleComplianceLevels();
+		if ((complianceLevels & AbstractCompilerTest.F_15) != 0) {
+			return ClassFileConstants.getComplianceLevelForJavaVersion(ClassFileConstants.MAJOR_VERSION_15);
+		}
+		if ((complianceLevels & AbstractCompilerTest.F_14) != 0) {
+			return ClassFileConstants.getComplianceLevelForJavaVersion(ClassFileConstants.MAJOR_VERSION_14);
+		}
 		if ((complianceLevels & AbstractCompilerTest.F_13) != 0) {
 			return ClassFileConstants.getComplianceLevelForJavaVersion(ClassFileConstants.MAJOR_VERSION_13);
 		}
@@ -422,7 +454,7 @@ public class AbstractCompilerTest extends TestCase {
 			String version = System.getProperty("java.version");
 			if (version.startsWith("1.8.0_")) {
 				int build = Integer.parseInt(version.substring("1.8.0_".length()));
-				reflectNestedClassUseDollar = build >= 171; 
+				reflectNestedClassUseDollar = build >= 171;
 			} else if (version.startsWith("1.8.0-")) {
 				// Some versions start with 1.8.0- but don't have build qualifier.
 				// Just assume they are > 171 build. Nothing much can be done.
@@ -440,11 +472,13 @@ public class AbstractCompilerTest extends TestCase {
 	public static int getPossibleComplianceLevels() {
 		if (possibleComplianceLevels == UNINITIALIZED) {
 			String specVersion = System.getProperty("java.specification.version");
-			isJRE13Plus = CompilerOptions.VERSION_13.equals(specVersion);
+			isJRE15Plus =  CompilerOptions.VERSION_15.equals(specVersion);
+			isJRE14Plus = isJRE15Plus || CompilerOptions.VERSION_14.equals(specVersion);
+			isJRE13Plus = isJRE14Plus || CompilerOptions.VERSION_13.equals(specVersion);
 			isJRE12Plus = isJRE13Plus || CompilerOptions.VERSION_12.equals(specVersion);
 			isJRE11Plus = isJRE12Plus || CompilerOptions.VERSION_11.equals(specVersion);
-			isJRE9Plus = isJRE11Plus || CompilerOptions.VERSION_9.equals(specVersion)
-					||	CompilerOptions.VERSION_10.equals(specVersion);
+			isJRE10Plus = isJRE11Plus || CompilerOptions.VERSION_10.equals(specVersion);
+			isJRE9Plus = isJRE10Plus || CompilerOptions.VERSION_9.equals(specVersion);
 			initReflectionVersion();
 			String compliances = System.getProperty("compliance");
 			if (compliances != null) {
@@ -463,15 +497,26 @@ public class AbstractCompilerTest extends TestCase {
 					} else if (CompilerOptions.VERSION_1_8.equals(compliance)) {
 						possibleComplianceLevels |= F_1_8;
 					} else if (CompilerOptions.VERSION_9.equals(compliance)) {
-						possibleComplianceLevels |= F_9;
+						if (isJRE9Plus)
+							possibleComplianceLevels |= F_9;
 					} else if (CompilerOptions.VERSION_10.equals(compliance)) {
-						possibleComplianceLevels |= F_10;
+						if (isJRE10Plus)
+							possibleComplianceLevels |= F_10;
 					} else if (CompilerOptions.VERSION_11.equals(compliance)) {
-						possibleComplianceLevels |= F_11;
+						if (isJRE11Plus)
+							possibleComplianceLevels |= F_11;
 					} else if (CompilerOptions.VERSION_12.equals(compliance)) {
-						possibleComplianceLevels |= F_12;
+						if (isJRE12Plus)
+							possibleComplianceLevels |= F_12;
 					} else if (CompilerOptions.VERSION_13.equals(compliance)) {
-						possibleComplianceLevels |= F_13;
+						if (isJRE13Plus)
+							possibleComplianceLevels |= F_13;
+					} else if (CompilerOptions.VERSION_14.equals(compliance)) {
+						if (isJRE14Plus)
+							possibleComplianceLevels |= F_14;
+					} else if (CompilerOptions.VERSION_15.equals(compliance)) {
+						if (isJRE15Plus)
+							possibleComplianceLevels |= F_15;
 					} else {
 						System.out.println("Ignoring invalid compliance (" + compliance + ")");
 						System.out.print("Use one of ");
@@ -486,13 +531,15 @@ public class AbstractCompilerTest extends TestCase {
 						System.out.print(CompilerOptions.VERSION_10 + ", ");
 						System.out.print(CompilerOptions.VERSION_11 + ", ");
 						System.out.print(CompilerOptions.VERSION_12 + ", ");
-						System.out.println(CompilerOptions.VERSION_13);
+						System.out.print(CompilerOptions.VERSION_13 + ", ");
+						System.out.println(CompilerOptions.VERSION_14 + ", ");
+						System.out.println(CompilerOptions.VERSION_15);
 					}
 				}
 				if (possibleComplianceLevels == 0) {
 					System.out.println("Defaulting to all possible compliances");
 					possibleComplianceLevels = UNINITIALIZED;
-				}						
+				}
 			}
 			if (possibleComplianceLevels == UNINITIALIZED) {
 				if (!RUN_JAVAC) {
@@ -540,6 +587,14 @@ public class AbstractCompilerTest extends TestCase {
 					if (canRun13) {
 						possibleComplianceLevels |= F_13;
 					}
+					boolean canRun14 = canRun13 && !CompilerOptions.VERSION_13.equals(specVersion);
+					if (canRun14) {
+						possibleComplianceLevels |= F_14;
+					}
+					boolean canRun15 = canRun14 && !CompilerOptions.VERSION_14.equals(specVersion);
+					if (canRun15) {
+						possibleComplianceLevels |= F_15;
+					}
 				} else if ("1.0".equals(specVersion)
 							|| CompilerOptions.VERSION_1_1.equals(specVersion)
 							|| CompilerOptions.VERSION_1_2.equals(specVersion)
@@ -564,6 +619,12 @@ public class AbstractCompilerTest extends TestCase {
 												possibleComplianceLevels |= F_12;
 												if (!CompilerOptions.VERSION_12.equals(specVersion)) {
 													possibleComplianceLevels |= F_13;
+													if (!CompilerOptions.VERSION_13.equals(specVersion)) {
+														possibleComplianceLevels |= F_14;
+														if (!CompilerOptions.VERSION_14.equals(specVersion)) {
+															possibleComplianceLevels |= F_15;
+														}
+													}
 												}
 											}
 										}
@@ -721,7 +782,7 @@ public class AbstractCompilerTest extends TestCase {
 			options.put(CompilerOptions.OPTION_Source, CompilerOptions.VERSION_10);
 			options.put(CompilerOptions.OPTION_TargetPlatform, CompilerOptions.VERSION_10);
 		} else {
-			// This is already good enough to cover versions from future 
+			// This is already good enough to cover versions from future
 			// (as long as versionFromJdkLevel does its job)
 			String ver = CompilerOptions.versionFromJdkLevel(this.complianceLevel);
 			options.put(CompilerOptions.OPTION_Compliance, ver);
@@ -740,7 +801,7 @@ public class AbstractCompilerTest extends TestCase {
 	}
 
 	protected static String getVersionString(long compliance) {
-		String version = "version 13 : 57.0";
+		String version = "version 15 : 59.0";
 		if (compliance < ClassFileConstants.JDK9) return "version 1.8 : 52.0";
 		if (compliance == ClassFileConstants.JDK9) return "version 9 : 53.0";
 		if (compliance == ClassFileConstants.JDK10) return "version 10 : 54.0";
@@ -749,7 +810,7 @@ public class AbstractCompilerTest extends TestCase {
 			int major = Integer.parseInt(ver) + ClassFileConstants.MAJOR_VERSION_0;
 			return "version " + ver + " : " + major + ".0";
 		}
-		if (compliance >= ClassFileConstants.getComplianceLevelForJavaVersion(ClassFileConstants.MAJOR_VERSION_13)) return version; // keep this stmt for search for next bump up
+		if (compliance >= ClassFileConstants.getComplianceLevelForJavaVersion(ClassFileConstants.MAJOR_VERSION_15)) return version; // keep this stmt for search for next bump up
 		return version;
 	}
 
