@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2016 IBM Corporation and others.
+ * Copyright (c) 2000, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -13,7 +13,8 @@
  *******************************************************************************/
 package org.eclipse.jdt.core.tests.dom;
 
-import java.lang.reflect.*;
+import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
 import java.util.ArrayList;
 import java.util.Arrays;
 
@@ -34,6 +35,7 @@ public static Class[] getAllTestClasses() {
 		ASTConverterBugsTest.class,
 		ASTConverterBugsTestJLS3.class,
 		ASTConverterJavadocTest.class,
+		ASTConverterJavadocTest_15.class,
 		ASTConverter15Test.class,
 		ASTConverter16Test.class,
 		ASTConverter17Test.class,
@@ -52,7 +54,8 @@ public static Class[] getAllTestClasses() {
 		TypeAnnotationsConverterTest.class,
 		ASTConverter18Test.class,
 		ASTConverter9Test.class,
-		ASTConverter13Test.class,
+		ASTConverter14Test.class,
+		ASTConverter_15Test.class,
 	};
 }
 public static Test suite() {
