@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2017 IBM Corporation and others.
+ * Copyright (c) 2000, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -99,6 +99,7 @@ public class AttachedJavadocTests extends ModifyingResourceTests {
 	/**
 	 * Create project and set the jar placeholder.
 	 */
+	@Override
 	public void setUpSuite() throws Exception {
 		super.setUpSuite();
 
@@ -129,6 +130,7 @@ public class AttachedJavadocTests extends ModifyingResourceTests {
 	/**
 	 * Reset the jar placeholder and delete project.
 	 */
+	@Override
 	public void tearDownSuite() throws Exception {
 		this.deleteProject("AttachedJavadocProject"); //$NON-NLS-1$
 		this.root = null;
@@ -582,8 +584,12 @@ public class AttachedJavadocTests extends ModifyingResourceTests {
 			IType type = classFile.getType();
 			IField field = type.getField("f"); //$NON-NLS-1$
 			assertNotNull(field);
-			String javadoc = field.getAttachedJavadoc(new NullProgressMonitor()); //$NON-NLS-1$
-			assertNull("Should not have a javadoc", javadoc); //$NON-NLS-1$
+			try {
+				String javadoc = field.getAttachedJavadoc(new NullProgressMonitor()); //$NON-NLS-1$
+				assertNull("Should not have a javadoc", javadoc); //$NON-NLS-1$
+			} catch(JavaModelException e) {
+				// Ignore
+			}
 		} finally {
 			// restore classpath
 			if (savedEntries != null) {
@@ -595,7 +601,7 @@ public class AttachedJavadocTests extends ModifyingResourceTests {
 	 * @bug 304394: IJavaElement#getAttachedJavadoc(IProgressMonitor) should support referenced entries
 	 * Test that javadoc is picked up from the referenced classpath entry when the javadoc location is added
 	 * to that entry
-	 * 
+	 *
 	 * @see "https://bugs.eclipse.org/bugs/show_bug.cgi?id=304394"
 	 * @throws Exception
 	 */
@@ -614,7 +620,7 @@ public class AttachedJavadocTests extends ModifyingResourceTests {
 						&& "/AttachedJavadocProject/lib/test6.jar".equals(entry.getPath().toString())) { //$NON-NLS-1$
 
 					chainedJar = entries[i];
-					addLibrary(this.project, "/lib/chaining.jar", null, new String[0], 
+					addLibrary(this.project, "/lib/chaining.jar", null, new String[0],
 							new String[] {
 								"META-INF/MANIFEST.MF",
 								"Manifest-Version: 1.0\n" +
@@ -626,10 +632,10 @@ public class AttachedJavadocTests extends ModifyingResourceTests {
 					break;
 				}
 			}
-			
+
 			this.project.setRawClasspath(entries, new IClasspathEntry[]{chainedJar}, this.project.getOutputLocation(), null);
 			waitForAutoBuild();
-			
+
 			IPackageFragment packageFragment = this.root.getPackageFragment("p1.p2"); //$NON-NLS-1$
 			IOrdinaryClassFile classFile = packageFragment.getOrdinaryClassFile("X.class"); //$NON-NLS-1$
 			IType type = classFile.getType();
@@ -646,9 +652,9 @@ public class AttachedJavadocTests extends ModifyingResourceTests {
 	}
 	/**
 	 * Additional test for bug 304394.
-	 * Test that javadoc is picked up from the raw classpath entry when the referenced entry doesn't 
+	 * Test that javadoc is picked up from the raw classpath entry when the referenced entry doesn't
 	 * contain the javadoc location attrribute.
-	 * 
+	 *
 	 * @see "https://bugs.eclipse.org/bugs/show_bug.cgi?id=304394"
 	 * @throws Exception
 	 */
@@ -667,7 +673,7 @@ public class AttachedJavadocTests extends ModifyingResourceTests {
 						&& "/AttachedJavadocProject/lib/test6.jar".equals(entry.getPath().toString())) { //$NON-NLS-1$
 
 					chainedJar = entries[i];
-					addLibrary(this.project, "/lib/chaining.jar", null, new String[0], 
+					addLibrary(this.project, "/lib/chaining.jar", null, new String[0],
 							new String[] {
 								"META-INF/MANIFEST.MF",
 								"Manifest-Version: 1.0\n" +
@@ -679,11 +685,11 @@ public class AttachedJavadocTests extends ModifyingResourceTests {
 					break;
 				}
 			}
-			
+
 			chainedJar = JavaCore.newLibraryEntry(chainedJar.getPath(), null, null, null, null, chainedJar.isExported());
 			this.project.setRawClasspath(entries, new IClasspathEntry[]{chainedJar}, this.project.getOutputLocation(), null);
 			waitForAutoBuild();
-			
+
 			IPackageFragment packageFragment = this.root.getPackageFragment("p1.p2"); //$NON-NLS-1$
 			IOrdinaryClassFile classFile = packageFragment.getOrdinaryClassFile("X.class"); //$NON-NLS-1$
 			IType type = classFile.getType();
@@ -755,6 +761,7 @@ public class AttachedJavadocTests extends ModifyingResourceTests {
 			final PerProjectInfo projectInfo = JavaModelManager.getJavaModelManager().getPerProjectInfoCheckExistence(this.project.getProject());
 			final Object varThis = this;
 			Thread thread = new Thread(){
+				@Override
 				public void run() {
 					Object javadocContent = projectInfo.javadocCache.get(type);
 					while(javadocContent == null || javadocContent == BinaryType.EMPTY_JAVADOC) {
@@ -810,16 +817,16 @@ public class AttachedJavadocTests extends ModifyingResourceTests {
 
 			String javadoc = method.getAttachedJavadoc(null);
 			assertNotNull(javadoc);
-			assertEquals("Wrong contents", "<H3>\r\n" + 
-					"PublicAbstractClass.InnerFinalException</H3>\r\n" + 
-					"<PRE>\r\n" + 
-					"public <B>PublicAbstractClass.InnerFinalException</B>(java.lang.String&nbsp;property,\r\n" + 
-					"                                               java.lang.String&nbsp;msg)</PRE>\r\n" + 
-					"<DL>\r\n" + 
-					"<DD>javadoc for InnerFinalException(String property, String msg)\r\n" + 
-					"<P>\r\n" + 
-					"<DL>\r\n" + 
-					"<DT><B>Parameters:</B><DD><CODE>property</CODE> - the property argument<DD><CODE>msg</CODE> - the message argument</DL>\r\n" + 
+			assertEquals("Wrong contents", "<H3>\r\n" +
+					"PublicAbstractClass.InnerFinalException</H3>\r\n" +
+					"<PRE>\r\n" +
+					"public <B>PublicAbstractClass.InnerFinalException</B>(java.lang.String&nbsp;property,\r\n" +
+					"                                               java.lang.String&nbsp;msg)</PRE>\r\n" +
+					"<DL>\r\n" +
+					"<DD>javadoc for InnerFinalException(String property, String msg)\r\n" +
+					"<P>\r\n" +
+					"<DL>\r\n" +
+					"<DT><B>Parameters:</B><DD><CODE>property</CODE> - the property argument<DD><CODE>msg</CODE> - the message argument</DL>\r\n" +
 					"</DL>\r\n", javadoc);
 		} finally {
 			this.project.setRawClasspath(entries, null);
@@ -846,16 +853,16 @@ public class AttachedJavadocTests extends ModifyingResourceTests {
 
 			String javadoc = method.getAttachedJavadoc(null);
 			assertNotNull(javadoc);
-			assertEquals("Wrong contents", "<H3>\r\n" + 
-					"PublicAbstractClass.InnerFinalException</H3>\r\n" + 
-					"<PRE>\r\n" + 
-					"public <B>PublicAbstractClass.InnerFinalException</B>(java.lang.String&nbsp;property,\r\n" + 
-					"                                               java.lang.String&nbsp;msg)</PRE>\r\n" + 
-					"<DL>\r\n" + 
-					"<DD>javadoc for InnerFinalException(String property, String msg)\r\n" + 
-					"<P>\r\n" + 
-					"<DL>\r\n" + 
-					"<DT><B>Parameters:</B><DD><CODE>property</CODE> - the property argument<DD><CODE>msg</CODE> - the message argument</DL>\r\n" + 
+			assertEquals("Wrong contents", "<H3>\r\n" +
+					"PublicAbstractClass.InnerFinalException</H3>\r\n" +
+					"<PRE>\r\n" +
+					"public <B>PublicAbstractClass.InnerFinalException</B>(java.lang.String&nbsp;property,\r\n" +
+					"                                               java.lang.String&nbsp;msg)</PRE>\r\n" +
+					"<DL>\r\n" +
+					"<DD>javadoc for InnerFinalException(String property, String msg)\r\n" +
+					"<P>\r\n" +
+					"<DL>\r\n" +
+					"<DT><B>Parameters:</B><DD><CODE>property</CODE> - the property argument<DD><CODE>msg</CODE> - the message argument</DL>\r\n" +
 					"</DL>\r\n", javadoc);
 		} finally {
 			this.project.setRawClasspath(entries, null);
@@ -883,16 +890,16 @@ public class AttachedJavadocTests extends ModifyingResourceTests {
 			String javadoc = method.getAttachedJavadoc(null);
 			assertNotNull(javadoc);
 			assertEquals("Wrong contents",
-					"<H3>\r\n" + 
-					"PublicAbstractClass.A.InnerFinalException</H3>\r\n" + 
-					"<PRE>\r\n" + 
-					"public <B>PublicAbstractClass.A.InnerFinalException</B>(java.lang.String&nbsp;property,\r\n" + 
-					"                                                 java.lang.String&nbsp;msg)</PRE>\r\n" + 
-					"<DL>\r\n" + 
-					"<DD>javadoc for InnerFinalException(String property, String msg)\r\n" + 
-					"<P>\r\n" + 
-					"<DL>\r\n" + 
-					"<DT><B>Parameters:</B><DD><CODE>property</CODE> - the property argument<DD><CODE>msg</CODE> - the message argument</DL>\r\n" + 
+					"<H3>\r\n" +
+					"PublicAbstractClass.A.InnerFinalException</H3>\r\n" +
+					"<PRE>\r\n" +
+					"public <B>PublicAbstractClass.A.InnerFinalException</B>(java.lang.String&nbsp;property,\r\n" +
+					"                                                 java.lang.String&nbsp;msg)</PRE>\r\n" +
+					"<DL>\r\n" +
+					"<DD>javadoc for InnerFinalException(String property, String msg)\r\n" +
+					"<P>\r\n" +
+					"<DL>\r\n" +
+					"<DT><B>Parameters:</B><DD><CODE>property</CODE> - the property argument<DD><CODE>msg</CODE> - the message argument</DL>\r\n" +
 					"</DL>\r\n", javadoc);
 		} finally {
 			this.project.setRawClasspath(entries, null);
@@ -919,16 +926,16 @@ public class AttachedJavadocTests extends ModifyingResourceTests {
 
 			String javadoc = method.getAttachedJavadoc(null);
 			assertNotNull(javadoc);
-			assertEquals("Wrong contents", "<H3>\r\n" + 
-					"PublicAbstractClass.A.InnerFinalException</H3>\r\n" + 
-					"<PRE>\r\n" + 
-					"public <B>PublicAbstractClass.A.InnerFinalException</B>(java.lang.String&nbsp;property,\r\n" + 
-					"                                                 java.lang.String&nbsp;msg)</PRE>\r\n" + 
-					"<DL>\r\n" + 
-					"<DD>javadoc for InnerFinalException(String property, String msg)\r\n" + 
-					"<P>\r\n" + 
-					"<DL>\r\n" + 
-					"<DT><B>Parameters:</B><DD><CODE>property</CODE> - the property argument<DD><CODE>msg</CODE> - the message argument</DL>\r\n" + 
+			assertEquals("Wrong contents", "<H3>\r\n" +
+					"PublicAbstractClass.A.InnerFinalException</H3>\r\n" +
+					"<PRE>\r\n" +
+					"public <B>PublicAbstractClass.A.InnerFinalException</B>(java.lang.String&nbsp;property,\r\n" +
+					"                                                 java.lang.String&nbsp;msg)</PRE>\r\n" +
+					"<DL>\r\n" +
+					"<DD>javadoc for InnerFinalException(String property, String msg)\r\n" +
+					"<P>\r\n" +
+					"<DL>\r\n" +
+					"<DT><B>Parameters:</B><DD><CODE>property</CODE> - the property argument<DD><CODE>msg</CODE> - the message argument</DL>\r\n" +
 					"</DL>\r\n", javadoc);
 		} finally {
 			this.project.setRawClasspath(entries, null);
@@ -957,16 +964,16 @@ public class AttachedJavadocTests extends ModifyingResourceTests {
 			assertNotNull(javadoc);
 			assertEquals(
 					"Wrong contents",
-					"<H3>\r\n" + 
-					"foo</H3>\r\n" + 
-					"<PRE>\r\n" + 
-					"public void <B>foo</B>()</PRE>\r\n" + 
-					"<DL>\r\n" + 
-					"<DD>Test method\r\n" + 
-					"<P>\r\n" + 
-					"<DD><DL>\r\n" + 
-					"</DL>\r\n" + 
-					"</DD>\r\n" + 
+					"<H3>\r\n" +
+					"foo</H3>\r\n" +
+					"<PRE>\r\n" +
+					"public void <B>foo</B>()</PRE>\r\n" +
+					"<DL>\r\n" +
+					"<DD>Test method\r\n" +
+					"<P>\r\n" +
+					"<DD><DL>\r\n" +
+					"</DL>\r\n" +
+					"</DD>\r\n" +
 					"</DL>\r\n",
 					javadoc);
 		} finally {
@@ -996,16 +1003,16 @@ public class AttachedJavadocTests extends ModifyingResourceTests {
 			assertNotNull(javadoc);
 			assertEquals(
 					"Wrong contents",
-					"<H3>\r\n" + 
-					"PublicAbstractClass.InnerFinalException</H3>\r\n" + 
-					"<PRE>\r\n" + 
-					"public <B>PublicAbstractClass.InnerFinalException</B>()</PRE>\r\n" + 
-					"<DL>\r\n" + 
-					"<DD>javadoc for InnerFinalException()\r\n" + 
-					"<P>\r\n" + 
-					"</DL>\r\n" + 
-					"\r\n" + 
-					"<!-- ============ METHOD DETAIL ========== -->\r\n" + 
+					"<H3>\r\n" +
+					"PublicAbstractClass.InnerFinalException</H3>\r\n" +
+					"<PRE>\r\n" +
+					"public <B>PublicAbstractClass.InnerFinalException</B>()</PRE>\r\n" +
+					"<DL>\r\n" +
+					"<DD>javadoc for InnerFinalException()\r\n" +
+					"<P>\r\n" +
+					"</DL>\r\n" +
+					"\r\n" +
+					"<!-- ============ METHOD DETAIL ========== -->\r\n" +
 					"\r\n",
 					javadoc);
 		} finally {
@@ -1199,37 +1206,37 @@ public class AttachedJavadocTests extends ModifyingResourceTests {
 			} catch (Exception e) {
 				fail("Should not be an exception");
 			}
-			addLibrary(this.project, "valid.jar", null, 
+			addLibrary(this.project, "valid.jar", null,
 					new String[]{
 						"p/X.java",
 						"package p;\n" +
 						"/** Javadoc for class X */\n" +
-						"public class X {}"	}, 
+						"public class X {}"	},
 					JavaCore.VERSION_1_4);
-			addLibrary(this.project, "invalid.jar", null, 
+			addLibrary(this.project, "invalid.jar", null,
 					new String[]{
 						"q/Y.java",
 						"package q;\n" +
 						"/** Javadoc for class Y */\n" +
-						"public class Y {}"	}, 
+						"public class Y {}"	},
 					JavaCore.VERSION_1_4);
 
 			IClasspathAttribute attribute = JavaCore.newClasspathAttribute(IClasspathAttribute.JAVADOC_LOCATION_ATTRIBUTE_NAME, validUrl.toExternalForm());
-			IClasspathEntry validEntry = 
-					JavaCore.newLibraryEntry(new Path("/AttachedJavadocProject/valid.jar"), 
-							null, 
-							null, 
-							new IAccessRule[]{}, 
-							new IClasspathAttribute[] { attribute }, 
+			IClasspathEntry validEntry =
+					JavaCore.newLibraryEntry(new Path("/AttachedJavadocProject/valid.jar"),
+							null,
+							null,
+							new IAccessRule[]{},
+							new IClasspathAttribute[] { attribute },
 							false);
-			
+
 			attribute = JavaCore.newClasspathAttribute(IClasspathAttribute.JAVADOC_LOCATION_ATTRIBUTE_NAME, invalidUrl.toExternalForm());
-			IClasspathEntry invalidEntry = 
-					JavaCore.newLibraryEntry(new Path("/AttachedJavadocProject/invalid.jar"), 
-							null, 
-							null, 
-							new IAccessRule[]{}, 
-							new IClasspathAttribute[] { attribute }, 
+			IClasspathEntry invalidEntry =
+					JavaCore.newLibraryEntry(new Path("/AttachedJavadocProject/invalid.jar"),
+							null,
+							null,
+							new IAccessRule[]{},
+							new IClasspathAttribute[] { attribute },
 							false);
 
 			IClasspathEntry[] newClasspath = new IClasspathEntry[oldClasspath.length + 2];
@@ -1261,7 +1268,7 @@ public class AttachedJavadocTests extends ModifyingResourceTests {
 				fail("Should not throw JavaModelException");
 			}
 			assertNull("Should not have a javadoc", javadoc);
-			
+
 			packageFragment = invalidRoot.getPackageFragment("q");
 			classFile = packageFragment.getOrdinaryClassFile("Y.class");
 			type = classFile.getType();
@@ -1349,7 +1356,7 @@ public class AttachedJavadocTests extends ModifyingResourceTests {
 			IPackageFragment packageFragment = jarRoot.getPackageFragment("org.eclipse.pub");
 			javadoc = packageFragment.getAttachedJavadoc(new NullProgressMonitor());
 			assertNotNull("Should have a javadoc", javadoc); //$NON-NLS-1$
-			assertTrue("Should contain", javadoc.contains("Some package description"));			
+			assertTrue("Should contain", javadoc.contains("Some package description"));
 		} catch (IndexOutOfBoundsException e) {
 			assertTrue("Should not happen", false);
 		}
