@@ -50,6 +50,7 @@ public static Test suite() {
 	return buildModelTestSuite(ClassFileTests.class);
 }
 
+@Override
 public void setUpSuite() throws Exception {
 	super.setUpSuite();
 	IJavaProject javaProject = createJavaProject("P", new String[0], new String[] {"JCL15_LIB", "/P/lib"}, "", JavaCore.VERSION_1_5);
@@ -92,7 +93,7 @@ public void setUpSuite() throws Exception {
 		"package generic;\n" +
 		"import java.util.Collection;\n" +
 		"public class GenericField {\n" +
-		"	protected Collection<String> myField;\n" + 
+		"	protected Collection<String> myField;\n" +
 		"}",
 		"annotated/X.java",
 		"package annotated;\n" +
@@ -164,12 +165,12 @@ public void setUpSuite() throws Exception {
 		"annotated/Y.java",
 		"package annotated;\n" +
 		"import java.lang.annotation.*;\n" +
-		"import static java.lang.annotation.ElementType.*;\n" + 
-		"import static java.lang.annotation.RetentionPolicy.*;\n" + 
+		"import static java.lang.annotation.ElementType.*;\n" +
+		"import static java.lang.annotation.RetentionPolicy.*;\n" +
 		"@Deprecated\n" +
-		"@Documented\n" + 
+		"@Documented\n" +
 		"@Inherited\n" +
-		"@Retention(SOURCE)\n" + 
+		"@Retention(SOURCE)\n" +
 		"@Target({PACKAGE, TYPE, ANNOTATION_TYPE, METHOD, CONSTRUCTOR, FIELD, LOCAL_VARIABLE, PARAMETER})\n" +
 		"public @interface Y {\n" +
 		"}",
@@ -196,45 +197,47 @@ public void setUpSuite() throws Exception {
 		"annotated/MyAnnotation.java",
 		"package annotated;\n" +
 		"import java.lang.annotation.Retention;\n" +
-		"import java.lang.annotation.RetentionPolicy;\n" + 
-		"@Retention(value = RetentionPolicy.RUNTIME)\n" + 
+		"import java.lang.annotation.RetentionPolicy;\n" +
+		"@Retention(value = RetentionPolicy.RUNTIME)\n" +
 		"public @interface MyAnnotation {}",
 		"annotated/MyAnnotation2.java",
 		"package annotated;\n" +
 		"import java.lang.annotation.Retention;\n" +
-		"import java.lang.annotation.RetentionPolicy;\n" + 
-		"@Retention(value = RetentionPolicy.SOURCE)\n" + 
+		"import java.lang.annotation.RetentionPolicy;\n" +
+		"@Retention(value = RetentionPolicy.SOURCE)\n" +
 		"public @interface MyAnnotation2 {}",
 		"annotated/MyAnnotation3.java",
 		"package annotated;\n" +
 		"import java.lang.annotation.Retention;\n" +
-		"import java.lang.annotation.RetentionPolicy;\n" + 
-		"@Retention(value = RetentionPolicy.CLASS)\n" + 
+		"import java.lang.annotation.RetentionPolicy;\n" +
+		"@Retention(value = RetentionPolicy.CLASS)\n" +
 		"public @interface MyAnnotation3 {}",
 		"test342757/X.java",
 		"package test342757;\n" +
-		"public class X {\n" + 
-		"	class B {\n" + 
-		"		public B(@Deprecated @Annot String s) {}\n" + 
-		"		public void foo(@Deprecated @Annot int j) {}\n" + 
-		"	}\n" + 
+		"public class X {\n" +
+		"	class B {\n" +
+		"		public B(@Deprecated @Annot String s) {}\n" +
+		"		public void foo(@Deprecated @Annot int j) {}\n" +
+		"	}\n" +
 		"}",
 		"test342757/Annot.java",
 		"package test342757;\n" +
-		"import java.lang.annotation.Retention;\n" + 
-		"import static java.lang.annotation.RetentionPolicy.*;\n" + 
-		"@Retention(CLASS)\n" + 
+		"import java.lang.annotation.Retention;\n" +
+		"import static java.lang.annotation.RetentionPolicy.*;\n" +
+		"@Retention(CLASS)\n" +
 		"@interface Annot {}",
 	};
 	addLibrary(javaProject, "lib.jar", "libsrc.zip", pathAndContents, JavaCore.VERSION_1_5);
 	this.jarRoot = javaProject.getPackageFragmentRoot(getFile("/P/lib.jar"));
 }
 
+@Override
 public void tearDownSuite() throws Exception {
 	super.tearDownSuite();
 	deleteProject("P");
 }
 
+@Override
 protected void tearDown() throws Exception {
 	if (this.workingCopy != null)
 		this.workingCopy.discardWorkingCopy();
@@ -434,10 +437,10 @@ public void testAnnotations16() throws JavaModelException {
 public void testAnnotations17() throws JavaModelException {
 	IType type = this.jarRoot.getPackageFragment("annotated").getOrdinaryClassFile("Y.class").getType();
 	assertAnnotationsEqual(
-		"@java.lang.annotation.Target({java.lang.annotation.ElementType.TYPE, java.lang.annotation.ElementType.FIELD, java.lang.annotation.ElementType.METHOD, java.lang.annotation.ElementType.PARAMETER, java.lang.annotation.ElementType.CONSTRUCTOR, java.lang.annotation.ElementType.LOCAL_VARIABLE, java.lang.annotation.ElementType.ANNOTATION_TYPE, java.lang.annotation.ElementType.PACKAGE})\n" + 
-		"@java.lang.annotation.Retention(java.lang.annotation.RetentionPolicy.SOURCE)\n" + 
-		"@java.lang.Deprecated\n" + 
-		"@java.lang.annotation.Documented\n" + 
+		"@java.lang.annotation.Target({java.lang.annotation.ElementType.TYPE, java.lang.annotation.ElementType.FIELD, java.lang.annotation.ElementType.METHOD, java.lang.annotation.ElementType.PARAMETER, java.lang.annotation.ElementType.CONSTRUCTOR, java.lang.annotation.ElementType.LOCAL_VARIABLE, java.lang.annotation.ElementType.ANNOTATION_TYPE, java.lang.annotation.ElementType.PACKAGE})\n" +
+		"@java.lang.annotation.Retention(java.lang.annotation.RetentionPolicy.SOURCE)\n" +
+		"@java.lang.Deprecated\n" +
+		"@java.lang.annotation.Documented\n" +
 		"@java.lang.annotation.Inherited\n",
 		type.getAnnotations());
 }
@@ -539,9 +542,9 @@ public void testAnnotations26() throws JavaModelException {
 	IType type = this.jarRoot.getPackageFragment("test342757").getOrdinaryClassFile("X$B.class").getType();
 	IMethod[] methods = type.getMethods();
 	String expected =
-			"@test342757.Annot\n" + 
-			"@java.lang.Deprecated\n" + 
-			"@test342757.Annot\n" + 
+			"@test342757.Annot\n" +
+			"@java.lang.Deprecated\n" +
+			"@test342757.Annot\n" +
 			"@java.lang.Deprecated\n";
 	StringBuffer buffer = new StringBuffer();
 	for (int i = 0, max = methods.length; i < max; i++) {
@@ -1579,7 +1582,7 @@ public void testGenericFieldGetTypeSignature() throws JavaModelException {
 		assertNotNull(typeParam);
 		assertStringsEqual("Type parameter bounds signatures",
 				"Ljava.lang.Object;\n" +
-				"Lgeneric.I<-TT;>;\n", 
+				"Lgeneric.I<-TT;>;\n",
 				typeParam.getBoundsSignatures());
 	}
 
@@ -1588,7 +1591,7 @@ public void testGenericFieldGetTypeSignature() throws JavaModelException {
 				"X.class").getType();
 		IMethod method = type.getMethod("foo", new String[] { "TK;", "TV;" });
 		ITypeParameter typeParam = method.getTypeParameter("V");
-		assertStringsEqual("Type parameter bounds signatures", 
+		assertStringsEqual("Type parameter bounds signatures",
 							"TT;\n", typeParam.getBoundsSignatures());
 	}
 	//https://bugs.eclipse.org/bugs/show_bug.cgi?id=316937
@@ -1646,12 +1649,13 @@ public void testGenericFieldGetTypeSignature() throws JavaModelException {
 	 * Ensures that if more than one thread try to open a class file at the same time, the children are correct.
 	 */
 	public void testBug372687() throws CoreException {
-		String expected = "X.class\n" + 
-						  "  class X\n" + 
-						  "    X()\n" + 
+		String expected = "X.class\n" +
+						  "  class X\n" +
+						  "    X()\n" +
 						  "    void foo()";
 		class GetClassThread extends Thread {
 			public String childString;
+			@Override
 			public void run(){
 				IOrdinaryClassFile clazz = ClassFileTests.this.jarRoot.getPackageFragment("workingcopy").getOrdinaryClassFile("X.class");
 				try {
