@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
+ * Copyright (c) 2000, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -13,7 +13,8 @@
  *******************************************************************************/
 package org.eclipse.jdt.core.tests.model;
 
-import java.util.Vector;
+import java.util.ArrayList;
+import java.util.List;
 
 import org.eclipse.jdt.core.CompletionProposal;
 import org.eclipse.jdt.core.CompletionRequestor;
@@ -24,28 +25,27 @@ import org.eclipse.jdt.core.Signature;
  * {@link CompletionTestsRequestor} is deprecated. {@link CompletionTestsRequestor2} must be used instead.
  */
 //TODO all instances of CompletionTestsRequestor should be replaced by an instance of CompletionTestsRequestor2
-@SuppressWarnings({"rawtypes", "unchecked"})
 public class CompletionTestsRequestor extends CompletionRequestor {
-	private Vector elements = new Vector();
-	private Vector completions = new Vector();
-	private Vector relevances = new Vector();
-	private Vector completionStart = new Vector();
-	private Vector completionEnd = new Vector();
+	private List<String> elements = new ArrayList<>();
+	private List<String> completions = new ArrayList<>();
+	private List<String> relevances = new ArrayList<>();
+	private List<String> completionStart = new ArrayList<>();
+	private List<String> completionEnd = new ArrayList<>();
 
 	public boolean debug = false;
 
 	private void acceptCommon(CompletionProposal proposal) {
-		this.completions.addElement(new String(proposal.getCompletion()));
-		this.relevances.addElement(String.valueOf(proposal.getRelevance()));
-		this.completionStart.addElement(String.valueOf(proposal.getReplaceStart()));
-		this.completionEnd.addElement(String.valueOf(proposal.getReplaceEnd()));
+		this.completions.add(new String(proposal.getCompletion()));
+		this.relevances.add(String.valueOf(proposal.getRelevance()));
+		this.completionStart.add(String.valueOf(proposal.getReplaceStart()));
+		this.completionEnd.add(String.valueOf(proposal.getReplaceEnd()));
 	}
 	public void accept(CompletionProposal proposal) {
 		char[] typeName = null;
 		switch(proposal.getKind()) {
 			case CompletionProposal.ANONYMOUS_CLASS_DECLARATION :
 				typeName = Signature.getSignatureSimpleName(proposal.getDeclarationSignature());
-				this.elements.addElement(new String(typeName));
+				this.elements.add(new String(typeName));
 				acceptCommon(proposal);
 				if (this.debug)
 					System.out.println("anonymous type " + new String(typeName));
@@ -56,13 +56,13 @@ public class CompletionTestsRequestor extends CompletionRequestor {
 
 				} else if((proposal.getFlags() & Flags.AccInterface) != 0) {
 					typeName = Signature.getSignatureSimpleName(proposal.getSignature());
-					this.elements.addElement(new String(typeName));
+					this.elements.add(new String(typeName));
 					acceptCommon(proposal);
 					if (this.debug)
 						System.out.println("Interface " + new String(typeName));
 				} else {
 					typeName = Signature.getSignatureSimpleName(proposal.getSignature());
-					this.elements.addElement(new String(typeName));
+					this.elements.add(new String(typeName));
 					acceptCommon(proposal);
 					if (this.debug) {
 						if(Signature.getTypeSignatureKind(proposal.getSignature()) == Signature.TYPE_VARIABLE_SIGNATURE) {
@@ -75,56 +75,56 @@ public class CompletionTestsRequestor extends CompletionRequestor {
 				break;
 
 			case CompletionProposal.FIELD_REF :
-				this.elements.addElement(new String(proposal.getName()));
+				this.elements.add(new String(proposal.getName()));
 				acceptCommon(proposal);
 				if (this.debug)
 					System.out.println("Field " + new String(proposal.getName()));
 				break;
 
 			case CompletionProposal.KEYWORD:
-				this.elements.addElement(new String(proposal.getName()));
+				this.elements.add(new String(proposal.getName()));
 				acceptCommon(proposal);
 				if (this.debug)
 					System.out.println("Keyword " + new String(proposal.getName()));
 				break;
 
 			case CompletionProposal.LABEL_REF:
-				this.elements.addElement(new String(proposal.getName()));
+				this.elements.add(new String(proposal.getName()));
 				acceptCommon(proposal);
 				if (this.debug)
 					System.out.println("Label " + new String(proposal.getName()));
 				break;
 
 			case CompletionProposal.LOCAL_VARIABLE_REF:
-				this.elements.addElement(new String(proposal.getName()));
+				this.elements.add(new String(proposal.getName()));
 				acceptCommon(proposal);
 				if (this.debug)
 					System.out.println("Local variable " + new String(proposal.getName()));
 				break;
 
 			case CompletionProposal.METHOD_REF:
-				this.elements.addElement(new String(proposal.getName()));
+				this.elements.add(new String(proposal.getName()));
 				acceptCommon(proposal);
 				if (this.debug)
 					System.out.println("method " + new String(proposal.getName()));
 				break;
 
 			case CompletionProposal.METHOD_DECLARATION:
-				this.elements.addElement(new String(proposal.getName()));
+				this.elements.add(new String(proposal.getName()));
 				acceptCommon(proposal);
 				if (this.debug)
 					System.out.println("method declaration " + new String(proposal.getName()));
 				break;
 
 			case CompletionProposal.PACKAGE_REF:
-				this.elements.addElement(new String(proposal.getDeclarationSignature()));
+				this.elements.add(new String(proposal.getDeclarationSignature()));
 				acceptCommon(proposal);
 				if (this.debug)
 					System.out.println("package " + new String(proposal.getDeclarationSignature()));
 				break;
 
 			case CompletionProposal.VARIABLE_DECLARATION:
-				this.elements.addElement(new String(proposal.getName()));
+				this.elements.add(new String(proposal.getName()));
 				acceptCommon(proposal);
 				if (this.debug)
 					System.out.println("variable name " + new String(proposal.getName()));
@@ -168,19 +168,19 @@ public class CompletionTestsRequestor extends CompletionRequestor {
 
 		StringBuffer buffer =  new StringBuffer();
 		buffer.append("element:");
-		buffer.append(this.elements.elementAt(i));
+		buffer.append(this.elements.get(i));
 		buffer.append("    completion:");
-		buffer.append(this.completions.elementAt(i));
+		buffer.append(this.completions.get(i));
 		if(position) {
 			buffer.append("    position:[");
-			buffer.append(this.completionStart.elementAt(i));
+			buffer.append(this.completionStart.get(i));
 			buffer.append(",");
-			buffer.append(this.completionEnd.elementAt(i));
+			buffer.append(this.completionEnd.get(i));
 			buffer.append("]");
 		}
 		if(relevance) {
 			buffer.append("    relevance:");
-			buffer.append(this.relevances.elementAt(i));
+			buffer.append(this.relevances.get(i));
 		}
 		return buffer.toString();
 	}
