@@ -7,7 +7,7 @@
  * https://www.eclipse.org/legal/epl-2.0/
  *
  * SPDX-License-Identifier: EPL-2.0
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -15,6 +15,7 @@ package org.eclipse.jdt.core.tests.model;
 
 import java.util.Arrays;
 import java.util.Comparator;
+import java.util.function.Predicate;
 
 import org.eclipse.jdt.core.CompletionContext;
 import org.eclipse.jdt.core.CompletionProposal;
@@ -22,8 +23,10 @@ import org.eclipse.jdt.core.CompletionRequestor;
 import org.eclipse.jdt.core.Flags;
 import org.eclipse.jdt.core.IJavaElement;
 import org.eclipse.jdt.core.Signature;
+import org.eclipse.jdt.core.compiler.CharOperation;
 import org.eclipse.jdt.core.compiler.IProblem;
 import org.eclipse.jdt.internal.codeassist.InternalCompletionContext;
+import org.eclipse.jdt.internal.codeassist.impl.RestrictedIdentifiers;
 import org.eclipse.jdt.internal.compiler.ast.ASTNode;
 import org.eclipse.jdt.internal.compiler.util.ObjectVector;
 import org.eclipse.jdt.internal.core.JavaElement;
@@ -52,6 +55,8 @@ public class CompletionTestsRequestor2 extends CompletionRequestor {
 
 	public boolean debug = false;
 
+	private Predicate<CompletionProposal> proposalFilter = x -> true;
+
 	public CompletionTestsRequestor2() {
 		this(false, false);
 	}
@@ -96,6 +101,9 @@ public class CompletionTestsRequestor2 extends CompletionRequestor {
 		this.context = cc;
 	}
 	public void accept(CompletionProposal proposal) {
+		if (!this.proposalFilter.test(proposal))
+			return;
+
 		int length = this.proposals.length;
 		if (++this.proposalsPtr== length) {
 			System.arraycopy(this.proposals, 0, this.proposals = new CompletionProposal[length+PROPOSALS_INCREMENT], 0, length);
@@ -275,6 +283,7 @@ public class CompletionTestsRequestor2 extends CompletionRequestor {
 	public String getReversedResults() {
 		if(this.proposalsPtr < 0) return "";
 		Arrays.sort(this.proposals, new Comparator() {
+			@Override
 			public int compare(Object o1, Object o2) {
 				if (o1 == o2)
 					return 0;
@@ -338,8 +347,12 @@ public class CompletionTestsRequestor2 extends CompletionRequestor {
 			case CompletionProposal.FIELD_REF_WITH_CASTED_RECEIVER :
 				buffer.append("FIELD_REF_WITH_CASTED_RECEIVER"); //$NON-NLS-1$
 				break;
-			case CompletionProposal.KEYWORD :
-				buffer.append("KEYWORD"); //$NON-NLS-1$
+			case CompletionProposal.KEYWORD :{
+				if(isRestrictedIdentifier(proposal.getCompletion()))
+						buffer.append("RESTRICTED_IDENTIFIER");
+					else
+						buffer.append("KEYWORD"); //$NON-NLS-1$
+			}
 				break;
 			case CompletionProposal.LABEL_REF :
 				buffer.append("LABEL_REF"); //$NON-NLS-1$
@@ -515,6 +528,18 @@ public class CompletionTestsRequestor2 extends CompletionRequestor {
 		return buffer;
 	}
 
+	private boolean isRestrictedIdentifier(char[] completion) {
+		if(CharOperation.equals(completion, RestrictedIdentifiers.RECORD))
+			return true;
+		if(CharOperation.equals(completion, RestrictedIdentifiers.SEALED))
+			return true;
+		if(CharOperation.equals(completion, RestrictedIdentifiers.NON_SEALED))
+			return true;
+		if(CharOperation.equals(completion, RestrictedIdentifiers.PERMITS))
+			return true;
+
+		return false;
+	}
 	protected CompletionProposal[] quickSort(CompletionProposal[] collection, int left, int right) {
 		int original_left = left;
 		int original_right = right;
@@ -624,60 +649,63 @@ public class CompletionTestsRequestor2 extends CompletionRequestor {
 	public void setComputeEnclosingElement(boolean computeEnclosingElement) {
 		this.computeEnclosingElement = computeEnclosingElement;
 	}
-	
+	public void setProposalFilter(Predicate<CompletionProposal> proposalFilter) {
+		this.proposalFilter = proposalFilter;
+	}
+
 	public boolean canUseDiamond(int proposalNo) {
 		if (proposalNo < this.proposals.length && this.proposals[proposalNo] != null) {
 			return this.proposals[proposalNo].canUseDiamond(this.context);
 		}
 		return false;
 	}
-	
+
 	public String getCompletionNode() {
 		if (this.context instanceof InternalCompletionContext) {
 			InternalCompletionContext internalCompletionContext = (InternalCompletionContext) this.context;
 			ASTNode astNode = internalCompletionContext.getCompletionNode();
 			if (astNode != null) return astNode.toString();
-			
+
 		}
 		return null;
 	}
-	
+
 	public String getCompletionNodeParent() {
 		if (this.context instanceof InternalCompletionContext) {
 			InternalCompletionContext internalCompletionContext = (InternalCompletionContext) this.context;
 			ASTNode astNode = internalCompletionContext.getCompletionNodeParent();
 			if (astNode != null) return astNode.toString();
-			
+
 		}
 		return null;
 	}
-	
+
 	public String getVisibleLocalVariables() {
 		if (this.context instanceof InternalCompletionContext) {
 			InternalCompletionContext internalCompletionContext = (InternalCompletionContext) this.context;
 			ObjectVector locals = internalCompletionContext.getVisibleLocalVariables();
 			if (locals != null) return locals.toString();
-			
+
 		}
 		return null;
 	}
-	
+
 	public String getVisibleFields() {
 		if (this.context instanceof InternalCompletionContext) {
 			InternalCompletionContext internalCompletionContext = (InternalCompletionContext) this.context;
 			ObjectVector fields = internalCompletionContext.getVisibleFields();
 			if (fields != null) return fields.toString();
-			
+
 		}
 		return null;
 	}
-	
+
 	public String getVisibleMethods() {
 		if (this.context instanceof InternalCompletionContext) {
 			InternalCompletionContext internalCompletionContext = (InternalCompletionContext) this.context;
 			ObjectVector methods = internalCompletionContext.getVisibleMethods();
 			if (methods != null) return methods.toString();
-			
+
 		}
 		return null;
 	}
