@@ -20,10 +20,10 @@ import org.eclipse.jdt.core.search.SearchPattern;
 
 /**
  * Class to test the matching regions API method added on {@link SearchPattern}
- * 
+ *
  * @see SearchPattern#getMatchingRegions(String, String, int)
  * @see "https://bugs.eclipse.org/bugs/show_bug.cgi?id=218605"
- * 
+ *
  * @since 3.5
  */
 public class MatchingRegionsTest extends AbstractJavaModelTests {
@@ -1589,4 +1589,59 @@ public void testSubword_backtrackStartAndFail() {
 	int[] regions = SearchPattern.getMatchingRegions("listener", name, SearchPattern.R_SUBWORD_MATCH);
 	assertEquals("Unexpected matching regions", null, printRegions(name, regions));
 }
+public void testSubword_fieldPrefix() {
+	String name = "_field";
+	int[] regions = SearchPattern.getMatchingRegions("field", name, SearchPattern.R_SUBWORD_MATCH);
+	assertEquals("Unexpected matching regions", "_[field]", printRegions(name, regions));
+}
+public void testSubword_contentAssistFilter() {
+	String name = "substring(int beginIndex)";
+	int[] regions = SearchPattern.getMatchingRegions("index", name, SearchPattern.R_SUBWORD_MATCH);
+	assertEquals("Unexpected matching regions", null, printRegions(name, regions));
+}
+public void testSubword_caps_boundaries1() {
+	String name = "CASE_INSENSITIVE_ORDER";
+	int[] regions = SearchPattern.getMatchingRegions("ind", name, SearchPattern.R_SUBWORD_MATCH);
+	assertEquals("Unexpected matching regions", null, printRegions(name, regions));
+}
+public void testSubword_caps_boundaries2() {
+	String name = "CASE_INSENSITIVE_ORDER";
+	int[] regions = SearchPattern.getMatchingRegions("ini", name, SearchPattern.R_SUBWORD_MATCH);
+	assertEquals("Unexpected matching regions", null, printRegions(name, regions));
+}
+public void testSubword_caps_boundaries3() {
+	String name = "CASE_INSENSITIVE_ORDER";
+	int[] regions = SearchPattern.getMatchingRegions("sensitive", name, SearchPattern.R_SUBWORD_MATCH);
+	assertEquals("Unexpected matching regions", null, printRegions(name, regions));
+}
+public void testSubword_caps_backtracking() {
+	String name = "LIST_LISTENER";
+	int[] regions = SearchPattern.getMatchingRegions("listener", name, SearchPattern.R_SUBWORD_MATCH);
+	assertEquals("Unexpected matching regions", "LIST_[LISTENER]", printRegions(name, regions));
+}
+public void testSubword_snakeCase() {
+	String name = "add_list_listener";
+	int[] regions = SearchPattern.getMatchingRegions("addlistener", name, SearchPattern.R_SUBWORD_MATCH);
+	assertEquals("Unexpected matching regions", "[add]_list_[listener]", printRegions(name, regions));
+}
+public void testSubword_mixedCamelCase1() {
+	String name = "IImportWizard";
+	int[] regions = SearchPattern.getMatchingRegions("import", name, SearchPattern.R_SUBWORD_MATCH);
+	assertEquals("Unexpected matching regions", "I[Import]Wizard", printRegions(name, regions));
+}
+public void testSubword_mixedCamelCase2() {
+	String name = "HTMLTable";
+	int[] regions = SearchPattern.getMatchingRegions("table", name, SearchPattern.R_SUBWORD_MATCH);
+	assertEquals("Unexpected matching regions", "HTML[Table]", printRegions(name, regions));
+}
+public void testSubword_mixedCamelCase3() {
+	String name = "CustomHTMLTable";
+	int[] regions = SearchPattern.getMatchingRegions("table", name, SearchPattern.R_SUBWORD_MATCH);
+	assertEquals("Unexpected matching regions", "CustomHTML[Table]", printRegions(name, regions));
+}
+public void testSubword_mixedCamelCase4() {
+	String name = "ImportHTML";
+	int[] regions = SearchPattern.getMatchingRegions("html", name, SearchPattern.R_SUBWORD_MATCH);
+	assertEquals("Unexpected matching regions", "Import[HTML]", printRegions(name, regions));
+}
 }
