@@ -53,6 +53,7 @@ public static Test suite() {
 public ResolveTests_1_5(String name) {
 	super(name);
 }
+@Override
 public ICompilationUnit getWorkingCopy(String path, String source) throws JavaModelException {
 	return super.getWorkingCopy(path, source, this.wcOwner);
 }
@@ -69,6 +70,7 @@ private IJavaElement[] selectAfter(String path, String source, String selection)
 	int start = str.lastIndexOf(selection) + selection.length();
 	return this.wc.codeSelect(start, 0, this.wcOwner);
 }
+@Override
 public void setUpSuite() throws Exception {
 	super.setUpSuite();
 
@@ -76,16 +78,19 @@ public void setUpSuite() throws Exception {
 
 	waitUntilIndexesReady();
 }
+@Override
 protected void setUp() throws Exception {
 	super.setUp();
 	this.wcOwner = new WorkingCopyOwner(){};
 }
+@Override
 public void tearDownSuite() throws Exception {
 	deleteProject("Resolve");
 
 	super.tearDownSuite();
 }
 
+@Override
 protected void tearDown() throws Exception {
 	if (this.wc != null) {
 		this.wc.discardWorkingCopy();
@@ -2732,7 +2737,7 @@ public void test0117() throws Exception {
 			"		@A(n=) String str;\n" +
 			"	}\n" +
 			"}");
-	
+
 	this.workingCopies[1] = getWorkingCopy(
 			"/Resolve/src/test/A.java",
 			"package test;\n" +
@@ -2765,7 +2770,7 @@ public void test0118() throws Exception {
 			"		@A(n=) String str;\n" +
 			"	}\n" +
 			"}");
-	
+
 	this.workingCopies[1] = getWorkingCopy(
 			"/Resolve/src/test/A.java",
 			"package test;\n" +
@@ -2779,7 +2784,7 @@ public void test0118() throws Exception {
 	int start = str.lastIndexOf("str");
 	int length = "str".length();
 	IJavaElement[] elements =  this.workingCopies[0].codeSelect(start, length, this.wcOwner);
-	
+
 	if (elements != null &&
 			elements.length > 0 &&
 			elements[0] instanceof IAnnotatable) {
@@ -2802,14 +2807,14 @@ public void test0119() throws Exception {
 			"		return Collections.emptyList();\n" +
 			"	}\n" +
 			"}");
-	
+
 	this.workingCopies[1] = getWorkingCopy(
 			"/Resolve/src/test/Collections.java",
 			"package test;\n" +
 			"public class Collections  {\n" +
 			"	public static final <T> List<T> emptyList() {return null;}\n" +
 			"}");
-	
+
 	this.workingCopies[1] = getWorkingCopy(
 			"/Resolve/src/test/List.java",
 			"package test;\n" +
@@ -2839,14 +2844,14 @@ public void test0120() throws Exception {
 			"		List<String> local = Collections.emptyList();\n" +
 			"	}\n" +
 			"}");
-	
+
 	this.workingCopies[1] = getWorkingCopy(
 			"/Resolve/src/test/Collections.java",
 			"package test;\n" +
 			"public class Collections  {\n" +
 			"	public static final <T> List<T> emptyList() {return null;}\n" +
 			"}");
-	
+
 	this.workingCopies[1] = getWorkingCopy(
 			"/Resolve/src/test/List.java",
 			"package test;\n" +
@@ -2876,14 +2881,14 @@ public void test0121() throws Exception {
 			"	public void foo() {\n" +
 			"	}\n" +
 			"}");
-	
+
 	this.workingCopies[1] = getWorkingCopy(
 			"/Resolve/src/test/Collections.java",
 			"package test;\n" +
 			"public class Collections  {\n" +
 			"	public static final <T> List<T> emptyList() {return null;}\n" +
 			"}");
-	
+
 	this.workingCopies[1] = getWorkingCopy(
 			"/Resolve/src/test/List.java",
 			"package test;\n" +
@@ -2931,13 +2936,13 @@ public void test0123() throws Exception {
 	this.workingCopies[0] = getWorkingCopy(
 			"/Resolve/src/test/Test.java",
 			"package test;\n" +
-			"public class Test {\n" + 
-			"        <T> T bar(T t) { return t; }\n" + 
-			"        void foo(boolean b, Runnable r) {\n" + 
-			"                Zork z = null;\n" + 
-			"                String s = (String) bar(z); // 5\n" + 
-			"        }\n" + 
-			"}\n" + 
+			"public class Test {\n" +
+			"        <T> T bar(T t) { return t; }\n" +
+			"        void foo(boolean b, Runnable r) {\n" +
+			"                Zork z = null;\n" +
+			"                String s = (String) bar(z); // 5\n" +
+			"        }\n" +
+			"}\n" +
 			"\n");
 
 	String str = this.workingCopies[0].getSource();
@@ -2957,10 +2962,10 @@ public void test0124() throws Exception {
 	this.workingCopies = new ICompilationUnit[1];
 	this.workingCopies[0] = getWorkingCopy(
 			"/Resolve/src/Test.java",
-			"public class Test {\n" + 
-			"	public static void main(String[] args) {\n" + 
-			"		String str\ud842\udf9f = null;\n" + 
-			"	}\n" + 
+			"public class Test {\n" +
+			"	public static void main(String[] args) {\n" +
+			"		String str\ud842\udf9f = null;\n" +
+			"	}\n" +
 			"}");
 
 	String str = this.workingCopies[0].getSource();
@@ -2987,7 +2992,7 @@ public void test0125() throws CoreException {
 		WorkingCopyOwner owner = newWorkingCopyOwner(new BasicProblemRequestor());
 		ICompilationUnit cu = getWorkingCopy(
 				"/P0125/src/Test.java",
-				"public enum Test {\n" + 
+				"public enum Test {\n" +
 				"}",
 				owner);
 		IType type = cu.getType("Test");
