@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
+ * Copyright (c) 2000, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -15,7 +15,8 @@ package org.eclipse.jdt.core.tests.model;
 
 import java.io.FileInputStream;
 import java.io.IOException;
-import java.util.Vector;
+import java.util.ArrayList;
+import java.util.List;
 
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.resources.IResource;
@@ -38,7 +39,6 @@ import junit.framework.Test;
 /*
  * Test indexing support.
  */
-@SuppressWarnings({"rawtypes", "unchecked"})
 public class SearchTests extends ModifyingResourceTests implements IJavaSearchConstants {
 	/*
 	 * Empty jar contents.
@@ -78,21 +78,21 @@ public class SearchTests extends ModifyingResourceTests implements IJavaSearchCo
 		}
 	}
 	public static class SearchMethodNameRequestor extends MethodNameRequestor {
-		Vector results = new Vector<>();
+		List<String> results = new ArrayList<>();
 		@Override
 		public void acceptMethod(
-				char[] methodName, 
-				int parameterCount, 
-				char[] declaringQualifier, 
-				char[] simpleTypeName, 
-				int typeModifiers, 
-				char[] packageName, 
-				char[] signature, 
-				char[][] parameterTypes, 
-				char[][] parameterNames, 
-				char[] returnType, 
-				int modifiers, 
-				String path, 
+				char[] methodName,
+				int parameterCount,
+				char[] declaringQualifier,
+				char[] simpleTypeName,
+				int typeModifiers,
+				char[] packageName,
+				char[] signature,
+				char[][] parameterTypes,
+				char[][] parameterNames,
+				char[] returnType,
+				int modifiers,
+				String path,
 				int methodIndex) {
 			StringBuffer buffer = new StringBuffer();
 			char c = '.';
@@ -107,9 +107,9 @@ public class SearchTests extends ModifyingResourceTests implements IJavaSearchCo
 			buffer.append(methodName);
 			buffer.append('(');
 			parameterTypes = signature == null ? parameterTypes : Signature.getParameterTypes(signature);
-			
+
 			for (int i = 0; i < parameterCount; i++) {
-				
+
 				if (parameterTypes != null) {
 					char[] parameterType;
 					if (parameterTypes.length != parameterCount) {
@@ -136,7 +136,7 @@ public class SearchTests extends ModifyingResourceTests implements IJavaSearchCo
 				if (parameterCount > 1 && i < parameterCount - 1) buffer.append(',');
 			}
 			buffer.append(')');
-			this.results.addElement(buffer.toString());
+			this.results.add(buffer.toString());
 		}
 		static void checkAndAddtoBuffer(StringBuffer buffer, char[] precond, char c) {
 			if (precond == null || precond.length == 0) return;
@@ -162,14 +162,14 @@ public class SearchTests extends ModifyingResourceTests implements IJavaSearchCo
 		}
 	}
 	public static class SearchTypeNameRequestor extends TypeNameRequestor {
-		Vector results = new Vector();
+		List<String> results = new ArrayList<>();
 		public void acceptType(int modifiers, char[] packageName, char[] simpleTypeName, char[][] enclosingTypeNames, String path) {
 			char[] typeName =
 				CharOperation.concat(
 					CharOperation.concatWith(enclosingTypeNames, '$'),
 					simpleTypeName,
 					'$');
-			this.results.addElement(new String(CharOperation.concat(packageName, typeName, '.')));
+			this.results.add(new String(CharOperation.concat(packageName, typeName, '.')));
 		}
 		public String toString(){
 			int length = this.results.size();
@@ -323,6 +323,7 @@ protected void assertValidMatchRule(String pattern, int rule, int expected) {
 		assertEquals(pattern+"' does not match expected match rule!", expectedRule, validatedRule);
 	}
 }
+@Override
 public void setUpSuite() throws Exception {
 	super.setUpSuite();
 	createJavaProject("P");
@@ -345,6 +346,7 @@ public void setUpSuite() throws Exception {
 		"}"
 	);
 }
+@Override
 public void tearDownSuite() throws Exception {
 	deleteProject("P");
 	super.tearDownSuite();
@@ -472,6 +474,7 @@ public void testConcurrentJob() throws CoreException, InterruptedException, IOEx
 		final boolean[] success = new boolean[1];
 		final WaitUntilReadyMonitor monitor = new WaitUntilReadyMonitor();
 		Thread thread = new Thread() {
+			@Override
 			public void run() {
 				try {
 					assertAllTypes(
@@ -1138,8 +1141,8 @@ public void testSearchPatternCreation37() {
 		"QUALIFIED_REFERENCE | " +
 		"THIS_REFERENCE | " +
 		"IMPLICIT_THIS_REFERENCE | " +
-		"METHOD_REFERENCE_EXPRESSION | " + // unused slots
-		" | " +
+		"METHOD_REFERENCE_EXPRESSION | " +
+		"PERMITTYPE_TYPE_REFERENCE | " + // unused slots
 		" | ",
 		searchPattern);
 }
