@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2019 IBM Corporation and others.
+ * Copyright (c) 2019, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -34,6 +34,7 @@ import org.eclipse.jdt.core.dom.SwitchStatement;
 import org.eclipse.jdt.core.dom.TypeDeclaration;
 import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
 import org.eclipse.jdt.core.dom.VariableDeclarationStatement;
+import org.eclipse.jdt.core.dom.YieldStatement;
 import org.eclipse.jdt.core.dom.rewrite.ASTRewrite;
 import org.eclipse.jdt.core.dom.rewrite.ListRewrite;
 
@@ -43,7 +44,7 @@ public class ASTRewritingSwitchExpressionsTest extends ASTRewritingTest {
 
 
 	public ASTRewritingSwitchExpressionsTest(String name) {
-		super(name, 12);
+		super(name, 14);
 	}
 
 	public ASTRewritingSwitchExpressionsTest(String name, int apiLevel) {
@@ -54,18 +55,21 @@ public class ASTRewritingSwitchExpressionsTest extends ASTRewritingTest {
 		return createSuite(ASTRewritingSwitchExpressionsTest.class);
 	}
 
+	@SuppressWarnings("deprecation")
 	@Override
 	protected void setUp() throws Exception {
 		super.setUp();
-		if (this.apiLevel == AST.JLS13 ) {
-			this.project1.setOption(JavaCore.COMPILER_PB_ENABLE_PREVIEW_FEATURES, JavaCore.ENABLED);
-			this.project1.setOption(JavaCore.COMPILER_PB_REPORT_PREVIEW_FEATURES, JavaCore.IGNORE);
-			this.project1.setOption(JavaCore.COMPILER_COMPLIANCE, JavaCore.VERSION_13);
-			this.project1.setOption(JavaCore.COMPILER_SOURCE, JavaCore.VERSION_13);
-			this.project1.setOption(JavaCore.COMPILER_CODEGEN_TARGET_PLATFORM, JavaCore.VERSION_13);
+		if (this.apiLevel == AST_INTERNAL_JLS15 ) {
+			this.project1.setOption(JavaCore.COMPILER_COMPLIANCE, JavaCore.VERSION_15);
+			this.project1.setOption(JavaCore.COMPILER_SOURCE, JavaCore.VERSION_15);
+			this.project1.setOption(JavaCore.COMPILER_CODEGEN_TARGET_PLATFORM, JavaCore.VERSION_15);
+		} else if (this.apiLevel == AST_INTERNAL_JLS14 ) {
+			this.project1.setOption(JavaCore.COMPILER_COMPLIANCE, JavaCore.VERSION_14);
+			this.project1.setOption(JavaCore.COMPILER_SOURCE, JavaCore.VERSION_14);
+			this.project1.setOption(JavaCore.COMPILER_CODEGEN_TARGET_PLATFORM, JavaCore.VERSION_14);
 		}
 	}
-	 	
+
 	@SuppressWarnings("rawtypes")
 	public void testSwitchExpressions_since_12() throws Exception {
 		IPackageFragment pack1= this.sourceFolder.createPackageFragment("test1", false, null);
@@ -112,7 +116,7 @@ public class ASTRewritingSwitchExpressionsTest extends ASTRewritingTest {
 			caseStatement1.setSwitchLabeledRule(true);
 			caseStatement1.expressions().add(ast.newNumberLiteral("1"));
 			caseStatement1.expressions().add(ast.newNumberLiteral("2"));
-			
+
 
 			Statement statement1= ast.newReturnStatement();
 
@@ -169,9 +173,8 @@ public class ASTRewritingSwitchExpressionsTest extends ASTRewritingTest {
 		buf.append("}\n");
 		assertEqualString(preview, buf.toString());
 	}
-	@Deprecated
 	@SuppressWarnings("rawtypes")
-	public void _testSwitchExpressions_02_since_12() throws Exception {
+	public void testSwitchExpressions_02_since_12() throws Exception {
 		IPackageFragment pack1= this.sourceFolder.createPackageFragment("test1", false, null);
 		StringBuffer buf= new StringBuffer();
 		buf.append("package test1;\n");
@@ -203,7 +206,7 @@ public class ASTRewritingSwitchExpressionsTest extends ASTRewritingTest {
 			caseStatement1.setSwitchLabeledRule(true);
 			caseStatement1.expressions().add(ast.newNumberLiteral("1024"));
 
-			BreakStatement breakStatement = ast.newBreakStatement();
+			YieldStatement breakStatement = ast.newYieldStatement();
 			breakStatement.setExpression(ast.newNumberLiteral("2048"));
 			ListRewrite listRewrite= rewrite.getListRewrite(switchStatement, SwitchStatement.STATEMENTS_PROPERTY);
 
@@ -221,7 +224,7 @@ public class ASTRewritingSwitchExpressionsTest extends ASTRewritingTest {
 		buf.append("        switch (i) {\n");
 		buf.append("            case 1, 2 -> i= 1;\n");
 		buf.append("            case 3 -> i= 3;\n");
-		buf.append("            case 1024 -> break 2048;\n");
+		buf.append("            case 1024 -> yield 2048;\n");
 		buf.append("            default -> i= 4;\n");
 		buf.append("        }\n");
 		buf.append("    }\n");
@@ -229,9 +232,11 @@ public class ASTRewritingSwitchExpressionsTest extends ASTRewritingTest {
 		assertEqualString(preview, buf.toString());
 	}
 
-	@Deprecated
 	@SuppressWarnings("rawtypes")
-	public void _testSwitchExpressions_03_since_12() throws Exception {
+	public void testSwitchExpressions_03_since_12() throws Exception {
+		/*
+		 * if (this.apiLevel == AST_INTERNAL_JLS15) { return; }
+		 */
 		IPackageFragment pack1= this.sourceFolder.createPackageFragment("test1", false, null);
 		String s	=
 				"package test1;\n"+
@@ -241,10 +246,10 @@ public class ASTRewritingSwitchExpressionsTest extends ASTRewritingTest {
 				"		switch (i) {\n"+
 				"			case 1 -> {\n"+
 				" 				int z = 100;\n"+
-				" 				break z;\n"+
+				" 				yield z;\n"+
 				"			}\n"+
 				"			default -> {\n"+
-				"				break 12;\n"+
+				"				yield 12;\n"+
 				"			}\n"+
 				"		};\n"+
 				"		return tw;\n"+
@@ -278,10 +283,10 @@ public class ASTRewritingSwitchExpressionsTest extends ASTRewritingTest {
 			SwitchCase caseStatement2= ast.newSwitchCase(); // default
 			caseStatement2.setSwitchLabeledRule(true);
 
-			BreakStatement breakStatement = ast.newBreakStatement();
-			breakStatement.setExpression(ast.newNumberLiteral("2048"));
+			YieldStatement yieldStatement = ast.newYieldStatement();
+			yieldStatement.setExpression(ast.newNumberLiteral("2048"));
 			Block block1 = ast.newBlock();
-			block1.statements().add(breakStatement);
+			block1.statements().add(yieldStatement);
 
 			SwitchCase defaultCase = (SwitchCase) switchExpression.statements().get(2);
 			ListRewrite listRewrite= rewrite.getListRewrite(switchExpression, SwitchExpression.STATEMENTS_PROPERTY);
@@ -300,11 +305,13 @@ public class ASTRewritingSwitchExpressionsTest extends ASTRewritingTest {
 		buf.append("		switch (i) {\n");
 		buf.append("			case 1 -> {\n");
 		buf.append(" 				int z = 100;\n");
-		buf.append(" 				break z;\n");
+		buf.append(" 				yield z;\n");
 		buf.append("			}\n");
-		buf.append("			case 100, 200 -> {break 2048;}\n");
+		buf.append("			case 100, 200 -> {\n");
+		buf.append("    yield 2048;\n");
+		buf.append("}\n");
 		buf.append("            default -> {\n");
-		buf.append("				break 12;\n");
+		buf.append("				yield 12;\n");
 		buf.append("			}\n");
 		buf.append("		};\n");
 		buf.append("		return tw;\n");
@@ -317,7 +324,7 @@ public class ASTRewritingSwitchExpressionsTest extends ASTRewritingTest {
 	}
 
 	@SuppressWarnings("rawtypes")
-	public void _testSwitchStatement_Bug543720_since_12() throws Exception {
+	public void testSwitchStatement_Bug543720_since_12() throws Exception {
 		IPackageFragment pack1= this.sourceFolder.createPackageFragment("test1", false, null);
 		String s	=
 				"package test1;\n"+
@@ -398,7 +405,10 @@ public class ASTRewritingSwitchExpressionsTest extends ASTRewritingTest {
 		assertEqualString(preview, buf.toString());
 	}
 	@SuppressWarnings("rawtypes")
-	public void _testSwitchExpressions_04_since_12() throws Exception {
+	public void testSwitchExpressions_04_since_12() throws Exception {
+		/*
+		 * if (this.apiLevel == AST_INTERNAL_JLS15) { return; }
+		 */
 		IPackageFragment pack1= this.sourceFolder.createPackageFragment("test1", false, null);
 		String s	= "package test1;\n"+
 				"public class X {\n"+
@@ -408,10 +418,10 @@ public class ASTRewritingSwitchExpressionsTest extends ASTRewritingTest {
 				"			case 1 -> \n"+
 				"			 {\n"+
 				" 				int z = 100;\n"+
-				" 				break z;\n"+
+				" 				yield z;\n"+
 				"			}\n"+
 				"			default -> {\n"+
-				"				break 12;\n"+
+				"				yield 12;\n"+
 				"			}\n"+
 				"		};\n"+
 				"		return tw;\n"+
@@ -456,7 +466,7 @@ public class ASTRewritingSwitchExpressionsTest extends ASTRewritingTest {
 		buf.append("		int tw =\n");
 		buf.append("		switch (x) {\n");
 		buf.append("			default -> {\n");
-		buf.append("				break 12;\n");
+		buf.append("				yield 12;\n");
 		buf.append("			}\n");
 		buf.append("		};\n");
 		buf.append("		return tw;\n");
@@ -788,10 +798,10 @@ public class ASTRewritingSwitchExpressionsTest extends ASTRewritingTest {
 		builder.append("package test1;\n");
 		builder.append("public class X {\n");
 		builder.append("    public String foo(int i) {\n" +
-				"		String ret = switch(i%2) {\n" + 
-				"		case 0 -> \"even\";\n" + 
-				"		default -> \"\";\n" + 
-				"		};\n" + 
+				"		String ret = switch(i%2) {\n" +
+				"		case 0 -> \"even\";\n" +
+				"		default -> \"\";\n" +
+				"		};\n" +
 				"		return ret;");
 		builder.append("    }\n");
 		builder.append("}\n");
@@ -820,7 +830,7 @@ public class ASTRewritingSwitchExpressionsTest extends ASTRewritingTest {
 			rewrite.set(cse1, SwitchCase.SWITCH_LABELED_RULE_PROPERTY, Boolean.FALSE, null);
 			SwitchCase cse2 = (SwitchCase) statements.get(2);
 			rewrite.set(cse2, SwitchCase.SWITCH_LABELED_RULE_PROPERTY, Boolean.FALSE, null);
-			
+
 			ListRewrite listRewrite= rewrite.getListRewrite(initializer, SwitchExpression.STATEMENTS_PROPERTY);
 			SwitchCase caseStatement1= ast.newSwitchCase();
 			caseStatement1.setSwitchLabeledRule(false);
@@ -838,12 +848,12 @@ public class ASTRewritingSwitchExpressionsTest extends ASTRewritingTest {
 		builder.append("package test1;\n");
 		builder.append("public class X {\n");
 		builder.append("    public String foo(int i) {\n" +
-				"		String ret = switch(i%2) {\n" + 
-				"		case 0 : \"even\";\n" + 
+				"		String ret = switch(i%2) {\n" +
+				"		case 0 : \"even\";\n" +
 				"            case 1:\n" +
-				"                \"odd\";\n" + 
-				"		default : \"\";\n" + 
-				"		};\n" + 
+				"                \"odd\";\n" +
+				"		default : \"\";\n" +
+				"		};\n" +
 				"		return ret;");
 		builder.append("    }\n");
 		builder.append("}\n");
