@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2018 IBM Corporation and others.
+ * Copyright (c) 2000, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -550,8 +550,13 @@ public class CompletionJavadocParser extends JavadocParser {
 
 		@Override
 		protected boolean parseReference() throws InvalidInputException {
+			return parseReference(false);
+		}
+
+		@Override
+		protected boolean parseReference(boolean allowModule) throws InvalidInputException {
 			boolean completed = this.completionNode != null;
-			boolean valid = super.parseReference();
+			boolean valid = super.parseReference(allowModule);
 			if (!completed && this.completionNode != null) {
 				this.completionNode.addCompletionFlags(CompletionOnJavadoc.FORMAL_REFERENCE);
 			}
