@@ -120,7 +120,7 @@ public class ExplicitConstructorCall extends Statement implements Invocation {
 				if ((this.bits & ASTNode.Unchecked) != 0 && this.genericTypeArguments == null) {
 					// https://bugs.eclipse.org/bugs/show_bug.cgi?id=277643, align with javac on JLS 15.12.2.6
 					thrownExceptions = currentScope.environment().convertToRawTypes(this.binding.thrownExceptions, true, true);
-				}				
+				}
 				// check exceptions
 				flowContext.checkExceptionHandlers(
 					thrownExceptions,
@@ -310,7 +310,8 @@ public class ExplicitConstructorCall extends Statement implements Invocation {
 			if (methodDeclaration == null
 					|| !methodDeclaration.isConstructor()
 					|| ((ConstructorDeclaration) methodDeclaration).constructorCall != this) {
-				scope.problemReporter().invalidExplicitConstructorCall(this);
+				if (!(methodDeclaration instanceof CompactConstructorDeclaration)) // already flagged for CCD
+						scope.problemReporter().invalidExplicitConstructorCall(this);
 				// fault-tolerance
 				if (this.qualification != null) {
 					this.qualification.resolveType(scope);
@@ -486,7 +487,7 @@ public class ExplicitConstructorCall extends Statement implements Invocation {
 	public void setFieldIndex(int depth) {
 		// ignore for here
 	}
-	
+
 	@Override
 	public void traverse(ASTVisitor visitor, BlockScope scope) {
 		if (visitor.visit(this, scope)) {
@@ -516,17 +517,17 @@ public class ExplicitConstructorCall extends Statement implements Invocation {
 	public void registerInferenceContext(ParameterizedGenericMethodBinding method, InferenceContext18 infCtx18) {
 		// Nothing to do.
 	}
-	
+
 	@Override
 	public void registerResult(TypeBinding targetType, MethodBinding method) {
 		// Nothing to do.
 	}
-	
+
 	@Override
 	public InferenceContext18 getInferenceContext(ParameterizedMethodBinding method) {
 		return null;
 	}
-	
+
 	@Override
 	public void cleanUpInferenceContexts() {
 		// Nothing to do.
