@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2018 IBM Corporation and others.
+ * Copyright (c) 2000, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -7,7 +7,7 @@
  * https://www.eclipse.org/legal/epl-2.0/
  *
  * SPDX-License-Identifier: EPL-2.0
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *     Stephan Herrmann <stephan@cs.tu-berlin.de> - Contributions for
@@ -35,7 +35,7 @@
  *							Bug 529556 - [18.3] Add content assist support for 'var' as a type
  *        Andy Clement (GoPivotal, Inc) aclement@gopivotal.com - Contributions for
  *							Bug 409250 - [1.8][compiler] Various loose ends in 308 code generation
- *							Bug 426616 - [1.8][compiler] Type Annotations, multiple problems 
+ *							Bug 426616 - [1.8][compiler] Type Annotations, multiple problems
  *******************************************************************************/
 package org.eclipse.jdt.internal.compiler.ast;
 
@@ -55,7 +55,6 @@ import org.eclipse.jdt.internal.compiler.flow.*;
 import org.eclipse.jdt.internal.compiler.lookup.*;
 import org.eclipse.jdt.internal.compiler.parser.RecoveryScanner;
 
-@SuppressWarnings("rawtypes")
 public class LocalDeclaration extends AbstractVariableDeclaration {
 
 	public LocalVariableBinding binding;
@@ -81,9 +80,9 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, Fl
 		return flowInfo;
 	}
 	this.initialization.checkNPEbyUnboxing(currentScope, flowContext, flowInfo);
-	
+
 	FlowInfo preInitInfo = null;
-	boolean shouldAnalyseResource = this.binding != null 
+	boolean shouldAnalyseResource = this.binding != null
 			&& flowInfo.reachMode() == FlowInfo.REACHABLE
 			&& currentScope.compilerOptions().analyseResourceLeaks
 			&& FakedTrackingVariable.isAnyCloseable(this.initialization.resolvedType);
@@ -187,17 +186,17 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, Fl
 	}
 
 	// for local variables
-	public void getAllAnnotationContexts(int targetType, LocalVariableBinding localVariable, List allAnnotationContexts) {
+	public void getAllAnnotationContexts(int targetType, LocalVariableBinding localVariable, List<AnnotationContext> allAnnotationContexts) {
 		AnnotationCollector collector = new AnnotationCollector(this, targetType, localVariable, allAnnotationContexts);
 		this.traverseWithoutInitializer(collector, (BlockScope) null);
 	}
 
 	// for arguments
-	public void getAllAnnotationContexts(int targetType, int parameterIndex, List allAnnotationContexts) {
+	public void getAllAnnotationContexts(int targetType, int parameterIndex, List<AnnotationContext> allAnnotationContexts) {
 		AnnotationCollector collector = new AnnotationCollector(this, targetType, parameterIndex, allAnnotationContexts);
 		this.traverse(collector, (BlockScope) null);
 	}
-		
+
 	public boolean isArgument() {
 		return false;
 	}
@@ -208,7 +207,7 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, Fl
 		// Perform upwards projection on type wrt mentioned type variables
 		TypeBinding[] mentionedTypeVariables= findCapturedTypeVariables(newType);
 		if (mentionedTypeVariables != null && mentionedTypeVariables.length > 0) {
-			newType = newType.upwardsProjection(this.binding.declaringScope, mentionedTypeVariables); 	
+			newType = newType.upwardsProjection(this.binding.declaringScope, mentionedTypeVariables);
 		}
 		this.type.resolvedType = newType;
 		if (this.binding != null) {
@@ -231,7 +230,7 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, Fl
 		if (mentioned.isEmpty()) return null;
 		return mentioned.toArray(new TypeVariableBinding[mentioned.size()]);
 	}
-	
+
 	private static Expression findPolyExpression(Expression e) {
 		// This is simpler than using an ASTVisitor, since we only care about a very few select cases.
 		if (e instanceof FunctionalExpression) {
@@ -245,12 +244,21 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, Fl
 			}
 			if (candidate != null) return candidate;
 		}
+		if (e instanceof SwitchExpression) {
+			SwitchExpression se = (SwitchExpression)e;
+			for (Expression re : se.resultExpressions) {
+				Expression candidate = findPolyExpression(re);
+				if (candidate != null) return candidate;
+			}
+		}
 		return null;
 	}
-	
+
 	@Override
 	public void resolve(BlockScope scope) {
-
+		resolve(scope, false);
+	}
+	public void resolve(BlockScope scope, boolean isPatternVariable) {
 		// prescan NNBD
 		handleNonNullByDefault(scope, this.annotations, this);
 
@@ -289,11 +297,11 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, Fl
 
 		Binding existingVariable = scope.getBinding(this.name, Binding.VARIABLE, this, false /*do not resolve hidden field*/);
 		if (existingVariable != null && existingVariable.isValidBinding()){
-			boolean localExists = existingVariable instanceof LocalVariableBinding; 
+			boolean localExists = existingVariable instanceof LocalVariableBinding;
 			if (localExists && (this.bits & ASTNode.ShadowsOuterLocal) != 0 && scope.isLambdaSubscope() && this.hiddenVariableDepth == 0) {
-					scope.problemReporter().lambdaRedeclaresLocal(this);
+				scope.problemReporter().lambdaRedeclaresLocal(this);
 			} else if (localExists && this.hiddenVariableDepth == 0) {
-					scope.problemReporter().redefineLocal(this);
+				scope.problemReporter().redefineLocal(this);
 			} else {
 				scope.problemReporter().localVariableHiding(this, existingVariable, false);
 			}
@@ -304,10 +312,10 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, Fl
 		}
 		if (isTypeNameVar) {
 			// Create binding for the initializer's type
-			// In order to resolve self-referential initializers, we must declare the variable with a placeholder type (j.l.Object), and then patch it later 
+			// In order to resolve self-referential initializers, we must declare the variable with a placeholder type (j.l.Object), and then patch it later
 			this.binding = new LocalVariableBinding(this, variableType != null ? variableType : scope.getJavaLangObject(), this.modifiers, false) {
 				private boolean isInitialized = false;
-				
+
 				@Override
 				public void markReferenced() {
 					if (! this.isInitialized) {
@@ -332,6 +340,9 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, Fl
 
 		if (variableType == null) {
 			if (this.initialization != null) {
+				if (this.initialization instanceof CastExpression) {
+					((CastExpression)this.initialization).setVarTypeDeclaration(true);
+				}
 				this.initialization.resolveType(scope); // want to report all possible errors
 				if (isTypeNameVar && this.initialization.resolvedType != null) {
 					if (TypeBinding.equalsEquals(TypeBinding.NULL, this.initialization.resolvedType)) {
@@ -351,13 +362,13 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, Fl
 		if (variableTypeInferenceError) {
 			return;
 		}
-
 		boolean resolveAnnotationsEarly = false;
-		if (scope.environment().usesNullTypeAnnotations() 
+		if (scope.environment().usesNullTypeAnnotations()
 				&& !isTypeNameVar // 'var' does not provide a target type
-				&& variableType.isValidBinding()) { 
+				&& variableType != null && variableType.isValidBinding()) {
 			resolveAnnotationsEarly = this.initialization instanceof Invocation
 					|| this.initialization instanceof ConditionalExpression
+					|| this.initialization instanceof SwitchExpression
 					|| this.initialization instanceof ArrayInitializer;
 		}
 		if (resolveAnnotationsEarly) {
@@ -474,8 +485,8 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, Fl
 		}
 		visitor.endVisit(this, scope);
 	}
-	
-	private void traverseWithoutInitializer(ASTVisitor visitor, BlockScope scope) {		
+
+	private void traverseWithoutInitializer(ASTVisitor visitor, BlockScope scope) {
 		if (visitor.visit(this, scope)) {
 			if (this.annotations != null) {
 				int annotationsLength = this.annotations.length;
@@ -488,12 +499,12 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, Fl
 	}
 
 	public boolean isRecoveredFromLoneIdentifier() { // recovered from lonely identifier or identifier cluster ?
-		return this.name == RecoveryScanner.FAKE_IDENTIFIER && 
+		return this.name == RecoveryScanner.FAKE_IDENTIFIER &&
 				(this.type instanceof SingleTypeReference || (this.type instanceof QualifiedTypeReference && !(this.type instanceof ArrayQualifiedTypeReference))) && this.initialization == null && !this.type.isBaseTypeReference();
 	}
-	
+
 	public boolean isTypeNameVar(Scope scope) {
 		return this.type != null && this.type.isTypeNameVar(scope);
 	}
-	
+
 }
