@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2017 IBM Corporation and others.
+ * Copyright (c) 2000, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -36,9 +36,10 @@ public class SyntheticMethodBinding extends MethodBinding {
 	public MethodBinding targetMethod;			// method or constructor
 	public TypeBinding targetEnumType; 			// enum type
 	public LambdaExpression lambda;
-	
+	public RecordComponentBinding recordComponentBinding;
+
 	/** Switch (one from many) linked to the switch table */
-	public SwitchStatement switchStatement; 
+	public SwitchStatement switchStatement;
 	/**
 	 * Method reference expression whose target FI is Serializable. Should be set when
 	 * purpose is {@link #SerializableMethodReference}
@@ -72,13 +73,15 @@ public class SyntheticMethodBinding extends MethodBinding {
      * Is never directly materialized in bytecode
      */
     public static final int SerializableMethodReference = 18;
-    
+    public static final int RecordOverrideToString = 19;
+    public static final int RecordOverrideHashCode = 20;
+    public static final int RecordOverrideEquals = 21;
+
 	public int sourceStart = 0; // start position of the matching declaration
 	public int index; // used for sorting access methods in the class file
 	public int fakePaddedParameters = 0; // added in synthetic constructor to avoid name clash.
 
 	public SyntheticMethodBinding(FieldBinding targetField, boolean isReadAccess, boolean isSuperAccess, ReferenceBinding declaringClass) {
-
 		this.modifiers = ClassFileConstants.AccDefault | ClassFileConstants.AccStatic | ClassFileConstants.AccSynthetic;
 		this.tagBits |= (TagBits.AnnotationResolved | TagBits.DeprecatedAnnotationResolved);
 		SourceTypeBinding declaringSourceType = (SourceTypeBinding) declaringClass;
@@ -151,7 +154,6 @@ public class SyntheticMethodBinding extends MethodBinding {
 				setSelector(CharOperation.concat(TypeConstants.SYNTHETIC_ACCESS_METHOD_PREFIX, String.valueOf(++methodId).toCharArray()));
 			}
 		} while (needRename);
-
 		// retrieve sourceStart position for the target field for line number attributes
 		FieldDeclaration[] fieldDecls = declaringSourceType.scope.referenceContext.fields;
 		if (fieldDecls != null) {
@@ -302,7 +304,7 @@ public class SyntheticMethodBinding extends MethodBinding {
 			this.modifiers |= ClassFileConstants.AccStrictfp;
 		}
 	}
-	
+
 	/**
 	 * Construct $deserializeLambda$ method
 	 */
@@ -319,7 +321,7 @@ public class SyntheticMethodBinding extends MethodBinding {
 		int methodId = knownAccessMethods == null ? 0 : knownAccessMethods.length;
 		this.index = methodId;
 	}
-	
+
 	/**
 	 * Construct enum special methods: values or valueOf methods
 	 */
@@ -329,7 +331,7 @@ public class SyntheticMethodBinding extends MethodBinding {
 		this.index = knownAccessMethods == null ? 0 : knownAccessMethods.length;
 		StringBuffer buffer = new StringBuffer();
 		buffer.append(TypeConstants.SYNTHETIC_ENUM_CONSTANT_INITIALIZATION_METHOD_PREFIX).append(this.index);
-		this.selector = String.valueOf(buffer).toCharArray(); 
+		this.selector = String.valueOf(buffer).toCharArray();
 		this.modifiers = ClassFileConstants.AccPrivate | ClassFileConstants.AccStatic;
 		this.tagBits |= (TagBits.AnnotationResolved | TagBits.DeprecatedAnnotationResolved);
 		this.purpose = SyntheticMethodBinding.TooManyEnumsConstants;
@@ -423,7 +425,7 @@ public class SyntheticMethodBinding extends MethodBinding {
 	    this.modifiers = ClassFileConstants.AccSynthetic | ClassFileConstants.AccPrivate | ClassFileConstants.AccStatic;
 		this.tagBits |= (TagBits.AnnotationResolved | TagBits.DeprecatedAnnotationResolved);
 	    this.returnType = publicConstructor.declaringClass;
-	
+
 	    int realParametersLength = privateConstructor.parameters.length;
 	    int enclosingInstancesLength = enclosingInstances.length;
 	    int parametersLength =  enclosingInstancesLength + realParametersLength;
@@ -431,7 +433,7 @@ public class SyntheticMethodBinding extends MethodBinding {
 	    System.arraycopy(enclosingInstances, 0, this.parameters, 0, enclosingInstancesLength);
 	    System.arraycopy(privateConstructor.parameters, 0, this.parameters, enclosingInstancesLength, realParametersLength);
 	    this.fakePaddedParameters = publicConstructor.parameters.length - realParametersLength;
-	    
+
 	    this.thrownExceptions = publicConstructor.thrownExceptions;
 	    this.purpose = SyntheticMethodBinding.FactoryMethod;
 	    this.targetMethod = publicConstructor;
@@ -440,6 +442,54 @@ public class SyntheticMethodBinding extends MethodBinding {
 		this.index = methodId;
 	}
 
+	public SyntheticMethodBinding(ReferenceBinding declaringClass, RecordComponentBinding rcb, int index) {
+		SourceTypeBinding declaringSourceType = (SourceTypeBinding) declaringClass;
+		assert declaringSourceType.isRecord();
+		this.declaringClass = declaringSourceType;
+		this.modifiers = ClassFileConstants.AccPublic;
+		// rcb not resolved fully yet - to be filled in later - see STB.components()
+//		if (rcb.type instanceof TypeVariableBinding ||
+//				rcb.type instanceof ParameterizedTypeBinding)
+//			this.modifiers |= ExtraCompilerModifiers.AccGenericSignature;
+		if (this.declaringClass.isStrictfp())
+			this.modifiers |= ClassFileConstants.AccStrictfp;
+		this.tagBits |= (TagBits.AnnotationResolved | TagBits.DeprecatedAnnotationResolved);
+		this.parameters = Binding.NO_PARAMETERS;
+//		this.returnType = rcb.type; Not resolved yet - to be filled in later
+		this.selector = rcb.name;
+		this.recordComponentBinding = rcb;
+//		this.targetReadField = ??; // not fully resolved yet - to be filled in later
+		this.purpose = SyntheticMethodBinding.FieldReadAccess;
+		this.thrownExceptions = Binding.NO_EXCEPTIONS;
+		this.declaringClass = declaringSourceType;
+		this.index = index;
+		this.sourceStart = rcb.sourceRecordComponent().sourceStart;
+	}
+	public SyntheticMethodBinding(ReferenceBinding declaringClass, char[] selector, int index) {
+		SourceTypeBinding declaringSourceType = (SourceTypeBinding) declaringClass;
+		assert declaringSourceType.isRecord();
+		this.declaringClass = declaringSourceType;
+		this.modifiers = ClassFileConstants.AccPublic | ClassFileConstants.AccFinal;
+		if (this.declaringClass.isStrictfp())
+				this.modifiers |= ClassFileConstants.AccStrictfp;
+		this.tagBits |= (TagBits.AnnotationResolved | TagBits.DeprecatedAnnotationResolved);
+	    this.selector = selector;
+	    this.thrownExceptions = Binding.NO_EXCEPTIONS;
+		if (selector == TypeConstants.TOSTRING) {
+			this.returnType = declaringSourceType.scope.getJavaLangString();
+		    this.parameters = Binding.NO_PARAMETERS;
+		    this.purpose = SyntheticMethodBinding.RecordOverrideToString;
+		} else if (selector == TypeConstants.HASHCODE) {
+			this.returnType = TypeBinding.INT;
+		    this.parameters = Binding.NO_PARAMETERS;
+		    this.purpose = SyntheticMethodBinding.RecordOverrideHashCode;
+		} else if (selector == TypeConstants.EQUALS) {
+			this.returnType = TypeBinding.BOOLEAN;
+		    this.parameters = new TypeBinding[] {declaringSourceType.scope.getJavaLangObject()};
+		    this.purpose = SyntheticMethodBinding.RecordOverrideEquals;
+		}
+		this.index = index;
+	}
 	/**
 	 * An constructor accessor is a constructor with an extra argument (declaringClass), in case of
 	 * collision with an existing constructor, then add again an extra argument (declaringClass again).
@@ -597,7 +647,7 @@ public class SyntheticMethodBinding extends MethodBinding {
 	protected boolean isConstructorRelated() {
 		return this.purpose == SyntheticMethodBinding.ConstructorAccess;
 	}
-	
+
 	@Override
 	public LambdaExpression sourceLambda() {
 		return this.lambda;
