@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2013 IBM Corporation and others.
+ * Copyright (c) 2000, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -17,7 +17,24 @@ import java.util.List;
 
 import org.eclipse.jdt.core.compiler.CharOperation;
 import org.eclipse.jdt.core.compiler.InvalidInputException;
-import org.eclipse.jdt.internal.compiler.ast.*;
+import org.eclipse.jdt.internal.compiler.ast.ASTNode;
+import org.eclipse.jdt.internal.compiler.ast.Expression;
+import org.eclipse.jdt.internal.compiler.ast.IJavadocTypeReference;
+import org.eclipse.jdt.internal.compiler.ast.Javadoc;
+import org.eclipse.jdt.internal.compiler.ast.JavadocAllocationExpression;
+import org.eclipse.jdt.internal.compiler.ast.JavadocArgumentExpression;
+import org.eclipse.jdt.internal.compiler.ast.JavadocArrayQualifiedTypeReference;
+import org.eclipse.jdt.internal.compiler.ast.JavadocArraySingleTypeReference;
+import org.eclipse.jdt.internal.compiler.ast.JavadocFieldReference;
+import org.eclipse.jdt.internal.compiler.ast.JavadocImplicitTypeReference;
+import org.eclipse.jdt.internal.compiler.ast.JavadocMessageSend;
+import org.eclipse.jdt.internal.compiler.ast.JavadocModuleReference;
+import org.eclipse.jdt.internal.compiler.ast.JavadocQualifiedTypeReference;
+import org.eclipse.jdt.internal.compiler.ast.JavadocReturnStatement;
+import org.eclipse.jdt.internal.compiler.ast.JavadocSingleNameReference;
+import org.eclipse.jdt.internal.compiler.ast.JavadocSingleTypeReference;
+import org.eclipse.jdt.internal.compiler.ast.TypeDeclaration;
+import org.eclipse.jdt.internal.compiler.ast.TypeReference;
 import org.eclipse.jdt.internal.compiler.classfmt.ClassFileConstants;
 import org.eclipse.jdt.internal.compiler.util.Util;
 
@@ -25,6 +42,11 @@ import org.eclipse.jdt.internal.compiler.util.Util;
  * Parser specialized for decoding javadoc comments
  */
 public class JavadocParser extends AbstractCommentParser {
+	private static final JavadocSingleNameReference[] NO_SINGLE_NAME_REFERENCE = new JavadocSingleNameReference[0];
+	private static final JavadocSingleTypeReference[] NO_SINGLE_TYPE_REFERENCE = new JavadocSingleTypeReference[0];
+	private static final JavadocQualifiedTypeReference[] NO_QUALIFIED_TYPE_REFERENCE = new JavadocQualifiedTypeReference[0];
+	private static final TypeReference[] NO_TYPE_REFERENCE = new TypeReference[0];
+	private static final Expression[] NO_EXPRESSION = new Expression[0];
 
 	// Public fields
 	public Javadoc docComment;
@@ -41,7 +63,7 @@ public class JavadocParser extends AbstractCommentParser {
 	// returns whether this JavadocParser should report errors or not (overrides reportProblems)
 	// see "https://bugs.eclipse.org/bugs/show_bug.cgi?id=192449"
 	public boolean shouldReportProblems = true;
-	
+
 	// flag to let the parser know that the current tag is waiting for a description
 	// see "https://bugs.eclipse.org/bugs/show_bug.cgi?id=222900"
 	private int tagWaitingForDescription;
@@ -182,14 +204,24 @@ public class JavadocParser extends AbstractCommentParser {
 	protected Object createFieldReference(Object receiver) throws InvalidInputException {
 		try {
 			// Get receiver type
-			TypeReference typeRef = (TypeReference) receiver;
+			TypeReference typeRef = null;
+			boolean useReceiver = false;
+			if (receiver instanceof JavadocModuleReference) {
+				JavadocModuleReference jRef = (JavadocModuleReference)receiver;
+				if (jRef.typeReference != null) {
+					typeRef = jRef.typeReference;
+					useReceiver =  true;
+				}
+			} else {
+				typeRef = (TypeReference) receiver;
+			}
 			if (typeRef == null) {
 				char[] name = this.sourceParser.compilationUnit.getMainTypeName();
 				typeRef = new JavadocImplicitTypeReference(name, this.memberStart);
 			}
 			// Create field
 			JavadocFieldReference field = new JavadocFieldReference(this.identifierStack[0], this.identifierPositionStack[0]);
-			field.receiver = typeRef;
+			field.receiver = useReceiver ? (Expression)receiver : typeRef;
 			field.tagSourceStart = this.tagSourceStart;
 			field.tagSourceEnd = this.tagSourceEnd;
 			field.tagValue = this.tagValue;
@@ -204,7 +236,15 @@ public class JavadocParser extends AbstractCommentParser {
 	protected Object createMethodReference(Object receiver, List arguments) throws InvalidInputException {
 		try {
 			// Get receiver type
-			TypeReference typeRef = (TypeReference) receiver;
+			TypeReference typeRef = null;
+			if (receiver instanceof JavadocModuleReference) {
+				JavadocModuleReference jRef = (JavadocModuleReference)receiver;
+				if (jRef.typeReference != null) {
+					typeRef = jRef.typeReference;
+				}
+			} else {
+				typeRef = (TypeReference) receiver;
+			}
 			// Decide whether we have a constructor or not
 			boolean isConstructor = false;
 			int length = this.identifierLengthStack[0];	// may be > 1 for member class constructor reference
@@ -326,6 +366,43 @@ public class JavadocParser extends AbstractCommentParser {
 		return typeRef;
 	}
 
+	protected JavadocModuleReference createModuleReference(int moduleRefTokenCount) {
+		JavadocModuleReference moduleRef = null;
+		char[][] tokens = new char[moduleRefTokenCount][];
+		System.arraycopy(this.identifierStack, 0, tokens, 0, moduleRefTokenCount);
+		long[] positions = new long[moduleRefTokenCount];
+		System.arraycopy(this.identifierPositionStack, 0, positions, 0, moduleRefTokenCount);
+		moduleRef = new JavadocModuleReference(tokens, positions, this.tagSourceStart, this.tagSourceEnd);
+		return moduleRef;
+	}
+
+	@Override
+	protected Object createModuleTypeReference(int primitiveToken, int moduleRefTokenCount) {
+		JavadocModuleReference moduleRef= createModuleReference(moduleRefTokenCount);
+
+		TypeReference typeRef = null;
+		int size = this.identifierLengthStack[this.identifierLengthPtr];
+		int newSize= size-moduleRefTokenCount;
+		if (newSize == 1) { // Single Type ref
+			typeRef = new JavadocSingleTypeReference(
+						this.identifierStack[this.identifierPtr],
+						this.identifierPositionStack[this.identifierPtr],
+						this.tagSourceStart,
+						this.tagSourceEnd);
+		} else if (newSize > 1) { // Qualified Type ref
+			char[][] tokens = new char[newSize][];
+			System.arraycopy(this.identifierStack, this.identifierPtr - newSize + 1, tokens, 0, newSize);
+			long[] positions = new long[newSize];
+			System.arraycopy(this.identifierPositionStack, this.identifierPtr - newSize + 1, positions, 0, newSize);
+			typeRef = new JavadocQualifiedTypeReference(tokens, positions, this.tagSourceStart, this.tagSourceEnd);
+		} else {
+			this.lastIdentifierEndPosition++;
+		}
+
+		moduleRef.setTypeReference(typeRef);
+		return moduleRef;
+	}
+
 	/*
 	 * Get current parsed type declaration.
 	 */
@@ -463,7 +540,7 @@ public class JavadocParser extends AbstractCommentParser {
 				}
 				break;
 		}
-		
+
 		// Read tag name
 		char[] tagName = new char[32];
 		int length = 0;
@@ -519,6 +596,9 @@ public class JavadocParser extends AbstractCommentParser {
 				if (length == TAG_AUTHOR_LENGTH && CharOperation.equals(TAG_AUTHOR, tagName, 0, length)) {
 					this.tagValue = TAG_AUTHOR_VALUE;
 					this.tagWaitingForDescription = this.tagValue;
+				}else if (length == TAG_API_NOTE_LENGTH && CharOperation.equals(TAG_API_NOTE, tagName, 0, length)) {
+					this.tagValue = TAG_API_NOTE_VALUE;
+					this.tagWaitingForDescription = this.tagValue;
 				}
 				break;
 			case 'c':
@@ -553,8 +633,18 @@ public class JavadocParser extends AbstractCommentParser {
 					}
 				}
 				break;
+			case 'h':
+				if (length == TAG_HIDDEN_LENGTH && CharOperation.equals(TAG_HIDDEN, tagName, 0, length)) {
+					valid = true;
+					this.tagValue = TAG_HIDDEN_VALUE;
+				}
+				break;
 			case 'i':
-				if (length == TAG_INHERITDOC_LENGTH && CharOperation.equals(TAG_INHERITDOC, tagName, 0, length)) {
+				if (length == TAG_INDEX_LENGTH && CharOperation.equals(TAG_INDEX, tagName, 0, length)) {
+					valid = true;
+					this.tagValue = TAG_INDEX_VALUE;
+					this.tagWaitingForDescription = this.tagValue;
+				} else if (length == TAG_INHERITDOC_LENGTH && CharOperation.equals(TAG_INHERITDOC, tagName, 0, length)) {
 					// https://bugs.eclipse.org/bugs/show_bug.cgi?id=247037, @inheritDoc usage is illegal
 					// outside of few block tags and the main description.
 					switch (this.lastBlockTagValue) {
@@ -580,19 +670,25 @@ public class JavadocParser extends AbstractCommentParser {
 							}
 					}
 					this.tagValue = TAG_INHERITDOC_VALUE;
+				} else if (length == TAG_IMPL_SPEC_LENGTH && CharOperation.equals(TAG_IMPL_SPEC, tagName, 0, length)) {
+					this.tagValue = TAG_IMPL_SPEC_VALUE;
+					this.tagWaitingForDescription = this.tagValue;
+				} else if (length == TAG_IMPL_NOTE_LENGTH && CharOperation.equals(TAG_IMPL_NOTE, tagName, 0, length)) {
+					this.tagValue = TAG_IMPL_NOTE_VALUE;
+					this.tagWaitingForDescription = this.tagValue;
 				}
 				break;
 			case 'l':
 				if (length == TAG_LINK_LENGTH && CharOperation.equals(TAG_LINK, tagName, 0, length)) {
 					this.tagValue = TAG_LINK_VALUE;
 					if (this.inlineTagStarted || (this.kind & COMPLETION_PARSER) != 0) {
-						valid= parseReference();
+						valid= parseReference(true);
 					}
 				} else if (length == TAG_LINKPLAIN_LENGTH && CharOperation.equals(TAG_LINKPLAIN, tagName, 0, length)) {
 					this.tagValue = TAG_LINKPLAIN_VALUE;
 					if (this.inlineTagStarted) {
-						valid = parseReference();
-					} 
+						valid = parseReference(true);
+					}
 				} else if (length == TAG_LITERAL_LENGTH && this.inlineTagStarted && CharOperation.equals(TAG_LITERAL, tagName, 0, length)) {
 					this.tagValue = TAG_LITERAL_VALUE;
 					this.tagWaitingForDescription = this.tagValue;
@@ -604,6 +700,11 @@ public class JavadocParser extends AbstractCommentParser {
 					if (!this.inlineTagStarted) {
 						valid = parseParam();
 					}
+				} else if (length == TAG_PROVIDES_LENGTH && CharOperation.equals(TAG_PROVIDES, tagName, 0, length)) {
+					this.tagValue = TAG_PROVIDES_VALUE;
+					if (!this.inlineTagStarted) {
+						valid = parseProvidesReference();
+					}
 				}
 				break;
 			case 'r':
@@ -618,7 +719,7 @@ public class JavadocParser extends AbstractCommentParser {
 				if (length == TAG_SEE_LENGTH && CharOperation.equals(TAG_SEE, tagName, 0, length)) {
 					this.tagValue = TAG_SEE_VALUE;
 					if (!this.inlineTagStarted) {
-						valid = parseReference();
+						valid = parseReference(true);
 					}
 				} else if (length == TAG_SERIAL_LENGTH && CharOperation.equals(TAG_SERIAL, tagName, 0, length)) {
 					this.tagValue = TAG_SERIAL_VALUE;
@@ -632,7 +733,13 @@ public class JavadocParser extends AbstractCommentParser {
 				} else if (length == TAG_SINCE_LENGTH && CharOperation.equals(TAG_SINCE, tagName, 0, length)) {
 					this.tagValue = TAG_SINCE_VALUE;
 					this.tagWaitingForDescription = this.tagValue;
-				}					
+				} else if (length == TAG_SYSTEM_PROPERTY_LENGTH && CharOperation.equals(TAG_SYSTEM_PROPERTY, tagName, 0, length)) {
+					this.tagValue = TAG_SYSTEM_PROPERTY_VALUE;
+					this.tagWaitingForDescription = this.tagValue;
+				} else if (length == TAG_SUMMARY_LENGTH && CharOperation.equals(TAG_SUMMARY, tagName, 0, length)) {
+					this.tagValue = TAG_SUMMARY_VALUE;
+					this.tagWaitingForDescription = this.tagValue;
+				}
 				break;
 			case 't':
 				if (length == TAG_THROWS_LENGTH && CharOperation.equals(TAG_THROWS, tagName, 0, length)) {
@@ -642,6 +749,14 @@ public class JavadocParser extends AbstractCommentParser {
 					}
 				}
 				break;
+			case 'u':
+				if (length == TAG_USES_LENGTH && CharOperation.equals(TAG_USES, tagName, 0, length)) {
+					this.tagValue = TAG_USES_VALUE;
+					if (!this.inlineTagStarted) {
+						valid = parseUsesReference();
+					}
+				}
+				break;
 			case 'v':
 				if (length == TAG_VALUE_LENGTH && CharOperation.equals(TAG_VALUE, tagName, 0, length)) {
 					this.tagValue = TAG_VALUE_VALUE;
@@ -896,7 +1011,7 @@ public class JavadocParser extends AbstractCommentParser {
 		// Set positions
 		if (this.inheritedPositions != null && this.inheritedPositionsPtr != this.inheritedPositions.length) {
 			// Compact array by shrinking.
-			System.arraycopy(this.inheritedPositions, 0, 
+			System.arraycopy(this.inheritedPositions, 0,
 					this.inheritedPositions = new long[this.inheritedPositionsPtr], 0, this.inheritedPositionsPtr);
 		}
 		this.docComment.inheritedPositions = this.inheritedPositions;
@@ -913,6 +1028,18 @@ public class JavadocParser extends AbstractCommentParser {
 			System.arraycopy(this.invalidParamReferencesStack, 0, this.docComment.invalidParameters, 0, this.invalidParamReferencesPtr+1);
 		}
 
+		this.docComment.usesReferences = this.usesReferencesPtr >= 0 ? new IJavadocTypeReference[this.usesReferencesPtr+1] : NO_QUALIFIED_TYPE_REFERENCE;
+		for (int i = 0; i <= this.usesReferencesPtr; ++i) {
+			TypeReference ref = this.usesReferencesStack[i];
+			this.docComment.usesReferences[i] = (IJavadocTypeReference)ref;
+		}
+
+		this.docComment.providesReferences = this.providesReferencesPtr >= 0 ? new IJavadocTypeReference[this.providesReferencesPtr+1] : NO_QUALIFIED_TYPE_REFERENCE;
+		for (int i = 0; i <= this.providesReferencesPtr; ++i) {
+			TypeReference ref = this.providesReferencesStack[i];
+			this.docComment.providesReferences[i] = (IJavadocTypeReference)ref;
+		}
+
 		// If no nodes stored return
 		if (this.astLengthPtr == -1) {
 			return;
@@ -923,12 +1050,12 @@ public class JavadocParser extends AbstractCommentParser {
 		for (int i=0; i<=this.astLengthPtr; i++) {
 			sizes[i%ORDERED_TAGS_NUMBER] += this.astLengthStack[i];
 		}
-		this.docComment.seeReferences = new Expression[sizes[SEE_TAG_EXPECTED_ORDER]];
-		this.docComment.exceptionReferences = new TypeReference[sizes[THROWS_TAG_EXPECTED_ORDER]];
+		this.docComment.seeReferences = sizes[SEE_TAG_EXPECTED_ORDER] > 0 ? new Expression[sizes[SEE_TAG_EXPECTED_ORDER]] : NO_EXPRESSION;
+		this.docComment.exceptionReferences = sizes[THROWS_TAG_EXPECTED_ORDER] > 0 ? new TypeReference[sizes[THROWS_TAG_EXPECTED_ORDER]] : NO_TYPE_REFERENCE;
 		int paramRefPtr = sizes[PARAM_TAG_EXPECTED_ORDER];
-		this.docComment.paramReferences = new JavadocSingleNameReference[paramRefPtr];
+		this.docComment.paramReferences = paramRefPtr > 0 ? new JavadocSingleNameReference[paramRefPtr] : NO_SINGLE_NAME_REFERENCE;
 		int paramTypeParamPtr = sizes[PARAM_TAG_EXPECTED_ORDER];
-		this.docComment.paramTypeParameters = new JavadocSingleTypeReference[paramTypeParamPtr];
+		this.docComment.paramTypeParameters = paramTypeParamPtr > 0 ? new JavadocSingleTypeReference[paramTypeParamPtr] : NO_SINGLE_TYPE_REFERENCE;
 
 		// Store nodes in arrays
 		while (this.astLengthPtr >= 0) {
@@ -975,4 +1102,76 @@ public class JavadocParser extends AbstractCommentParser {
 			System.arraycopy(this.docComment.paramTypeParameters, paramTypeParamPtr, this.docComment.paramTypeParameters = new JavadocSingleTypeReference[size - paramTypeParamPtr], 0, size - paramTypeParamPtr);
 		}
 	}
+
+	/*
+	 * Parse @uses tag declaration
+	 */
+	protected boolean parseUsesReference() {
+		int start = this.scanner.currentPosition;
+		try {
+			Object typeRef = parseQualifiedName(true);
+			if (this.abort) return false; // May be aborted by specialized parser
+			if (typeRef == null) {
+				if (this.reportProblems)
+					this.sourceParser.problemReporter().javadocMissingUsesClassName(this.tagSourceStart, this.tagSourceEnd, this.sourceParser.modifiers);
+			} else {
+				return pushUsesReference(typeRef);
+			}
+		} catch (InvalidInputException ex) {
+			if (this.reportProblems) this.sourceParser.problemReporter().javadocInvalidUsesClass(start, getTokenEndPosition());
+		}
+		return false;
+	}
+
+	protected boolean pushUsesReference(Object typeRef) {
+		if (this.usesReferencesPtr == -1l) {
+			this.usesReferencesStack = new TypeReference[10];
+		}
+		int stackLength = this.usesReferencesStack.length;
+		if (++this.usesReferencesPtr >= stackLength) {
+			System.arraycopy(
+				this.usesReferencesStack, 0,
+				this.usesReferencesStack = new TypeReference[stackLength + AST_STACK_INCREMENT], 0,
+				stackLength);
+		}
+		this.usesReferencesStack[this.usesReferencesPtr] = (TypeReference)typeRef;
+		return true;
+	}
+
+	/*
+	 * Parse @uses tag declaration
+	 */
+	protected boolean parseProvidesReference() {
+		int start = this.scanner.currentPosition;
+		try {
+			Object typeRef = parseQualifiedName(true);
+			if (this.abort) return false; // May be aborted by specialized parser
+			if (typeRef == null) {
+				if (this.reportProblems)
+					this.sourceParser.problemReporter().javadocMissingProvidesClassName(this.tagSourceStart, this.tagSourceEnd, this.sourceParser.modifiers);
+			} else {
+				return pushProvidesReference(typeRef);
+			}
+		} catch (InvalidInputException ex) {
+			if (this.reportProblems) this.sourceParser.problemReporter().javadocInvalidProvidesClass(start, getTokenEndPosition());
+		}
+		return false;
+	}
+
+	protected boolean pushProvidesReference(Object typeRef) {
+		if (this.providesReferencesPtr == -1l) {
+			this.providesReferencesStack = new TypeReference[10];
+		}
+		int stackLength = this.providesReferencesStack.length;
+		if (++this.providesReferencesPtr >= stackLength) {
+			System.arraycopy(
+				this.providesReferencesStack, 0,
+				this.providesReferencesStack = new TypeReference[stackLength + AST_STACK_INCREMENT], 0,
+				stackLength);
+		}
+		this.providesReferencesStack[this.providesReferencesPtr] = (TypeReference)typeRef;
+		return true;
+
+	}
+
 }
