@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2019 IBM Corporation and others.
+ * Copyright (c) 2000, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -24,6 +24,7 @@ import org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
 import org.eclipse.jdt.internal.compiler.lookup.ParameterizedGenericMethodBinding;
 import org.eclipse.jdt.internal.compiler.lookup.RawTypeBinding;
 import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
+import org.eclipse.jdt.internal.compiler.lookup.SyntheticMethodBinding;
 import org.eclipse.jdt.internal.compiler.lookup.TagBits;
 import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
 import org.eclipse.jdt.internal.compiler.lookup.TypeVariableBinding;
@@ -72,6 +73,22 @@ class MethodBinding implements IMethodBinding {
 		return this.binding.isConstructor();
 	}
 
+	/**
+	 * @see IMethodBinding#isCompactConstructor()
+	 */
+	@Override
+	public boolean isCompactConstructor() {
+		return this.binding.isCompactConstructor();
+	}
+
+	/**
+	 * @see IMethodBinding#isCanonicalConstructor()
+	 */
+	@Override
+	public boolean isCanonicalConstructor() {
+		return ((this.binding.tagBits & TagBits.IsCanonicalConstructor) != 0);
+	}
+
 	/**
 	 * @see IMethodBinding#isDefaultConstructor()
 	 * @since 3.0
@@ -232,7 +249,7 @@ class MethodBinding implements IMethodBinding {
 						((metaTagBits & TagBits.AnnotationTargetMASK) != 0)) {
 					continue;
 				}
-				
+
 				final IAnnotationBinding annotationInstance = this.resolver.getAnnotationInstance(internalAnnotation);
 				if (annotationInstance == null) {
 					continue;
@@ -241,7 +258,7 @@ class MethodBinding implements IMethodBinding {
 			}
 			if (convertedAnnotationCount == length) return tempAnnotations;
 			if (convertedAnnotationCount == 0) return AnnotationBinding.NoAnnotations;
-			
+
 			System.arraycopy(tempAnnotations, 0, (tempAnnotations = new IAnnotationBinding[convertedAnnotationCount]), 0, convertedAnnotationCount);
 			return tempAnnotations;
 		}
@@ -591,4 +608,10 @@ class MethodBinding implements IMethodBinding {
 	public IVariableBinding[] getSyntheticOuterLocals() {
 		return NO_VARIABLE_BINDINGS;
 	}
+
+	@Override
+	public boolean isSyntheticRecordMethod() {
+		return ((getDeclaringClass().isRecord()) &&
+				(this.binding instanceof SyntheticMethodBinding));
+	}
 }
\ No newline at end of file
