@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2019 IBM Corporation and others.
+ * Copyright (c) 2000, 2020 IBM Corporation and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -18,6 +18,7 @@ import java.util.List;
 
 import org.eclipse.jdt.core.dom.*;
 import org.eclipse.jdt.internal.compiler.parser.ScannerHelper;
+import org.eclipse.jdt.internal.core.dom.util.DOMASTUtil;
 
 /**
  * Internal AST visitor for serializing an AST in a quick and dirty fashion.
@@ -49,7 +50,7 @@ public class NaiveASTFlattener extends ASTVisitor {
 	 * @since 3.4
 	 */
 	private static final int JLS2 = AST.JLS2;
-	
+
 	/**
 	 * Internal synonym for {@link AST#JLS3}. Use to alleviate
 	 * deprecation warnings.
@@ -81,14 +82,14 @@ public class NaiveASTFlattener extends ASTVisitor {
 	 * @since 3.14
 	 */
 	private static final int JLS9 = AST.JLS9;
-	
+
 	/**
-	 * Internal synonym for {@link AST#JLS12}. Use to alleviate
+	 * Internal synonym for {@link AST#JLS14}. Use to alleviate
 	 * deprecation warnings.
-	 * @since 3.18
+	 * @deprecated
+	 * @since 3.22
 	 */
-	private static final int JLS12 = AST.JLS12;
-	
+	private static final int JLS14 = AST.JLS14;
 
 	/**
 	 * The string buffer into which the serialized representation of the AST is
@@ -209,6 +210,12 @@ public class NaiveASTFlattener extends ASTVisitor {
 		if (Modifier.isTransient(modifiers)) {
 			this.buffer.append("transient ");//$NON-NLS-1$
 		}
+		if (Modifier.isSealed(modifiers)) {
+			this.buffer.append("sealed ");//$NON-NLS-1$
+		}
+		if (Modifier.isNonSealed(modifiers)) {
+			this.buffer.append("non-sealed ");//$NON-NLS-1$
+		}
 	}
 
 	/**
@@ -259,7 +266,7 @@ public class NaiveASTFlattener extends ASTVisitor {
 			this.buffer.append('>');
 		}
 	}
-	
+
 	private void visitTypeAnnotations(AnnotatableType node) {
 		if (node.getAST().apiLevel() >= JLS8) {
 			visitAnnotationsList(node.annotations());
@@ -273,7 +280,7 @@ public class NaiveASTFlattener extends ASTVisitor {
 			this.buffer.append(' ');
 		}
 	}
-	
+
 	/**
 	 * Resets this printer so that it can be used again.
 	 */
@@ -460,23 +467,12 @@ public class NaiveASTFlattener extends ASTVisitor {
 
 	@Override
 	public boolean visit(BreakStatement node) {
-		if (node.getAST().apiLevel() >= JLS12 && node.isImplicit()  && node.getExpression() == null) {
-			return false;
-		}
 		printIndent();
-		if (node.getAST().apiLevel() < JLS12 || (node.getAST().apiLevel() >= JLS12 && !node.isImplicit())) {
-			this.buffer.append("break"); //$NON-NLS-1$
-		}
+		this.buffer.append("break");//$NON-NLS-1$
 		if (node.getLabel() != null) {
 			this.buffer.append(" ");//$NON-NLS-1$
 			node.getLabel().accept(this);
 		}
-		if (node.getAST().apiLevel() >= JLS12) {
-			if (node.getExpression() != null) {
-				this.buffer.append(" ");//$NON-NLS-1$
-				node.getExpression().accept(this);
-			}
-		}
 		this.buffer.append(";\n");//$NON-NLS-1$
 		return false;
 	}
@@ -614,7 +610,7 @@ public class NaiveASTFlattener extends ASTVisitor {
 		this.buffer.append(";\n");//$NON-NLS-1$
 		return false;
 	}
-	
+
 	@Override
 	public boolean visit(CreationReference node) {
 		node.getType().accept(this);
@@ -879,6 +875,10 @@ public class NaiveASTFlattener extends ASTVisitor {
 		node.getLeftOperand().accept(this);
 		this.buffer.append(" instanceof ");//$NON-NLS-1$
 		node.getRightOperand().accept(this);
+		if (DOMASTUtil.isInstanceofExpressionPatternSupported(node.getAST()) && node.getPatternVariable()!= null) {
+			this.buffer.append(" "); //$NON-NLS-1$
+			node.getPatternVariable().accept(this);
+		}
 		return false;
 	}
 
@@ -988,7 +988,7 @@ public class NaiveASTFlattener extends ASTVisitor {
 				this.buffer.append(">");//$NON-NLS-1$
 			}
 		}
-		if (!node.isConstructor()) {
+		if (!node.isConstructor()){
 			if (node.getAST().apiLevel() == JLS2) {
 				getReturnType(node).accept(this);
 			} else {
@@ -1002,31 +1002,33 @@ public class NaiveASTFlattener extends ASTVisitor {
 			this.buffer.append(" ");//$NON-NLS-1$
 		}
 		node.getName().accept(this);
-		this.buffer.append("(");//$NON-NLS-1$
-		if (node.getAST().apiLevel() >= JLS8) {
-			Type receiverType = node.getReceiverType();
-			if (receiverType != null) {
-				receiverType.accept(this);
-				this.buffer.append(' ');
-				SimpleName qualifier = node.getReceiverQualifier();
-				if (qualifier != null) {
-					qualifier.accept(this);
-					this.buffer.append('.');
-				}
-				this.buffer.append("this"); //$NON-NLS-1$
-				if (node.parameters().size() > 0) {
-					this.buffer.append(',');
+		if (!(DOMASTUtil.isRecordDeclarationSupported(node.getAST()) && node.isCompactConstructor())) {
+			this.buffer.append("(");//$NON-NLS-1$
+			if (node.getAST().apiLevel() >= JLS8) {
+				Type receiverType = node.getReceiverType();
+				if (receiverType != null) {
+					receiverType.accept(this);
+					this.buffer.append(' ');
+					SimpleName qualifier = node.getReceiverQualifier();
+					if (qualifier != null) {
+						qualifier.accept(this);
+						this.buffer.append('.');
+					}
+					this.buffer.append("this"); //$NON-NLS-1$
+					if (node.parameters().size() > 0) {
+						this.buffer.append(',');
+					}
 				}
 			}
-		}
-		for (Iterator it = node.parameters().iterator(); it.hasNext(); ) {
-			SingleVariableDeclaration v = (SingleVariableDeclaration) it.next();
-			v.accept(this);
-			if (it.hasNext()) {
-				this.buffer.append(",");//$NON-NLS-1$
+			for (Iterator it = node.parameters().iterator(); it.hasNext(); ) {
+				SingleVariableDeclaration v = (SingleVariableDeclaration) it.next();
+				v.accept(this);
+				if (it.hasNext()) {
+					this.buffer.append(",");//$NON-NLS-1$
+				}
 			}
+			this.buffer.append(")");//$NON-NLS-1$
 		}
-		this.buffer.append(")");//$NON-NLS-1$
 		int size = node.getExtraDimensions();
 		if (node.getAST().apiLevel() >= JLS8) {
 			List dimensions = node.extraDimensions();
@@ -1047,11 +1049,11 @@ public class NaiveASTFlattener extends ASTVisitor {
 					if (it.hasNext()) {
 						this.buffer.append(", ");//$NON-NLS-1$
 					}
-				}				
+				}
 				this.buffer.append(" ");//$NON-NLS-1$
-			} 
+			}
 		} else {
-			if (!node.thrownExceptionTypes().isEmpty()) {				
+			if (!node.thrownExceptionTypes().isEmpty()) {
 				this.buffer.append(" throws ");//$NON-NLS-1$
 				for (Iterator it = node.thrownExceptionTypes().iterator(); it.hasNext(); ) {
 					Type n = (Type) it.next();
@@ -1059,8 +1061,8 @@ public class NaiveASTFlattener extends ASTVisitor {
 					if (it.hasNext()) {
 						this.buffer.append(", ");//$NON-NLS-1$
 					}
-				}	
-				this.buffer.append(" ");//$NON-NLS-1$				
+				}
+				this.buffer.append(" ");//$NON-NLS-1$
 			}
 		}
 		if (node.getBody() == null) {
@@ -1300,6 +1302,17 @@ public class NaiveASTFlattener extends ASTVisitor {
 		return false;
 	}
 
+	@Override
+	public boolean visit(ModuleQualifiedName node) {
+		node.getModuleQualifier().accept(this);
+		this.buffer.append("/");//$NON-NLS-1$
+		ASTNode cNode = node.getName();
+		if (cNode != null) {
+			cNode.accept(this);
+		}
+		return false;
+	}
+
 	@Override
 	public boolean visit(QualifiedName node) {
 		node.getQualifier().accept(this);
@@ -1317,6 +1330,62 @@ public class NaiveASTFlattener extends ASTVisitor {
 		return false;
 	}
 
+	@Override
+	public boolean visit(RecordDeclaration node) {
+		if (node.getJavadoc() != null) {
+			node.getJavadoc().accept(this);
+		}
+		printIndent();
+		printModifiers(node.modifiers());
+		this.buffer.append("record ");//$NON-NLS-1$
+		node.getName().accept(this);
+		this.buffer.append(" ");//$NON-NLS-1$
+
+		if (!node.typeParameters().isEmpty()) {
+			this.buffer.append("<");//$NON-NLS-1$
+			for (Iterator it = node.typeParameters().iterator(); it.hasNext(); ) {
+				TypeParameter t = (TypeParameter) it.next();
+				t.accept(this);
+				if (it.hasNext()) {
+					this.buffer.append(",");//$NON-NLS-1$
+				}
+			}
+			this.buffer.append(">");//$NON-NLS-1$
+		}
+		this.buffer.append(" ");//$NON-NLS-1$
+		this.buffer.append("(");//$NON-NLS-1$
+		for (Iterator it = node.recordComponents().iterator(); it.hasNext(); ) {
+			SingleVariableDeclaration v = (SingleVariableDeclaration) it.next();
+			v.accept(this);
+			if (it.hasNext()) {
+				this.buffer.append(",");//$NON-NLS-1$
+			}
+		}
+		this.buffer.append(")");//$NON-NLS-1$
+		if (!node.superInterfaceTypes().isEmpty()) {
+			this.buffer.append(" implements ");//$NON-NLS-1$
+			for (Iterator it = node.superInterfaceTypes().iterator(); it.hasNext(); ) {
+				Type t = (Type) it.next();
+				t.accept(this);
+				if (it.hasNext()) {
+					this.buffer.append(", ");//$NON-NLS-1$
+				}
+			}
+			this.buffer.append(" ");//$NON-NLS-1$
+		}
+		this.buffer.append("{");//$NON-NLS-1$
+		if (!node.bodyDeclarations().isEmpty()) {
+			this.buffer.append("\n");//$NON-NLS-1$
+			for (Iterator it = node.bodyDeclarations().iterator(); it.hasNext(); ) {
+				BodyDeclaration d = (BodyDeclaration) it.next();
+				d.accept(this);
+				// other body declarations include trailing punctuation
+			}
+		}
+		this.buffer.append("}\n");//$NON-NLS-1$
+		return false;
+	}
+
 	@Override
 	public boolean visit(RequiresDirective node) {
 		printIndent();
@@ -1489,7 +1558,7 @@ public class NaiveASTFlattener extends ASTVisitor {
 
 	/*
 	 * @see ASTVisitor#visit(SuperMethodReference)
-	 * 
+	 *
 	 * @since 3.10
 	 */
 	@Override
@@ -1506,7 +1575,7 @@ public class NaiveASTFlattener extends ASTVisitor {
 
 	@Override
 	public boolean visit(SwitchCase node) {
-		if (node.getAST().apiLevel() >= JLS12) {
+		if ((node.getAST().apiLevel() >= JLS14)) {
 			if (node.isDefault()) {
 				this.buffer.append("default");//$NON-NLS-1$
 				this.buffer.append(node.isSwitchLabeledRule() ? " ->" : ":");//$NON-NLS-1$ //$NON-NLS-2$
@@ -1565,14 +1634,14 @@ public class NaiveASTFlattener extends ASTVisitor {
 		this.indent--;
 		printIndent();
 		this.buffer.append("}\n");//$NON-NLS-1$
-			
+
 	}
 	@Override
 	public boolean visit(SwitchExpression node) {
 		visitSwitchNode(node);
 		return false;
 	}
-	
+
 	@Override
 	public boolean visit(SwitchStatement node) {
 		visitSwitchNode(node);
@@ -1631,6 +1700,12 @@ public class NaiveASTFlattener extends ASTVisitor {
 		return false;
 	}
 
+	@Override
+	public boolean visit(TextBlock node) {
+		this.buffer.append(node.getEscapedValue());
+		return false;
+	}
+
 	@Override
 	public boolean visit(TextElement node) {
 		this.buffer.append(node.getText());
@@ -1750,6 +1825,19 @@ public class NaiveASTFlattener extends ASTVisitor {
 				this.buffer.append(" ");//$NON-NLS-1$
 			}
 		}
+		if (DOMASTUtil.isFeatureSupportedinAST(node.getAST(), Modifier.SEALED)) {
+			if (!node.permittedTypes().isEmpty()) {
+				this.buffer.append("permits ");//$NON-NLS-1$
+				for (Iterator it = node.permittedTypes().iterator(); it.hasNext(); ) {
+					Type t = (Type) it.next();
+					t.accept(this);
+					if (it.hasNext()) {
+						this.buffer.append(", ");//$NON-NLS-1$
+					}
+				}
+				this.buffer.append(" ");//$NON-NLS-1$
+			}
+		}
 		this.buffer.append("{\n");//$NON-NLS-1$
 		this.indent++;
 		for (Iterator it = node.bodyDeclarations().iterator(); it.hasNext(); ) {
@@ -1782,7 +1870,7 @@ public class NaiveASTFlattener extends ASTVisitor {
 
 	/*
 	 * @see ASTVisitor#visit(TypeMethodReference)
-	 * 
+	 *
 	 * @since 3.10
 	 */
 	@Override
@@ -1923,6 +2011,21 @@ public class NaiveASTFlattener extends ASTVisitor {
 		return false;
 	}
 
+	@Override
+	public boolean visit(YieldStatement node) {
+		if ((node.getAST().apiLevel() >= JLS14) && node.isImplicit()  && node.getExpression() == null) {
+			return false;
+		}
+		printIndent();
+		this.buffer.append("yield"); //$NON-NLS-1$
+		if (node.getExpression() != null) {
+			this.buffer.append(" ");//$NON-NLS-1$
+			node.getExpression().accept(this);
+		}
+		this.buffer.append(";\n");//$NON-NLS-1$
+		return false;
+	}
+
 	/**
 	 * @deprecated
 	 */
