@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2014, 2019 Mateusz Matela and others.
+ * Copyright (c) 2014, 2020 Mateusz Matela and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -12,6 +12,7 @@
  *     Mateusz Matela <mateusz.matela@gmail.com> - [formatter] Formatter does not format Java code correctly, especially when max line width is set - https://bugs.eclipse.org/303519
  *     Mateusz Matela <mateusz.matela@gmail.com> - [formatter] IndexOutOfBoundsException in TokenManager - https://bugs.eclipse.org/462945
  *     Mateusz Matela <mateusz.matela@gmail.com> - [formatter] follow up bug for comments - https://bugs.eclipse.org/458208
+ *     IBM Corporation - DOM AST changes for JEP 354
  *******************************************************************************/
 package org.eclipse.jdt.internal.formatter;
 
@@ -22,13 +23,15 @@ import static org.eclipse.jdt.internal.compiler.parser.TerminalTokens.TokenNameC
 import static org.eclipse.jdt.internal.compiler.parser.TerminalTokens.TokenNameLBRACE;
 import static org.eclipse.jdt.internal.compiler.parser.TerminalTokens.TokenNameRBRACE;
 import static org.eclipse.jdt.internal.compiler.parser.TerminalTokens.TokenNameSEMICOLON;
+import static org.eclipse.jdt.internal.compiler.parser.TerminalTokens.TokenNameTextBlock;
 import static org.eclipse.jdt.internal.compiler.parser.TerminalTokens.TokenNameelse;
 import static org.eclipse.jdt.internal.compiler.parser.TerminalTokens.TokenNamefinally;
-import static org.eclipse.jdt.internal.compiler.parser.TerminalTokens.TokenNamepackage;
 import static org.eclipse.jdt.internal.compiler.parser.TerminalTokens.TokenNamewhile;
 
+import java.util.ArrayList;
 import java.util.List;
 
+import org.eclipse.jdt.core.dom.AST;
 import org.eclipse.jdt.core.dom.ASTNode;
 import org.eclipse.jdt.core.dom.ASTVisitor;
 import org.eclipse.jdt.core.dom.AbstractTypeDeclaration;
@@ -50,37 +53,46 @@ import org.eclipse.jdt.core.dom.EnumConstantDeclaration;
 import org.eclipse.jdt.core.dom.EnumDeclaration;
 import org.eclipse.jdt.core.dom.FieldDeclaration;
 import org.eclipse.jdt.core.dom.ForStatement;
+import org.eclipse.jdt.core.dom.IExtendedModifier;
 import org.eclipse.jdt.core.dom.IfStatement;
 import org.eclipse.jdt.core.dom.ImportDeclaration;
 import org.eclipse.jdt.core.dom.Initializer;
 import org.eclipse.jdt.core.dom.LabeledStatement;
 import org.eclipse.jdt.core.dom.LambdaExpression;
-import org.eclipse.jdt.core.dom.MarkerAnnotation;
 import org.eclipse.jdt.core.dom.MethodDeclaration;
-import org.eclipse.jdt.core.dom.Modifier;
 import org.eclipse.jdt.core.dom.ModuleDeclaration;
 import org.eclipse.jdt.core.dom.ModuleDirective;
-import org.eclipse.jdt.core.dom.NormalAnnotation;
 import org.eclipse.jdt.core.dom.PackageDeclaration;
+import org.eclipse.jdt.core.dom.RecordDeclaration;
 import org.eclipse.jdt.core.dom.ReturnStatement;
-import org.eclipse.jdt.core.dom.SingleMemberAnnotation;
 import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
 import org.eclipse.jdt.core.dom.Statement;
 import org.eclipse.jdt.core.dom.SwitchCase;
 import org.eclipse.jdt.core.dom.SwitchExpression;
 import org.eclipse.jdt.core.dom.SwitchStatement;
+import org.eclipse.jdt.core.dom.TextBlock;
+import org.eclipse.jdt.core.dom.ThrowStatement;
 import org.eclipse.jdt.core.dom.TryStatement;
 import org.eclipse.jdt.core.dom.TypeDeclaration;
 import org.eclipse.jdt.core.dom.VariableDeclarationExpression;
 import org.eclipse.jdt.core.dom.VariableDeclarationStatement;
 import org.eclipse.jdt.core.dom.WhileStatement;
+import org.eclipse.jdt.core.dom.YieldStatement;
 import org.eclipse.jdt.core.formatter.DefaultCodeFormatterConstants;
+import org.eclipse.jdt.internal.formatter.DefaultCodeFormatterOptions.Alignment;
+import org.eclipse.jdt.internal.formatter.Token.WrapMode;
+import org.eclipse.jdt.internal.formatter.Token.WrapPolicy;
 
 public class LineBreaksPreparator extends ASTVisitor {
 	final private TokenManager tm;
 	final private DefaultCodeFormatterOptions options;
 
-	private boolean declarationModifierVisited;
+	/**
+	 * Internal synonym for constant AST.JSL14
+	 * to alleviate deprecation warnings once AST.JLS14 is deprecated in future.
+	 * @deprecated
+	 */
+	private static final int AST_INTERNAL_JLS14 = AST.JLS14;
 
 	public LineBreaksPreparator(TokenManager tokenManager, DefaultCodeFormatterOptions options) {
 		this.tm = tokenManager;
@@ -96,10 +108,18 @@ public class LineBreaksPreparator extends ASTVisitor {
 	@Override
 	public boolean visit(CompilationUnit node) {
 		List<ImportDeclaration> imports = node.imports();
-		if (!imports.isEmpty()) {
-			int index = this.tm.firstIndexIn(imports.get(0), -1);
-			if (index > 0)
-				this.tm.get(index).putLineBreaksBefore(this.options.blank_lines_before_imports + 1);
+		if (!imports.isEmpty() && this.tm.firstIndexIn(imports.get(0), -1) > 0)
+			putBlankLinesBefore(imports.get(0), this.options.blank_lines_before_imports);
+
+		for (int i = 1; i < imports.size(); i++) {
+			int from = this.tm.lastIndexIn(imports.get(i - 1), -1);
+			int to = this.tm.firstIndexIn(imports.get(i), -1);
+			for (int j = from; j < to; j++) {
+				Token token1 = this.tm.get(j);
+				Token token2 = this.tm.get(j + 1);
+				if (this.tm.countLineBreaksBetween(token1, token2) > 1)
+					putBlankLinesAfter(token1, this.options.blank_lines_between_import_groups);
+			}
 		}
 
 		List<AnnotationTypeDeclaration> types = node.types();
@@ -114,15 +134,14 @@ public class LineBreaksPreparator extends ASTVisitor {
 
 	@Override
 	public boolean visit(PackageDeclaration node) {
-		int blanks = this.options.blank_lines_before_package;
-		if (blanks > 0) {
-			List<Annotation> annotations = node.annotations();
-			int firstTokenIndex = annotations.isEmpty() ? this.tm.firstIndexBefore(node.getName(), TokenNamepackage)
-					: this.tm.firstIndexIn(annotations.get(0), -1);
-			this.tm.get(firstTokenIndex).putLineBreaksBefore(blanks + 1);
+		if (node.getJavadoc() == null) {
+			putBlankLinesBefore(node, this.options.blank_lines_before_package);
+		} else {
+			putBlankLinesAfter(this.tm.lastTokenIn(node.getJavadoc(), -1), this.options.blank_lines_before_package);
 		}
-		this.tm.lastTokenIn(node, TokenNameSEMICOLON).putLineBreaksAfter(this.options.blank_lines_after_package + 1);
-		this.declarationModifierVisited = false;
+
+		handleAnnotations(node.annotations(), this.options.insert_new_line_after_annotation_on_package);
+		putBlankLinesAfter(this.tm.lastTokenIn(node, -1), this.options.blank_lines_after_package);
 		return true;
 	}
 
@@ -136,41 +155,46 @@ public class LineBreaksPreparator extends ASTVisitor {
 	public boolean visit(TypeDeclaration node) {
 		handleBodyDeclarations(node.bodyDeclarations());
 
-		if (node.getName().getStartPosition() == -1)
-			return true; // this is a fake type created by parsing in class body mode
+		if (this.tm.isFake(node))
+			return true;
 
 		breakLineBefore(node);
-
+		handleAnnotations(node.modifiers(), this.options.insert_new_line_after_annotation_on_type);
 		handleBracedCode(node, node.getName(), this.options.brace_position_for_type_declaration,
 				this.options.indent_body_declarations_compare_to_type_header);
-
-		this.declarationModifierVisited = false;
 		return true;
 	}
 
 	private void handleBodyDeclarations(List<BodyDeclaration> bodyDeclarations) {
 		BodyDeclaration previous = null;
 		for (BodyDeclaration bodyDeclaration : bodyDeclarations) {
+			int blankLines = 0;
 			if (previous == null) {
-				putBlankLinesBefore(bodyDeclaration, this.options.blank_lines_before_first_class_body_declaration);
-			} else {
-				int blankLines = 0;
-				if (bodyDeclaration instanceof FieldDeclaration) {
-					blankLines = this.options.blank_lines_before_field;
-				} else if (bodyDeclaration instanceof AbstractTypeDeclaration) {
-					blankLines = this.options.blank_lines_before_member_type;
-				} else if (bodyDeclaration instanceof MethodDeclaration
-						|| bodyDeclaration instanceof AnnotationTypeMemberDeclaration) {
-					blankLines = this.options.blank_lines_before_method;
-				}
-
-				if (!sameChunk(previous, bodyDeclaration))
-					blankLines = Math.max(blankLines, this.options.blank_lines_before_new_chunk);
-
-				putBlankLinesBefore(bodyDeclaration, blankLines);
+				blankLines = this.options.blank_lines_before_first_class_body_declaration;
+			} else if (!sameChunk(previous, bodyDeclaration)) {
+				blankLines = this.options.blank_lines_before_new_chunk;
+			} else if (bodyDeclaration instanceof FieldDeclaration) {
+				blankLines = this.options.blank_lines_before_field;
+			} else if (bodyDeclaration instanceof AbstractTypeDeclaration) {
+				blankLines = this.options.blank_lines_before_member_type;
+			} else if (bodyDeclaration instanceof MethodDeclaration) {
+				blankLines = ((MethodDeclaration) bodyDeclaration).getBody() == null
+						&& ((MethodDeclaration) previous).getBody() == null
+								? this.options.blank_lines_before_abstract_method
+								: this.options.blank_lines_before_method;
+			} else if (bodyDeclaration instanceof AnnotationTypeMemberDeclaration) {
+				blankLines = this.options.blank_lines_before_method;
 			}
+			putBlankLinesBefore(bodyDeclaration, blankLines);
 			previous = bodyDeclaration;
 		}
+		if (previous != null) {
+			ASTNode parent = previous.getParent();
+			if (!(parent instanceof TypeDeclaration && this.tm.isFake((TypeDeclaration) parent))) {
+				Token lastToken = this.tm.lastTokenIn(parent, -1);
+				putBlankLinesBefore(lastToken, this.options.blank_lines_after_last_class_body_declaration);
+			}
+		}
 	}
 
 	private boolean sameChunk(BodyDeclaration bd1, BodyDeclaration bd2) {
@@ -180,24 +204,26 @@ public class LineBreaksPreparator extends ASTVisitor {
 			return true;
 		if ((bd1 instanceof FieldDeclaration || bd1 instanceof Initializer)
 				&& (bd2 instanceof FieldDeclaration || bd2 instanceof Initializer))
-			return true;
+			return true; // special case: initializers are often related to fields, don't separate
 		return false;
 	}
 
-	private void putBlankLinesBefore(ASTNode node, int linesCount) {
-		int index = this.tm.firstIndexIn(node, -1);
-		while (index > 0 && this.tm.get(index - 1).tokenType == TokenNameCOMMENT_JAVADOC)
-			index--;
-		this.tm.get(index).putLineBreaksBefore(linesCount + 1);
-	}
-
 	@Override
 	public boolean visit(EnumDeclaration node) {
+		handleAnnotations(node.modifiers(), this.options.insert_new_line_after_annotation_on_type);
 		handleBracedCode(node, node.getName(), this.options.brace_position_for_enum_declaration,
 				this.options.indent_body_declarations_compare_to_enum_declaration_header);
-		handleBodyDeclarations(node.bodyDeclarations());
 
+		List<BodyDeclaration> declarations = node.bodyDeclarations();
 		List<EnumConstantDeclaration> enumConstants = node.enumConstants();
+		if (!declarations.isEmpty()) {
+			if (!enumConstants.isEmpty()) {
+				declarations = new ArrayList<>(declarations);
+				declarations.add(0, enumConstants.get(0));
+			}
+			handleBodyDeclarations(declarations);
+		}
+
 		for (int i = 0; i < enumConstants.size(); i++) {
 			EnumConstantDeclaration declaration = enumConstants.get(i);
 			if (declaration.getJavadoc() != null)
@@ -218,21 +244,18 @@ public class LineBreaksPreparator extends ASTVisitor {
 			else
 				break;
 		}
-
-		this.declarationModifierVisited = false;
 		return true;
 	}
 
 	@Override
 	public boolean visit(AnnotationTypeDeclaration node) {
+		handleAnnotations(node.modifiers(), this.options.insert_new_line_after_annotation_on_type);
 		handleBracedCode(node, node.getName(), this.options.brace_position_for_annotation_type_declaration,
 				this.options.indent_body_declarations_compare_to_annotation_declaration_header);
 
 		handleBodyDeclarations(node.bodyDeclarations());
 		if (node.getModifiers() == 0)
 			this.tm.firstTokenBefore(node.getName(), TokenNameAT).breakBefore();
-
-		this.declarationModifierVisited = false;
 		return true;
 	}
 
@@ -249,18 +272,27 @@ public class LineBreaksPreparator extends ASTVisitor {
 		return true;
 	}
 
+	@Override
+	public boolean visit(RecordDeclaration node) {
+		handleAnnotations(node.modifiers(), this.options.insert_new_line_after_annotation_on_type);
+		handleBracedCode(node, node.getName(), this.options.brace_position_for_record_declaration,
+				this.options.indent_body_declarations_compare_to_record_header);
+		handleBodyDeclarations(node.bodyDeclarations());
+		return true;
+	}
+
 	@Override
 	public boolean visit(MethodDeclaration node) {
-		this.declarationModifierVisited = false;
+		handleAnnotations(node.modifiers(), this.options.insert_new_line_after_annotation_on_method);
 		if (node.getBody() == null)
 			return true;
 
-		String bracePosition = node.isConstructor() ? this.options.brace_position_for_constructor_declaration
-				: this.options.brace_position_for_method_declaration;
-		handleBracedCode(node.getBody(), null, bracePosition, this.options.indent_statements_compare_to_body);
-		Token openBrace = this.tm.firstTokenIn(node.getBody(), TokenNameLBRACE);
-		if (openBrace.getLineBreaksAfter() > 0) // if not, these are empty braces
-			openBrace.putLineBreaksAfter(this.options.blank_lines_at_beginning_of_method_body + 1);
+		String bracePosition = node.isCompactConstructor() ? this.options.brace_position_for_record_constructor
+				: node.isConstructor() ? this.options.brace_position_for_constructor_declaration
+						: this.options.brace_position_for_method_declaration;
+		handleBracedCode(node.getBody(), null, bracePosition, this.options.indent_statements_compare_to_body,
+				this.options.blank_lines_at_beginning_of_method_body, this.options.blank_lines_at_end_of_method_body);
+
 		return true;
 	}
 
@@ -287,58 +319,40 @@ public class LineBreaksPreparator extends ASTVisitor {
 		} else if (parent instanceof LambdaExpression) {
 			bracePosition = this.options.brace_position_for_lambda_body;
 		}
-		handleBracedCode(node, null, bracePosition, this.options.indent_statements_compare_to_block);
+		handleBracedCode(node, null, bracePosition, this.options.indent_statements_compare_to_block,
+				this.options.blank_lines_at_beginning_of_code_block, this.options.blank_lines_at_end_of_code_block);
+
+		if (parent instanceof Block) {
+			blankLinesAroundBlock(node, ((Block) parent).statements());
+		} else if (parent instanceof Statement && parent.getParent() instanceof Block) {
+			blankLinesAroundBlock(parent, ((Block) parent.getParent()).statements());
+		}
 
 		return true;
 	}
 
+	private void blankLinesAroundBlock(ASTNode blockStatement, List<ASTNode> siblings) {
+		putBlankLinesBefore(blockStatement, this.options.blank_lines_before_code_block);
+		if (!this.options.put_empty_statement_on_new_line) {
+			int blockIndex = siblings.indexOf(blockStatement);
+			if (blockIndex + 1 < siblings.size() && siblings.get(blockIndex + 1) instanceof EmptyStatement)
+				return;
+		}
+		putBlankLinesAfter(this.tm.lastTokenIn(blockStatement, -1), this.options.blank_lines_after_code_block);
+	}
+
 	@Override
 	public boolean visit(SwitchStatement node) {
 		handleBracedCode(node, node.getExpression(), this.options.brace_position_for_switch,
-				this.options.indent_switchstatements_compare_to_switch);
+				this.options.indent_switchstatements_compare_to_switch,
+				this.options.blank_lines_at_beginning_of_code_block, this.options.blank_lines_at_end_of_code_block);
 
 		List<Statement> statements = node.statements();
-		if (this.options.indent_switchstatements_compare_to_cases) {
-			int nonBreakStatementEnd = -1;
-			for (Statement statement : statements) {
-				boolean isBreaking = statement instanceof BreakStatement || statement instanceof ReturnStatement
-						|| statement instanceof ContinueStatement || statement instanceof Block;
-				if (isBreaking && !(statement instanceof Block))
-					adjustEmptyLineAfter(this.tm.lastIndexIn(statement, -1), -1);
-				if (statement instanceof SwitchCase) {
-					if (nonBreakStatementEnd >= 0) {
-						// indent only comments between previous and current statement
-						this.tm.get(nonBreakStatementEnd + 1).indent();
-						this.tm.firstTokenIn(statement, -1).unindent();
-					}
-				} else if (!(statement instanceof BreakStatement || statement instanceof Block)) {
-					indent(statement);
-				}
-				nonBreakStatementEnd = isBreaking ? -1 : this.tm.lastIndexIn(statement, -1);
-			}
-			if (nonBreakStatementEnd >= 0) {
-				// indent comments between last statement and closing brace 
-				this.tm.get(nonBreakStatementEnd + 1).indent();
-				this.tm.lastTokenIn(node, TokenNameRBRACE).unindent();
-			}
-		}
-		if (this.options.indent_breaks_compare_to_cases) {
-			for (Statement statement : statements) {
-				if (statement instanceof BreakStatement)
-					indent(statement);
-			}
-		}
+		doSwitchStatementsIndentation(node, statements);
+		doSwitchStatementsLineBreaks(statements);
 
-		boolean arrowMode = statements.stream()
-				.anyMatch(s -> s instanceof SwitchCase && ((SwitchCase) s).isSwitchLabeledRule());
-		for (Statement statement : statements) {
-			if (statement instanceof Block)
-				continue; // will add break in visit(Block) if necessary
-			if (arrowMode && !(statement instanceof SwitchCase))
-				continue;
-			if (this.options.put_empty_statement_on_new_line || !(statement instanceof EmptyStatement))
-				breakLineBefore(statement);
-		}
+		if (node.getParent() instanceof Block)
+			blankLinesAroundBlock(node, ((Block) node.getParent()).statements());
 
 		return true;
 	}
@@ -346,14 +360,21 @@ public class LineBreaksPreparator extends ASTVisitor {
 	@Override
 	public boolean visit(SwitchExpression node) {
 		handleBracedCode(node, node.getExpression(), this.options.brace_position_for_switch,
-				this.options.indent_switchstatements_compare_to_switch);
+				this.options.indent_switchstatements_compare_to_switch,
+				this.options.blank_lines_at_beginning_of_code_block, this.options.blank_lines_at_end_of_code_block);
 
 		List<Statement> statements = node.statements();
+		doSwitchStatementsIndentation(node, statements);
+		doSwitchStatementsLineBreaks(statements);
+
+		return true;
+	}
+
+	private void doSwitchStatementsIndentation(ASTNode switchNode, List<Statement> statements) {
 		if (this.options.indent_switchstatements_compare_to_cases) {
 			int nonBreakStatementEnd = -1;
 			for (Statement statement : statements) {
-				boolean isBreaking = statement instanceof BreakStatement || statement instanceof ReturnStatement
-						|| statement instanceof ContinueStatement || statement instanceof Block;
+				boolean isBreaking = isSwitchBreakingStatement(statement);
 				if (isBreaking && !(statement instanceof Block))
 					adjustEmptyLineAfter(this.tm.lastIndexIn(statement, -1), -1);
 				if (statement instanceof SwitchCase) {
@@ -362,36 +383,47 @@ public class LineBreaksPreparator extends ASTVisitor {
 						this.tm.get(nonBreakStatementEnd + 1).indent();
 						this.tm.firstTokenIn(statement, -1).unindent();
 					}
-				} else if (!(statement instanceof BreakStatement || statement instanceof Block)) {
+				} else if (!(statement instanceof BreakStatement || statement instanceof YieldStatement
+						|| statement instanceof Block)) {
 					indent(statement);
 				}
 				nonBreakStatementEnd = isBreaking ? -1 : this.tm.lastIndexIn(statement, -1);
 			}
 			if (nonBreakStatementEnd >= 0) {
-				// indent comments between last statement and closing brace 
+				// indent comments between last statement and closing brace
 				this.tm.get(nonBreakStatementEnd + 1).indent();
-				this.tm.lastTokenIn(node, TokenNameRBRACE).unindent();
+				this.tm.lastTokenIn(switchNode, TokenNameRBRACE).unindent();
 			}
 		}
 		if (this.options.indent_breaks_compare_to_cases) {
 			for (Statement statement : statements) {
-				if (statement instanceof BreakStatement)
+				if (statement instanceof BreakStatement || statement instanceof YieldStatement)
 					indent(statement);
 			}
 		}
+	}
 
+	private void doSwitchStatementsLineBreaks(List<Statement> statements) {
 		boolean arrowMode = statements.stream()
-				.anyMatch(s -> s instanceof SwitchCase && ((SwitchCase) s).isSwitchLabeledRule());
+				.anyMatch(s -> s instanceof SwitchCase && s.getAST().apiLevel() >= AST_INTERNAL_JLS14 &&((SwitchCase) s).isSwitchLabeledRule());
+		Statement previous = null;
 		for (Statement statement : statements) {
-			if (statement instanceof Block)
-				continue; // will add break in visit(Block) if necessary
-			if (arrowMode && !(statement instanceof SwitchCase))
-				continue;
-			if (this.options.put_empty_statement_on_new_line || !(statement instanceof EmptyStatement))
-				breakLineBefore(statement);
+			boolean skip = statement instanceof Block // will add break in visit(Block) if necessary
+					|| (arrowMode && !(statement instanceof SwitchCase))
+					|| (statement instanceof EmptyStatement && !this.options.put_empty_statement_on_new_line);
+			if (!skip) {
+				boolean newGroup = !arrowMode && statement instanceof SwitchCase && isSwitchBreakingStatement(previous);
+				int blankLines = newGroup ? this.options.blank_lines_between_statement_groups_in_switch : 0;
+				putBlankLinesBefore(statement, blankLines);
+			}
+			previous = statement;
 		}
+	}
 
-		return true;
+	private boolean isSwitchBreakingStatement(Statement statement) {
+		return statement instanceof BreakStatement || statement instanceof ReturnStatement
+				|| statement instanceof ContinueStatement || statement instanceof ThrowStatement
+				|| statement instanceof YieldStatement || statement instanceof Block;
 	}
 
 	@Override
@@ -449,99 +481,62 @@ public class LineBreaksPreparator extends ASTVisitor {
 		return true;
 	}
 
-	@Override
-	public boolean visit(NormalAnnotation node) {
-		handleAnnotation(node);
-		return true;
-	}
-
-	@Override
-	public boolean visit(SingleMemberAnnotation node) {
-		handleAnnotation(node);
-		return true;
-	}
-
-	@Override
-	public boolean visit(MarkerAnnotation node) {
-		handleAnnotation(node);
-		return true;
-	}
-
 	@Override
 	public boolean visit(VariableDeclarationStatement node) {
-		this.declarationModifierVisited = false;
+		handleAnnotations(node.modifiers(), this.options.insert_new_line_after_annotation_on_local_variable);
 		return true;
 	}
 
 	@Override
 	public boolean visit(SingleVariableDeclaration node) {
-		this.declarationModifierVisited = false;
-		if (node.getParent() instanceof MethodDeclaration) {
-			// special case: annotations on parameters without modifiers should not be treated as type annotations
-			this.declarationModifierVisited = (node.getModifiers() == 0);
-		}
+		handleAnnotations(node.modifiers(),
+				node.getParent() instanceof EnhancedForStatement
+						? this.options.insert_new_line_after_annotation_on_local_variable
+						: this.options.insert_new_line_after_annotation_on_parameter);
 		return true;
 	}
 
 	@Override
 	public boolean visit(VariableDeclarationExpression node) {
-		this.declarationModifierVisited = false;
+		handleAnnotations(node.modifiers(), this.options.insert_new_line_after_annotation_on_local_variable);
 		return true;
 	}
 
 	@Override
 	public boolean visit(FieldDeclaration node) {
-		this.declarationModifierVisited = false;
+		handleAnnotations(node.modifiers(), this.options.insert_new_line_after_annotation_on_field);
 		return true;
 	}
 
 	@Override
 	public boolean visit(AnnotationTypeMemberDeclaration node) {
-		this.declarationModifierVisited = false;
+		handleAnnotations(node.modifiers(), this.options.insert_new_line_after_annotation_on_method);
 		return true;
 	}
 
 	@Override
 	public boolean visit(EnumConstantDeclaration node) {
-		this.declarationModifierVisited = false;
+		handleAnnotations(node.modifiers(), this.options.insert_new_line_after_annotation_on_enum_constant);
 		return true;
 	}
 
-	@Override
-	public boolean visit(Modifier node) {
-		this.declarationModifierVisited = true;
-		return true;
-	}
+	private void handleAnnotations(List<? extends IExtendedModifier> modifiers, boolean breakAfter) {
+		Annotation last = null;
+		int i;
+		for (i = 0; i < modifiers.size(); i++) {
+			if (modifiers.get(i).isModifier())
+				break;
+			last = (Annotation) modifiers.get(i);
+		}
+		if (last != null && breakAfter) {
+			this.tm.lastTokenIn(last, -1).breakAfter();
+		}
 
-	private void handleAnnotation(Annotation node) {
-		ASTNode parentNode = node.getParent();
-		boolean breakAfter = false;
-		boolean isTypeAnnotation = this.declarationModifierVisited;
-		if (isTypeAnnotation) {
-			breakAfter = this.options.insert_new_line_after_type_annotation;
-		} else if (parentNode instanceof PackageDeclaration) {
-			breakAfter = this.options.insert_new_line_after_annotation_on_package;
-		} else if (parentNode instanceof AbstractTypeDeclaration) {
-			breakAfter = this.options.insert_new_line_after_annotation_on_type;
-		} else if (parentNode instanceof EnumConstantDeclaration) {
-			breakAfter = this.options.insert_new_line_after_annotation_on_enum_constant;
-		} else if (parentNode instanceof FieldDeclaration) {
-			breakAfter = this.options.insert_new_line_after_annotation_on_field;
-		} else if (parentNode instanceof MethodDeclaration) {
-			breakAfter = this.options.insert_new_line_after_annotation_on_method;
-		} else if (parentNode instanceof AnnotationTypeMemberDeclaration) {
-			breakAfter = this.options.insert_new_line_after_annotation_on_method
-					&& ((AnnotationTypeMemberDeclaration) parentNode).getDefault() != node;
-		} else if (parentNode instanceof VariableDeclarationStatement
-				|| parentNode instanceof VariableDeclarationExpression) {
-			breakAfter = this.options.insert_new_line_after_annotation_on_local_variable;
-		} else if (parentNode instanceof SingleVariableDeclaration) {
-			breakAfter = this.options.insert_new_line_after_annotation_on_parameter;
-			if ((parentNode.getParent()) instanceof EnhancedForStatement)
-				breakAfter = this.options.insert_new_line_after_annotation_on_local_variable;
+		if (i < modifiers.size()) {
+			// any annotations following other modifiers will be associated with declaration type
+			handleAnnotations(modifiers.subList(i + 1, modifiers.size()),
+					this.options.insert_new_line_after_type_annotation);
 		}
-		if (breakAfter)
-			this.tm.lastTokenIn(node, -1).breakAfter();
 	}
 
 	@Override
@@ -617,20 +612,71 @@ public class LineBreaksPreparator extends ASTVisitor {
 	public boolean visit(ModuleDeclaration node) {
 		// using settings for type declaration and fields for now, add new settings if necessary
 		breakLineBefore(node);
-		handleBracedCode(node, node.getName(), this.options.brace_position_for_type_declaration,
-				this.options.indent_body_declarations_compare_to_type_header);
-
 		List<ModuleDirective> statements = node.moduleStatements();
+		handleBracedCode(node, node.getName(), this.options.brace_position_for_type_declaration,
+				this.options.indent_body_declarations_compare_to_type_header,
+				statements.isEmpty() ? 0 : this.options.blank_lines_before_first_class_body_declaration,
+				statements.isEmpty() ? 0 : this.options.blank_lines_after_last_class_body_declaration);
+
 		ModuleDirective previous = null;
 		for (ModuleDirective statement : statements) {
-			int blankLines = previous == null ? this.options.blank_lines_before_first_class_body_declaration
-					: previous.getClass().equals(statement.getClass()) ? this.options.blank_lines_before_field
-							: this.options.blank_lines_before_new_chunk;
-			putBlankLinesBefore(statement, blankLines);
+			if (previous != null) {
+				boolean cameChunk = previous.getClass().equals(statement.getClass());
+				putBlankLinesBefore(statement,
+						cameChunk ? this.options.blank_lines_before_field : this.options.blank_lines_before_new_chunk);
+			}
 			previous = statement;
 		}
+		return true;
+	}
 
-		this.declarationModifierVisited = false;
+	@Override
+	public boolean visit(TextBlock node) {
+		int indentOption = this.options.text_block_indentation;
+		if (indentOption == Alignment.M_INDENT_PRESERVE)
+			return true;
+		Token block = this.tm.firstTokenIn(node, TokenNameTextBlock);
+		ArrayList<Token> lines = new ArrayList<>();
+		lines.add(new Token(block.originalStart, block.originalStart + 2, 0)); // first line; """
+		int incidentalWhitespace = Integer.MAX_VALUE;
+		int blankLines = -1; // will go to 0 on line break after first line
+		int i = block.originalStart + 3;
+		while (i <= block.originalEnd) {
+			int lineStart = i;
+			int firstNonBlank = -1;
+			int lastNonBlank = -1;
+			while (i <= block.originalEnd) {
+				char c = this.tm.charAt(i++);
+				if (c == '\r' || c == '\n') {
+					char c2 = this.tm.charAt(i);
+					if ((c2 == '\r' || c2 == '\n') && c2 != c)
+						i++;
+					break;
+				}
+				if (c != ' ' && c != '\t') {
+					if (firstNonBlank == -1)
+						firstNonBlank = i - 1;
+					lastNonBlank = i - 1;
+				}
+			}
+			if (firstNonBlank != -1) {
+				Token line = new Token(lineStart, lastNonBlank, 0);
+				line.putLineBreaksBefore(blankLines + 1);
+				blankLines = 0;
+				lines.add(line);
+				incidentalWhitespace = Math.min(incidentalWhitespace, firstNonBlank - lineStart);
+			} else {
+				blankLines++;
+			}
+		}
+		WrapPolicy wrapPolicy = new WrapPolicy(WrapMode.DISABLED, 0, -1, 0, 0, 1, false, false);
+		for (i = 1; i < lines.size(); i++) {
+			Token t = lines.get(i);
+			Token line = new Token(t, t.originalStart + incidentalWhitespace, t.originalEnd, TokenNameTextBlock);
+			line.setWrapPolicy(wrapPolicy);
+			lines.set(i, line);
+		}
+		block.setInternalStructure(lines);
 		return true;
 	}
 
@@ -638,7 +684,37 @@ public class LineBreaksPreparator extends ASTVisitor {
 		this.tm.firstTokenIn(node, -1).breakBefore();
 	}
 
+	private void putBlankLinesBefore(ASTNode node, int linesCount) {
+		int index = this.tm.firstIndexIn(node, -1);
+		while (index > 0 && this.tm.get(index - 1).tokenType == TokenNameCOMMENT_JAVADOC)
+			index--;
+		putBlankLinesBefore(this.tm.get(index), linesCount);
+	}
+
+	private void putBlankLinesBefore(Token token, int linesCount) {
+		if (linesCount >= 0) {
+			token.putLineBreaksBefore(linesCount + 1);
+		} else {
+			token.putLineBreaksBefore(~linesCount + 1);
+			token.setPreserveLineBreaksBefore(false);
+		}
+	}
+
+	private void putBlankLinesAfter(Token token, int linesCount) {
+		if (linesCount >= 0) {
+			token.putLineBreaksAfter(linesCount + 1);
+		} else {
+			token.putLineBreaksAfter(~linesCount + 1);
+			token.setPreserveLineBreaksAfter(false);
+		}
+	}
+
 	private void handleBracedCode(ASTNode node, ASTNode nodeBeforeOpenBrace, String bracePosition, boolean indentBody) {
+		handleBracedCode(node, nodeBeforeOpenBrace, bracePosition, indentBody, 0, 0);
+	}
+
+	private void handleBracedCode(ASTNode node, ASTNode nodeBeforeOpenBrace, String bracePosition, boolean indentBody,
+			int blankLinesAfterOpeningBrace, int blankLinesBeforeClosingBrace) {
 		int openBraceIndex = nodeBeforeOpenBrace == null
 				? this.tm.firstIndexIn(node, TokenNameLBRACE)
 				: this.tm.firstIndexAfter(nodeBeforeOpenBrace, TokenNameLBRACE);
@@ -647,8 +723,8 @@ public class LineBreaksPreparator extends ASTVisitor {
 		Token closeBraceToken = this.tm.get(closeBraceIndex);
 		handleBracePosition(openBraceToken, closeBraceIndex, bracePosition);
 
-		openBraceToken.breakAfter();
-		closeBraceToken.breakBefore();
+		putBlankLinesAfter(openBraceToken, blankLinesAfterOpeningBrace);
+		putBlankLinesBefore(closeBraceToken, blankLinesBeforeClosingBrace);
 
 		if (indentBody) {
 			adjustEmptyLineAfter(openBraceIndex, 1);
