@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2014, 2019 Mateusz Matela and others.
+ * Copyright (c) 2014, 2020 Mateusz Matela and others.
  *
  * This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License 2.0
@@ -53,14 +53,16 @@ import java.util.function.ToIntFunction;
 
 import org.eclipse.jdt.core.dom.ASTNode;
 import org.eclipse.jdt.core.dom.ASTVisitor;
+import org.eclipse.jdt.core.dom.Annotation;
 import org.eclipse.jdt.core.dom.AnnotationTypeDeclaration;
+import org.eclipse.jdt.core.dom.AnnotationTypeMemberDeclaration;
 import org.eclipse.jdt.core.dom.AnonymousClassDeclaration;
 import org.eclipse.jdt.core.dom.ArrayInitializer;
+import org.eclipse.jdt.core.dom.AssertStatement;
 import org.eclipse.jdt.core.dom.Assignment;
 import org.eclipse.jdt.core.dom.Block;
 import org.eclipse.jdt.core.dom.CatchClause;
 import org.eclipse.jdt.core.dom.ClassInstanceCreation;
-import org.eclipse.jdt.core.dom.CompilationUnit;
 import org.eclipse.jdt.core.dom.ConditionalExpression;
 import org.eclipse.jdt.core.dom.ConstructorInvocation;
 import org.eclipse.jdt.core.dom.CreationReference;
@@ -74,8 +76,8 @@ import org.eclipse.jdt.core.dom.ExpressionMethodReference;
 import org.eclipse.jdt.core.dom.FieldAccess;
 import org.eclipse.jdt.core.dom.FieldDeclaration;
 import org.eclipse.jdt.core.dom.ForStatement;
+import org.eclipse.jdt.core.dom.IExtendedModifier;
 import org.eclipse.jdt.core.dom.IfStatement;
-import org.eclipse.jdt.core.dom.ImportDeclaration;
 import org.eclipse.jdt.core.dom.InfixExpression;
 import org.eclipse.jdt.core.dom.InfixExpression.Operator;
 import org.eclipse.jdt.core.dom.LambdaExpression;
@@ -84,9 +86,11 @@ import org.eclipse.jdt.core.dom.MethodInvocation;
 import org.eclipse.jdt.core.dom.Name;
 import org.eclipse.jdt.core.dom.NormalAnnotation;
 import org.eclipse.jdt.core.dom.OpensDirective;
+import org.eclipse.jdt.core.dom.PackageDeclaration;
 import org.eclipse.jdt.core.dom.ParameterizedType;
 import org.eclipse.jdt.core.dom.ProvidesDirective;
 import org.eclipse.jdt.core.dom.QualifiedName;
+import org.eclipse.jdt.core.dom.RecordDeclaration;
 import org.eclipse.jdt.core.dom.SingleMemberAnnotation;
 import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
 import org.eclipse.jdt.core.dom.Statement;
@@ -104,6 +108,7 @@ import org.eclipse.jdt.core.dom.TypeMethodReference;
 import org.eclipse.jdt.core.dom.TypeParameter;
 import org.eclipse.jdt.core.dom.UnionType;
 import org.eclipse.jdt.core.dom.VariableDeclaration;
+import org.eclipse.jdt.core.dom.VariableDeclarationExpression;
 import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
 import org.eclipse.jdt.core.dom.VariableDeclarationStatement;
 import org.eclipse.jdt.core.dom.WhileStatement;
@@ -225,8 +230,6 @@ public class WrapPreparator extends ASTVisitor {
 
 	final Aligner aligner;
 
-	int importsStart = -1, importsEnd = -1;
-
 	/*
 	 * temporary values used when calling {@link #handleWrap(int)} to avoid ArrayList initialization and long lists of
 	 * parameters
@@ -268,12 +271,8 @@ public class WrapPreparator extends ASTVisitor {
 	}
 
 	@Override
-	public boolean visit(CompilationUnit node) {
-		List<ImportDeclaration> imports = node.imports();
-		if (!imports.isEmpty()) {
-			this.importsStart = this.tm.firstIndexIn(imports.get(0), -1);
-			this.importsEnd = this.tm.lastIndexIn(imports.get(imports.size() - 1), -1);
-		}
+	public boolean visit(PackageDeclaration node) {
+		handleAnnotations(node.annotations(), this.options.alignment_for_annotations_on_package);
 		return true;
 	}
 
@@ -297,6 +296,8 @@ public class WrapPreparator extends ASTVisitor {
 
 	@Override
 	public boolean visit(TypeDeclaration node) {
+		handleAnnotations(node.modifiers(), this.options.alignment_for_annotations_on_type);
+
 		Type superclassType = node.getSuperclassType();
 		if (superclassType != null) {
 			this.wrapParentIndex = this.tm.lastIndexIn(node.getName(), -1);
@@ -325,10 +326,17 @@ public class WrapPreparator extends ASTVisitor {
 
 	@Override
 	public boolean visit(AnnotationTypeDeclaration node) {
+		handleAnnotations(node.modifiers(), this.options.alignment_for_annotations_on_type);
 		this.aligner.handleAlign(node.bodyDeclarations());
 		return true;
 	}
 
+	@Override
+	public boolean visit(AnnotationTypeMemberDeclaration node) {
+		handleAnnotations(node.modifiers(), this.options.alignment_for_annotations_on_method);
+		return true;
+	}
+
 	@Override
 	public boolean visit(AnonymousClassDeclaration node) {
 		this.aligner.handleAlign(node.bodyDeclarations());
@@ -336,11 +344,41 @@ public class WrapPreparator extends ASTVisitor {
 	}
 
 	@Override
-	public boolean visit(MethodDeclaration node) {
+	public boolean visit(RecordDeclaration node) {
+		handleAnnotations(node.modifiers(), this.options.alignment_for_annotations_on_type);
+
 		int lParen = this.tm.firstIndexAfter(node.getName(), TokenNameLPAREN);
-		int rParen = node.getBody() == null ? this.tm.lastIndexIn(node, TokenNameRPAREN)
-				: this.tm.firstIndexBefore(node.getBody(), TokenNameRPAREN);
-		handleParenthesesPositions(lParen, rParen, this.options.parenthesis_positions_in_method_declaration);
+		List<SingleVariableDeclaration> components = node.recordComponents();
+		int rParen = this.tm.firstIndexAfter(
+				components.isEmpty() ? node.getName() : components.get(components.size() - 1), TokenNameRPAREN);
+		handleParenthesesPositions(lParen, rParen, this.options.parenthesis_positions_in_record_declaration);
+
+		if (!components.isEmpty()) {
+			int wrappingOption = this.options.alignment_for_record_components;
+			this.wrapGroupEnd = this.tm.lastIndexIn(components.get(components.size() - 1), -1);
+			handleArguments(components, wrappingOption);
+		}
+
+		List<Type> superInterfaceTypes = node.superInterfaceTypes();
+		if (!superInterfaceTypes.isEmpty()) {
+			this.wrapParentIndex = this.tm.lastIndexIn(node.getName(), -1);
+			this.wrapIndexes.add(this.tm.firstIndexBefore(superInterfaceTypes.get(0), TokenNameimplements));
+			prepareElementsList(superInterfaceTypes, TokenNameCOMMA, -1);
+			handleWrap(this.options.alignment_for_superinterfaces_in_record_declaration, PREFERRED);
+		}
+		return true;
+	}
+
+	@Override
+	public boolean visit(MethodDeclaration node) {
+		handleAnnotations(node.modifiers(), this.options.alignment_for_annotations_on_method);
+
+		if (!node.isCompactConstructor()) {
+			int lParen = this.tm.firstIndexAfter(node.getName(), TokenNameLPAREN);
+			int rParen = node.getBody() == null ? this.tm.lastIndexIn(node, TokenNameRPAREN)
+					: this.tm.firstIndexBefore(node.getBody(), TokenNameRPAREN);
+			handleParenthesesPositions(lParen, rParen, this.options.parenthesis_positions_in_method_declaration);
+		}
 
 		List<SingleVariableDeclaration> parameters = node.parameters();
 		Type receiverType = node.getReceiverType();
@@ -360,7 +398,7 @@ public class WrapPreparator extends ASTVisitor {
 					? this.options.alignment_for_throws_clause_in_constructor_declaration
 					: this.options.alignment_for_throws_clause_in_method_declaration;
 			if ((wrappingOption & Alignment.M_INDENT_ON_COLUMN) == 0)
-				this.wrapParentIndex = lParen;
+				this.wrapParentIndex = this.tm.firstIndexAfter(node.getName(), TokenNameLPAREN);
 			prepareElementsList(exceptionTypes, TokenNameCOMMA, TokenNameRPAREN);
 			// instead of the first exception type, wrap the "throws" token
 			this.wrapIndexes.set(0, this.tm.firstIndexBefore(exceptionTypes.get(0), TokenNamethrows));
@@ -392,6 +430,8 @@ public class WrapPreparator extends ASTVisitor {
 
 	@Override
 	public boolean visit(EnumDeclaration node) {
+		handleAnnotations(node.modifiers(), this.options.alignment_for_annotations_on_type);
+
 		List<EnumConstantDeclaration> enumConstants = node.enumConstants();
 		int constantsEnd = -1;
 		if (!enumConstants.isEmpty()) {
@@ -439,6 +479,8 @@ public class WrapPreparator extends ASTVisitor {
 
 	@Override
 	public boolean visit(EnumConstantDeclaration node) {
+		handleAnnotations(node.modifiers(), this.options.alignment_for_annotations_on_enum_constant);
+
 		int lParen = this.tm.firstIndexAfter(node.getName(), -1);
 		while (this.tm.get(lParen).isComment())
 			lParen++;
@@ -967,16 +1009,34 @@ public class WrapPreparator extends ASTVisitor {
 
 	@Override
 	public boolean visit(FieldDeclaration node) {
+		handleAnnotations(node.modifiers(), this.options.alignment_for_annotations_on_field);
 		handleVariableDeclarations(node.fragments());
 		return true;
 	}
 
 	@Override
 	public boolean visit(VariableDeclarationStatement node) {
+		handleAnnotations(node.modifiers(), this.options.alignment_for_annotations_on_local_variable);
 		handleVariableDeclarations(node.fragments());
 		return true;
 	}
 
+	@Override
+	public boolean visit(VariableDeclarationExpression node) {
+		handleAnnotations(node.modifiers(), this.options.alignment_for_annotations_on_local_variable);
+		handleVariableDeclarations(node.fragments());
+		return true;
+	}
+
+	@Override
+	public boolean visit(SingleVariableDeclaration node) {
+		handleAnnotations(node.modifiers(),
+				node.getParent() instanceof EnhancedForStatement
+						? this.options.alignment_for_annotations_on_local_variable
+						: this.options.alignment_for_annotations_on_parameter);
+		return true;
+	}
+
 	@Override
 	public boolean visit(ParameterizedType node) {
 		prepareElementsList(node.typeArguments(), TokenNameCOMMA, TokenNameLESS);
@@ -1075,6 +1135,26 @@ public class WrapPreparator extends ASTVisitor {
 		return true;
 	}
 
+	@Override
+	public boolean visit(AssertStatement node) {
+		Expression message = node.getMessage();
+		if (message != null) {
+			int atColon = this.tm.firstIndexBefore(message, TokenNameCOLON);
+			int afterColon = this.tm.firstIndexIn(message, -1);
+			if (this.options.wrap_before_assertion_message_operator) {
+				this.wrapIndexes.add(atColon);
+				this.secondaryWrapIndexes.add(afterColon);
+			} else {
+				this.wrapIndexes.add(afterColon);
+				this.secondaryWrapIndexes.add(atColon);
+			}
+			this.wrapParentIndex = this.tm.firstIndexIn(node,  -1);
+			this.wrapGroupEnd = this.tm.lastIndexIn(node, -1);
+			handleWrap(this.options.alignment_for_assertion_message);
+		}
+		return true;
+	}
+
 	/**
 	 * Makes sure all new lines within given node will have wrap policy so that
 	 * wrap executor will fix their indentation if necessary.
@@ -1121,6 +1201,29 @@ public class WrapPreparator extends ASTVisitor {
 		handleWrap(wrappingOption);
 	}
 
+	private void handleAnnotations(List<? extends IExtendedModifier> modifiers, int wrappingOption) {
+		Annotation last = null;
+		int i;
+		for (i = 0; i < modifiers.size(); i++) {
+			if (modifiers.get(i).isModifier())
+				break;
+			Annotation annotation = (Annotation) modifiers.get(i);
+			if (i == 0) {
+				this.wrapParentIndex = this.tm.firstIndexIn(annotation, -1);
+			} else {
+				this.wrapIndexes.add(this.tm.firstIndexIn(annotation, -1));
+				this.wrapGroupEnd = this.tm.lastIndexIn(annotation, -1);
+			}
+			last = annotation;
+		}
+		handleWrap(wrappingOption, last);
+
+		if (i < modifiers.size()) {
+			// any annotations following other modifiers will be associated with declaration type
+			handleAnnotations(modifiers.subList(i + 1, modifiers.size()), this.options.alignment_for_type_annotations);
+		}
+	}
+
 	private void prepareElementsList(List<? extends ASTNode> elements, int separatorType, int wrapParentType) {
 		for (int i = 0; i < elements.size(); i++) {
 			ASTNode element = elements.get(i);
@@ -1238,6 +1341,8 @@ public class WrapPreparator extends ASTVisitor {
 		boolean isAlreadyWrapped = false;
 		if (indentOnColumn) {
 			extraIndent = 0;
+		} else if (parentNode instanceof Annotation) {
+			extraIndent = 0;
 		} else if (parentNode instanceof EnumDeclaration) {
 			// special behavior for compatibility with legacy formatter
 			extraIndent = ((wrappingOption & Alignment.M_INDENT_BY_ONE) != 0) ? 2 : 1;
@@ -1289,7 +1394,7 @@ public class WrapPreparator extends ASTVisitor {
 			wrapMode = WrapMode.FORCE;
 		} else if (isAlreadyWrapped) {
 			wrapMode = WrapMode.DISABLED; // to avoid triggering top priority wrapping
-		} else if (isTopPriority) { 
+		} else if (isTopPriority) {
 			wrapMode = WrapMode.TOP_PRIORITY;
 		}
 		extraIndent *= this.options.indentation_size;
@@ -1300,7 +1405,7 @@ public class WrapPreparator extends ASTVisitor {
 	public void finishUp(ASTNode astRoot, List<IRegion> regions) {
 		preserveExistingLineBreaks();
 		applyBreaksOutsideRegions(regions);
-		new WrapExecutor(this.tm, this.options).executeWraps();
+		new WrapExecutor(this.tm, this.options, regions).executeWraps();
 		this.aligner.alignComments();
 		wrapComments();
 		fixEnumConstantIndents(astRoot);
@@ -1318,9 +1423,7 @@ public class WrapPreparator extends ASTVisitor {
 
 			@Override
 			protected boolean token(Token token, int index) {
-				boolean isBetweenImports = index > WrapPreparator.this.importsStart
-						&& index < WrapPreparator.this.importsEnd;
-				int lineBreaks = getLineBreaksToPreserve(getPrevious(), token, isBetweenImports);
+				int lineBreaks = getLineBreaksToPreserve(getPrevious(), token);
 				if (lineBreaks > 1 || (!this.join_wrapped_lines && token.isWrappable()) || index == 0)
 					token.putLineBreaksBefore(lineBreaks);
 				return true;
@@ -1330,7 +1433,7 @@ public class WrapPreparator extends ASTVisitor {
 
 		Token last = this.tm.get(this.tm.size() - 1);
 		last.clearLineBreaksAfter();
-		int endingBreaks = getLineBreaksToPreserve(last, null, false);
+		int endingBreaks = getLineBreaksToPreserve(last, null);
 		if (endingBreaks > 0) {
 			last.putLineBreaksAfter(endingBreaks);
 		} else if ((this.kind & (CodeFormatter.K_COMPILATION_UNIT | CodeFormatter.K_MODULE_INFO)) != 0
@@ -1339,7 +1442,11 @@ public class WrapPreparator extends ASTVisitor {
 		}
 	}
 
-	int getLineBreaksToPreserve(Token token1, Token token2, boolean isBetweenImports) {
+	int getLineBreaksToPreserve(Token token1, Token token2) {
+		if ((token1 != null && !token1.isPreserveLineBreaksAfter())
+				|| (token2 != null && !token2.isPreserveLineBreaksBefore())) {
+			return 0;
+		}
 		if (token1 != null) {
 			List<Token> structure = token1.getInternalStructure();
 			if (structure != null && !structure.isEmpty())
@@ -1351,9 +1458,6 @@ public class WrapPreparator extends ASTVisitor {
 				token2 = structure.get(0);
 		}
 		int lineBreaks = WrapPreparator.this.tm.countLineBreaksBetween(token1, token2);
-		if (isBetweenImports)
-			return lineBreaks > 1 ? (this.options.blank_lines_between_import_groups + 1) : 0;
-
 		int toPreserve = this.options.number_of_empty_lines_to_preserve;
 		if (token1 != null && token2 != null)
 			toPreserve++; // n empty lines = n+1 line breaks, except for file start and end
@@ -1396,7 +1500,7 @@ public class WrapPreparator extends ASTVisitor {
 				isNLSTagInLine = true;
 			}
 			List<Token> structure = token.getInternalStructure();
-			if (structure != null && !structure.isEmpty() && !isNLSTagInLine) {
+			if (token.isComment() && structure != null && !structure.isEmpty() && !isNLSTagInLine) {
 				int startPosition = this.tm.getPositionInLine(i);
 				if (token.tokenType == TokenNameCOMMENT_LINE) {
 					commentWrapper.wrapLineComment(token, startPosition);
@@ -1432,9 +1536,9 @@ public class WrapPreparator extends ASTVisitor {
 				if (isEmpty)
 					break;
 				this.tm.get(openingParenIndex + 1).setWrapPolicy(new WrapPolicy(WrapMode.TOP_PRIORITY,
-						openingParenIndex, closingParenIndex, this.options.indentation_size, 1, 1, true, false));
+						openingParenIndex, closingParenIndex, this.options.indentation_size, this.currentDepth, 1, true, false));
 				this.tm.get(closingParenIndex).setWrapPolicy(new WrapPolicy(WrapMode.TOP_PRIORITY,
-						openingParenIndex, closingParenIndex, 0, 1, 1, false, false));
+						openingParenIndex, closingParenIndex, 0, this.currentDepth, 1, false, false));
 				break;
 			case DefaultCodeFormatterConstants.SEPARATE_LINES_IF_NOT_EMPTY:
 				if (isEmpty)
